!SIMONCHECKTHIS 
!L820: carbon_tau definition, initialisation, usage
!
!
!
! MODULE       : stomate_soilcarbon
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCfILL.LIC
!
!>\BRIEF       Calculate soil dynamics largely following the Century model
!
!!	
!!\n DESCRIPTION: 11 layers discretization: Layers 1-11 are soil layers
!! SIMON removes the necessity for 'new diffusion scheme' in cryotubate routine -it is necessarily used for the DOC component
!! RECENT CHANGE(S): None
!!
!! SVN		:
!! $HeadURL$ 
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================
!
MODULE stomate_soilcarbon
!
  ! modules used:
!!SIMON Added from ORCHIDOC***
  USE xios_orchidee
!!***
!!
  USE ioipsl_para
  USE stomate_data
  USE constantes
!SIMON ADDED to accommodate fbact  
  USE stomate_permafrost_soilcarbon
!!  
!!SIMON Added from ORCHIDOC***
  USE constantes_soil
  USE pft_parameters
  USE pft_parameters_var
  USE vertical_soil
!!*** 
  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC soilcarbon,soilcarbon_clear,altcalc_DOC

  ! Variables shared by all subroutines in this module
  REAL(r_std), DIMENSION(:,:), ALLOCATABLE, SAVE        :: alt
 REAL(r_std), DIMENSION(:,:), ALLOCATABLE, SAVE        :: altmax_lastyear            !! active layer thickness    
   INTEGER(i_std), DIMENSION(:,:),ALLOCATABLE, SAVE      :: altmax_ind_lastyear
     INTEGER(i_std), DIMENSION(:,:), ALLOCATABLE, SAVE     :: alt_ind !! active layer thickness  
   INTEGER(i_std), DIMENSION(:,:),ALLOCATABLE, SAVE      :: altmax_ind !! Maximum over the year active layer thickness
   REAL(r_std), DIMENSION(:,:),ALLOCATABLE, SAVE         :: z_root !! Rooting depth
   INTEGER(i_std), DIMENSION(:,:),ALLOCATABLE, SAVE      :: rootlev !! The deepest model level within the rooting depth
   LOGICAL,DIMENSION(:,:),ALLOCATABLE,  SAVE             :: veget_mask_2d      !! whether there is vegetation 
   REAL(r_std),DIMENSION(:,:),ALLOCATABLE, SAVE          :: lalo_global        !! Geogr. coordinates (latitude,longitude) (degrees)
   REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) 	:: heights_snow      !! total thickness of snow levels (m) 
  !$OMP THREADPRIVATE(heights_snow)
  REAL(r_std), DIMENSION(:,:,:), ALLOCATABLE, SAVE  :: carbon_32l_pftmean        !! Deep soil carbon profiles, mean over all PFTs
  REAL(r_std), DIMENSION(:,:,:), ALLOCATABLE, SAVE  :: carbon_32l_conct        !! Deep soil carbon profiles, mean over all PFTs, in concentration

  !$OMP THREADPRIVATE(carbon_32l_pftmean)
!!!  
!!!
  LOGICAL, SAVE    :: firstcall_soilcarbon = .TRUE.   !! Is this the first call? (true/false)
!$OMP THREADPRIVATE(firstcall_soilcarbon)
    REAL(r_std), SAVE, DIMENSION(ncarb)      :: carbon_tau       !! Residence time in carbon pools (days)
!$OMP THREADPRIVATE(carbon_tau)
    REAL(r_std), SAVE, DIMENSION(ncarb)      :: priming_param    !! parmater controlling the sensitvity of priming effect on  each pools to the amount of LOM
!$OMP THREADPRIVATE(priming_param)
    REAL(r_std), DIMENSION(:,:,:,:), ALLOCATABLE, SAVE      :: DOC_tau          !! Residence time of DOC (days) (kjpindex, npool, ndeep, nvm)
!$OMP THREADPRIVATE(DOC_tau)
!!!SIMON added the factors below for carbon_32l and DOC cryoturbation calculations in subroutine cryoturbate_DOC.
 REAL(r_std), DIMENSION(:,:,:,:), ALLOCATABLE, SAVE  :: alpha_C
 !$OMP THREADPRIVATE(alpha_C)
 
 REAL(r_std), DIMENSION(:,:,:,:), ALLOCATABLE, SAVE  :: beta_C
 !$OMP THREADPRIVATE(beta_C)
REAL(r_std), DIMENSION(:,:,:,:,:), ALLOCATABLE, SAVE :: alpha_DOC
 !$OMP THREADPRIVATE(alpha_DOC)
REAL(r_std), DIMENSION(:,:,:,:,:), ALLOCATABLE, SAVE  :: beta_DOC
 !$OMP THREADPRIVATE(beta_DOC)
REAL(r_std), DIMENSION(:,:,:,:), ALLOCATABLE, SAVE  :: alpha_litter_below
 !$OMP THREADPRIVATE(alpha_litter_below)
REAL(r_std), DIMENSION(:,:,:,:), ALLOCATABLE, SAVE :: beta_litter_below
 !$OMP THREADPRIVATE(beta_litter_below)

  REAL(r_std), DIMENSION(:,:), ALLOCATABLE, SAVE   :: xe_ALL
  !$OMP THREADPRIVATE(xe_ALL)
  
  INTEGER(i_std), SAVE                              :: yr_len = 360
  !!SIMON MADE zf_soil_B a new, allocatable variable here to avoid recursive looping of zf_soil through all input arg lists
  !!!zf_soil_B is otherwise exactly the same as zf_soil 
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)         :: zf_soil_B        !! depths of full levels (m)
  REAL(r_std), DIMENSION(:,:,:), ALLOCATABLE, SAVE  :: xc_cryoturb
 !$OMP THREADPRIVATE(xc_cryoturb)
  REAL(r_std), DIMENSION(:,:,:), ALLOCATABLE, SAVE  :: xd_cryoturb
  !$OMP THREADPRIVATE(xd_cryoturb)
  REAL(r_std), DIMENSION(:,:,:), ALLOCATABLE, SAVE  :: diff_k        !! Diffusion constant (m^2/s)
  LOGICAL, DIMENSION(:,:), ALLOCATABLE, SAVE        :: cryoturb_location
  LOGICAL, DIMENSION(:,:), ALLOCATABLE, SAVE        :: bioturb_location
  REAL(r_std), SAVE                                    :: mu_soil
 REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)         :: zi_soil        !! depths of intermediate levels (m)
  !$OMP THREADPRIVATE(zi_soil)
 REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:,:)     	:: zf_snow           !! depths of full levels (m) 
!$OMP THREADPRIVATE(zf_snow) 
 REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:,:)     	:: zi_snow           !! depths of full levels (m) 
!$OMP THREADPRIVATE(zi_snow)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)     	:: zf_snow_nopftdim  !! depths of full levels (m)
!  !$OMP THREADPRIVATE(zf_snow_nopftdim)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)     	:: zi_snow_nopftdim  !! depths of intermediate levels (m)
!  !$OMP THREADPRIVATE(zi_snow_nopftdim)
!
!
!
CONTAINS
!! ================================================================================================================================
!!  SUBROUTINE   : soilcarbon_clear
!!
!>\BRIEF        Set the flag ::firstcall_soilcarbon to .TRUE. and as such activate sections 1.1.2 and 1.2 of the subroutine soilcarbon 
!! (see below).
!! 
!_ ================================================================================================================================

  SUBROUTINE soilcarbon_clear
    firstcall_soilcarbon=.TRUE.
  ENDSUBROUTINE soilcarbon_clear


!! ================================================================================================================================
!!  SUBROUTINE   : soilcarbon
!!
!>\BRIEF        Computes the soil respiration and carbon stocks, essentially 
!! following Parton et al. (1987).
!!
!! DESCRIPTION	: The soil is divided into 3 carbon pools, with different 
!! characteristic turnover times : active (1-5 years), slow (20-40 years) 
!! and passive (200-1500 years).\n
!! There are three types of carbon transferred in the soil:\n
!! - carbon input in active and slow pools from litter decomposition,\n
!! - carbon fluxes between the three pools,\n
!! - carbon losses from the pools to the atmosphere, i.e., soil respiration.\n
!!
!! The subroutine performs the following tasks:\n
!!
!! Section 1.\n
!! The flux fractions (f) between carbon pools are defined based on Parton et 
!! al. (1987). The fractions are constants, except for the flux fraction from
!! the active pool to the slow pool, which depends on the clay content,\n
!! \latexonly
!! \input{soilcarbon_eq1.tex}
!! \endlatexonly\n
!! In addition, to each pool is assigned a constant turnover time.\n
!!
!! Section 2.\n
!! The carbon input, calculated in the stomate_litter module, is added to the 
!! carbon stock of the different pools.\n
!!
!! Section 3.\n
!! First, the outgoing carbon flux of each pool is calculated. It is 
!! proportional to the product of the carbon stock and the ratio between the 
!! iteration time step and the residence time:\n
!! \latexonly
!! \input{soilcarbon_eq2.tex}
!! \endlatexonly
!! ,\n
!! Note that in the case of crops, the additional multiplicative factor 
!! integrates the faster decomposition due to tillage (following Gervois et 
!! al. (2008)).
!! In addition, the flux from the active pool depends on the clay content:\n
!! \latexonly
!! \input{soilcarbon_eq3.tex}
!! \endlatexonly
!! ,\n
!! Each pool is then cut from the carbon amount corresponding to each outgoing
!! flux:\n
!! \latexonly
!! \input{soilcarbon_eq4.tex}
!! \endlatexonly\n
!! Second, the flux fractions lost to the atmosphere is calculated in each pool
!! by subtracting from 1 the pool-to-pool flux fractions. The soil respiration 
!! is then the summed contribution of all the pools,\n
!! \latexonly
!! \input{soilcarbon_eq5.tex}
!! \endlatexonly\n
!! Finally, each carbon pool accumulates the contribution of the other pools:
!! \latexonly
!! \input{soilcarbon_eq6.tex}
!! \endlatexonly
!!
!! Section 4.\n
!! If the flag SPINUP_ANALYTIC is set to true, the matrix A is updated following
!! Lardy (2011).
!!
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUTS VARIABLE(S): carbon, resp_hetero_soil
!!
!! REFERENCE(S)   :
!! - Parton, W.J., D.S. Schimel, C.V. Cole, and D.S. Ojima. 1987. Analysis of 
!! factors controlling soil organic matter levels in Great Plains grasslands. 
!! Soil Sci. Soc. Am. J., 51, 1173-1179.
!! - Gervois, S., P. Ciais, N. de Noblet-Ducoudre, N. Brisson, N. Vuichard, 
!! and N. Viovy (2008), Carbon and water balance of European croplands 
!! throughout the 20th century, Global Biogeochem. Cycles, 22, GB2022, 
!! doi:10.1029/2007GB003018.
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!! Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!!
!! FLOWCHART    :
!! \latexonly
!! \includegraphics[scale=0.5]{soilcarbon_flowchart.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

 SUBROUTINE soilcarbon (npts, dt, zz_coef_deep, clay, &
       soilcarbon_input, soilcarbon_input_DOC, floodcarbon_input, control_temp_below, control_moist_below, &
       carbon, carbon_32l, &
       resp_hetero_soil, resp_flood_soil, &
!       MatrixA, &
!!SIMON ADDED ORCHIDOC CHECK MATRIX A
!!MICT permafrost carbon variables added here:
       altmax, lalo, & 
       itau, zz_deep, &
       snowdz, fixed_cryoturbation_depth, &
!!MICT END
       litter_above,litter_below,&
       DOC, moist_soil, DOC_EXP, &
       lignin_struc_above, lignin_struc_below, &
       floodout, runoff_per_soil, drainage_per_soil, wat_flux0, wat_flux, &
       bulk_dens, soil_ph, poor_soils, veget_max, fbact, tprof, soil_mc_32l, soil_mc, soiltile, &
       DOC_to_topsoil, DOC_to_subsoil, flood_frac, &
       precip2ground, precip2canopy, canopy2ground, &
       dry_dep_canopy, DOC_precip2ground, DOC_precip2canopy, DOC_canopy2ground, &
       DOC_infil, DOC_noinfil, interception_storage, biomass, fastr)
!!SIMON END
!gmjc
!       resp_hetero_soil_part 
!end gmjc

!! 0. Variable and parameter declaration
    !! 0.1 Input variables
 
 !!!!MICT STUFF
    INTEGER(i_std), INTENT(in)                            :: itau              !! time step number
!
!   REAL(r_std), DIMENSION(npts,nvm), INTENT(in)		:: heights_snow      !! total thickness of snow levels (m)
!
!  REAL(r_std), DIMENSION(ndeep), INTENT(in)         :: zi_soil        !! depths of intermediate levels (m)
!
  REAL(r_std), DIMENSION(ndeep),   INTENT (in)        :: zz_coef_deep         !! deep vertical profile
!
   REAL(r_std), DIMENSION(ndeep),   INTENT (in)          :: zz_deep           !! deep vertical profile
!
 !  REAL(r_std), DIMENSION(npts,0:nsnow,nvm), INTENT(in)         :: zf_snow       !! depths of full levels (m)
!
!!    REAL(r_std), DIMENSION(ndeep), INTENT(in)   	         :: zprof               !! soil depths (m)
!
!!   REAL(r_std), DIMENSION(npts,ndeep,ncarb), INTENT(in)  :: deepC_pftmean        !! Deep soil carbon profiles, mean over all PFTs
!  
!   REAL(r_std), DIMENSION(npts,nvm), INTENT(in)        :: alt
!
!   INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)     :: alt_ind !! active layer thickness  
!
   REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)       :: altmax                     !! active layer thickness (m)
   REAL(r_std),DIMENSION(npts,2),INTENT(in)          :: lalo              !! Geogr. coordinates (latitude,longitude) (degrees)
!
!   INTEGER(i_std), DIMENSION(npts), INTENT(in)      :: altmax_ind !! Maximum over the year active layer thickness
!
!   INTEGER(i_std), DIMENSION(npts), INTENT(in)      :: altmax_ind_lastyear
!
!   REAL(r_std), DIMENSION(npts, nvm),INTENT(in)	         :: altmax_lastyear     !! Maximum active-layer thickness
!
!   REAL(r_std), DIMENSION(npts,nvm), INTENT(in)         :: z_root !! Rooting depth
!
!   INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)      :: rootlev !! The deepest model level within the rooting depth
!
   REAL(r_std), DIMENSION(npts,nsnow), INTENT(in)    :: snowdz            !! Snow depth [m]
!
    REAL(r_std), DIMENSION(npts,nvm)                 :: veget_max_bg 
!
!    REAL(r_std), INTENT(in)                                    :: diff_k_const
!    REAL(r_std), INTENT(in)                                    :: bio_diff_k_const
!
!
!
!!!!MICT INSERTION END
!!    
    INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
    REAL(r_std), INTENT(in)                               :: dt               !! Time step 
    REAL(r_std), DIMENSION(npts), INTENT(inout)                       :: clay                  !! Clay fraction (unitless, 0-1) 
!!! SIMON ADDED ORCHIDOC

    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(in)    :: soilcarbon_input !! Amount of carbon going into the carbon pools from litter decomposition \f$(gC m^{-2} day^{-1})$\f
!!!
!    REAL(r_std), DIMENSION(npts,nlevs), INTENT(in)        :: control_temp     !! Temperature control of heterotrophic respiration (unitless: 0->1)
!    REAL(r_std), DIMENSION(npts,nlevs), INTENT(in)        :: control_moist    !! Moisture control of heterotrophic respiration (unitless: 0.25->1)
     REAL(r_std), DIMENSION(npts), INTENT(inout)                       :: bulk_dens             !! Variable local of bulk density for the moment must change in the futur (kg m-3)
 
     REAL(r_std), DIMENSION(npts), INTENT(in)                          :: soil_ph               !! soil pH (pH unit, 0-14) 
     REAL(r_std), DIMENSION(npts), INTENT(in)                          :: poor_soils               !! soil pH (pH unit, 0-14) 
    REAL(r_std), DIMENSION(npts,ndeep,npool*2,nvm), INTENT(in)             :: control_temp_below     !! Temperature control of heterotrophic respiration (unitless: 0->1)
    REAL(r_std), DIMENSION(npts,ndeep,nvm), INTENT(in)                 :: control_moist_below    !! Moisture control of heterotrophic respiration (unitless: 0.25->1)

    REAL(r_std), DIMENSION(npts,nlitt,nvm,nelements), INTENT(in)      :: litter_above          !! Metabolic and structural litter,above ground
                                                                                               !! The unit is given by m^2 of ground
                                                                                               !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nbdl,nelements), INTENT(inout) ::litter_below           !! Metabolic and structural litter, below ground
                                                                                               !! The unit is given by m^2 of
                                                                                               !! ground @tex $(gC m^{-2}) $ @endtex
!    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                     :: soilhum               !! Daily soil humidity of each soil layer 
                                                                                               !! (unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                      :: lignin_struc_above    !! Ratio Lignin content in structural litter,
                                                                                               !! above ground, (0-1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,ndeep), INTENT(in)                 :: lignin_struc_below    !! Ratio Lignin content in structural litter,
                                                                                               !! below ground, (0-1, unitless)
    REAL(r_std),DIMENSION (npts), INTENT (in)                         :: floodout              !! flux out of floodplains
    REAL(r_std),DIMENSION (npts,nstm), INTENT(in)                    :: runoff_per_soil       !! Runoff per soil type [mm]
    REAL(r_std),DIMENSION (npts,nstm), INTENT(in)                    :: drainage_per_soil     !! Drainage per soil type [mm]
    REAL(r_std),DIMENSION (npts,nstm), INTENT(in)                     :: wat_flux0             !! Water flux in the first soil layers exported for soil C calculations(mm)
    REAL(r_std),DIMENSION (npts,nslm,nstm), INTENT(in)                :: wat_flux              !! Water flux in the soil layers exported for soil C calculations(mm)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                        :: veget_max             !! PFT "Maximal" coverage fraction of a PFT 
                                                                                               !! defined in the input vegetation map 
                                                                                               !! @tex $(m^2 m^{-2})$ @endtex 
                                                                                               !! The unit is given by m^2 of
                                                                                               !! water @tex $(gC m{-2} of water)$ @endtex
!!SIMON added soil_mc_32l to make soil_mc consistent with 32 layer operations
    REAL(r_std), DIMENSION(npts,ndeep,nvm), INTENT(in) :: fbact
!    REAL(r_std), DIMENSION(npts,ndeep,nvm) 		fbact_doc
!    REAL(r_std), DIMENSION(npts,ndeep,nvm) 		fbact_nodoc
    REAL(r_std), DIMENSION(npts,ndeep,nvm),INTENT (in)	    :: tprof             !! deep temperature profile    
    REAL(r_std),DIMENSION (npts,ndeep,nstm), INTENT(in)             :: soil_mc_32l               !! soil moisture content 32 layer consistent \f($m^3 \times m^3$)\f

    REAL(r_std),DIMENSION (npts,nbdl,nstm), INTENT(in)                :: soil_mc               !! soil moisture content \f($m^3 \times m^3$)\f
    REAL(r_std),DIMENSION (npts,nstm), INTENT (in)                    :: soiltile              !! Fraction of each soil tile (0-1, unitless)


!
    REAL(r_std),DIMENSION (npts,nflow), INTENT(in)                    :: DOC_to_topsoil        !! DOC inputs to top of the soil column, from reinfiltration on
                                                                                               !! floodplains and from irrigation
                                                                                               !! @tex $(gC m^{-2} day{-1})$ @endtex
    !! TF-DOC
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                      :: precip2ground         !! Precipitation not intercepted by vegetation
                                                                                               !! @tex $(mm.dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                      :: precip2canopy         !! Precipitation intercepted by vegetation
                                                                                               !! @tex $(mm.dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                      :: canopy2ground         !! Water flux from canopy to ground
                                                                                               !! @tex $(mm.dt^{-1})$ @endtex 
    REAL(r_std),DIMENSION (npts,nflow), INTENT(in)                    :: DOC_to_subsoil        !! DOC inputs to bottom of the soil column, from returnflow
                                                                                               !! in swamps and lakes
                                                                                               !! @tex $(gC m^{-2} day{-1})$ @endtex
    REAL(r_std), DIMENSION(npts), INTENT(in)                          :: flood_frac            !! Flooded fraction of grid box
                                                                                               !! @tex $(-)$ @endtex
    REAL(r_std), DIMENSION(npts), INTENT(in)                          :: fastr                 !! Fast reservoir (mm)
    
    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                     :: resp_hetero_soil      !! Soil heterotrophic respiration \f$(gC m^{-2} (dtradia one_day^{-1})^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                     :: resp_flood_soil       !! Soil heterotrophic respiration when flooded 
                                                                                               !! \f$(gC m^{-2} (dtradia one_\day^{-1})^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm,nexp,npool,nelements), INTENT(out):: DOC_EXP               !! Exported DOC through runoff, drainage, flood,
                                                                                               !! The unit is given by m^2 of
                                                                                               !! ground \f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(out)                    :: moist_soil            !! moisture in soil for one pixel (m3/m3)
    !! TF-DOC, maybe put all togetehr into one matrix, note that all Throughfall related fluxes are expressed relative to the total area of the pixel
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(out):: dry_dep_canopy        !! Increase in canopy storage of soluble OC & DOC
                                                                                    !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(out):: DOC_precip2canopy     !! Wet deposition of DOC onto canopy
                                                                                    !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(out):: DOC_precip2ground     !! Wet deposition of DOC not intecepted by canopy
                                                                                    !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(out):: DOC_canopy2ground     !! Wet deposition of DOC not intecepted by canopy
                                                                                    !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(out):: DOC_infil             !! Wet deposition of DOC infiltrating into the ground
                                                                                    !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(out):: DOC_noinfil           !! Wet deposition of DOC going into surface runoff
                                                                                    !! @tex $(gC.m^{-2})$ @endtex  
!!!SIMON END
    !! 0.3 Modified variables
!!!SIMON ADDED

!!!need to change
    REAL(r_std), DIMENSION(npts,nvm,ndeep,npool,nelements), INTENT(inout)        :: soilcarbon_input_DOC !! Amount of carbon going into the carbon pools 
                                                                                                        !! from litter decomposition \f$(gC m^{-2} day^{-1})$\f 
    REAL(r_std), DIMENSION(npts,nvm,npool,nelements), INTENT(inout)             :: floodcarbon_input    !! Amount of carbon going directly into the water column
                                                                                                        !! from litter decomposition \f$(gC m^{-2} day^{-1})$\f   
    REAL(r_std), DIMENSION(npts,ncarb,nvm,ndeep), INTENT(inout)                  :: carbon_32l           !! Soil carbon pools: active, slow, or passive, \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements), INTENT(inout)     :: DOC                !! Dissolved Organic Carbon in soil
    !! TF-DOC
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(inout) :: biomass           !! Biomass @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements), INTENT(inout):: interception_storage      !! Storage of DOC and solube OC in canopy
                                                                                          !! @tex $(gC.m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(inout) :: carbon             !! Soil carbon pools: active, slow, or passive, \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,nvm),INTENT(inout)        :: fixed_cryoturbation_depth  !! depth to hold cryoturbation to for fixed runs

!
!
!    REAL(r_std), DIMENSION(npts,nvm,npool,npool), INTENT(inout) :: MatrixA  !! Matrix containing the fluxes between the carbon pools
!!!SIMON END                                                                              !! per sechiba time step 
                                                                                !! @tex $(gC.m^2.day^{-1})$ @endtex
!gmjc
    !! 0.4 Local variables
!SIMON added for consistentency fbact
 
!!!    LOGICAL, DIMENSION(npts,nvm)		:: veget_mask_2d	
!!!SIMON ADDED: A new Tracking, Tuning variable
	REAL(r_std), DIMENSION(npts,nvm,ndeep) 					:: In_Slow
!!!END TUNING TRACKING VARS


    LOGICAL, SAVE                    :: ok_cryoturb               !! cryoturbate the carbon?
    LOGICAL, SAVE                    :: no_pfrost_decomp = .FALSE.!! Whether this is a spinup run
    REAL(r_std), DIMENSION(npts,ncarb,ncarb)                :: frac_carb        !! Flux fractions between carbon pools 
                                                                              !! (second index=origin, third index=destination) 
                                                                              !! (unitless, 0-1)
!!!SIMON ADDED ORCHIDOC
!    REAL(r_std), DIMENSION(npts,ncarb)                    :: frac_resp        !! Flux fractions from carbon pools to the atmosphere (respiration) (unitless, 0-1)
!    REAL(r_std), DIMENSION(npts,ncarb,nelements)          :: fluxtot          !! Total flux out of carbon pools \f$(gC m^{2})$\f
!    REAL(r_std), DIMENSION(npts,ncarb,ncarb,nelements)    :: flux             !! Fluxes between carbon pools \f$(gC m^{2})$\f
    CHARACTER(LEN=7), DIMENSION(ncarb)                    :: carbon_str       !! Name of the carbon pools for informative outputs (unitless)
!    INTEGER(i_std)                                        :: k,kk,m,j         !! Indices (unitless)
!!SIMON THESE ARE FROM MICT 
!    REAL(r_std), DIMENSION(npts,ndeep,nelements)                        :: fbact_a,fbact_s, fbact_p, fbact_docL, fbact_docR,temp
    REAL(r_std), DIMENSION(npts,ndeep,nvm)                        :: fbact_a,fbact_s, fbact_p, fbact_docL, fbact_docR,temp
    REAL(r_std), DIMENSION(npts) 			     :: alt_pftmean, altmax_pftmean, tsurf_pftmean

!!!MICT fbact end
!!SIMON Added fbact_npool to allow npool looping further below
    REAL(r_std), DIMENSION(npts,ndeep,nvm,npool)             :: fbact_npool      !! fbact_npool to allow npool looping
    REAL(r_std), DIMENSION(npts,ndeep,nvm,ncarb)             :: fbact_ncarb      !! fbact_npool to allow npool looping

    REAL(r_std), DIMENSION(npts,nvm,ndeep,npool)                   :: fluxtot_DOC      !! Total flux out of dissolved organic carbon pools \f$(gC m^{2}ground)$\f

    REAL(r_std), DIMENSION(npts,ncarb,nelements,ndeep)             :: fluxtot          !! Total flux out of carbon pools \f$(gC m^{2})$\f

    REAL(r_std), DIMENSION(npts,ncarb,nelements,ndeep)             :: fluxtot_flood    !! Total flux out of carbon pools \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,nvm,ndeep,npool)                   :: fluxtot_DOC_flood!! Total flux out of dissolved organic carbon pools \f$(gC m^{2}ground\)$\f
    REAL(r_std), DIMENSION(npts,npool,ncarb,nelements,nbdl)       :: flux             !! Fluxes between carbon pools \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,nvm,ndeep)                         :: litter_tot       !! total litter (gC/m**2 of ground)
    REAL(r_std), DIMENSION(npts,nvm,ndeep)                         :: LOM              !! total labile organic matter that could induce the decompostion rate
                                                                                      !! (gC/m**2 of ground)
    INTEGER(i_std)                                                :: k,kk,m,j,l,i,kkk,ier,iv,ip !! Indices (unitless)

!!!SIMON ADDED ORCHIDOC
!!SIMON ADDED FROM MICT
    REAL(r_std), SAVE                :: cryoturbation_diff_k_in   !! input time constant of cryoturbation (m^2/y)
    REAL(r_std), SAVE                :: bioturbation_diff_k_in    !! input time constant of bioturbation (m^2/y)
    
    INTEGER(i_std)                   :: month,year,dayno          !! current time parameters

    REAL(r_std)                      :: scnd
!
!!
!!END  MICT  
!!   
    REAL(r_std),DIMENSION(0:nbdl)                                 :: z_soil           !! Soil levels (m)
    REAL(r_std), DIMENSION(npts,nvm, ndeep,npool,nelements)        :: DOC_RE           !! Adsoprtion Desorption flux of DOC (mmol C/kg)
    REAL(r_std), DIMENSION(npts,nvm, ndeep,npool,nelements)        :: DOC_FLUX         !! DOC flux between layers in f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm, ndeep,npool,nelements)        :: DOC_FLUX_DIFF    !! DOC flux by diffusion between layers in f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm, ndeep,npool,nelements)        :: DOC_FLUX_DIFF_old !! DOC flux by diffusion between layers in f$(gC m^{-2} day^{-1})$\f for storage
    REAL(r_std), DIMENSION(npts,nvm,npool,nelements)              :: DOC_RUN          !! DOC lost with runoff in f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm,npool,nelements)              :: DOC_FLOOD        !! DOC lost with flooding in f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm,npool,nelements)              :: DOC_DRAIN        !! DOC lost with drainage in f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements)    :: DOC_old          !! Dissolved Organic Carbon in soil
                                                                                      !! The unit is given by m^2 of
                                                                                      !! ground @tex $(gC m{-2} of ground)$ @endtex variable used for storage
    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements)    :: DOC_old2         !! Dissolved Organic Carbon in soil
                                                                                      !! The unit is given by m^2 of
                                                                                      !! ground @tex $(gC m{-2} of ground)$ @endtex variable used for storage
    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements)    :: DOC_old_buffer   !! Dissolved Organic Carbon in soil
                                                                                      !! The unit is given by m^2 of
                                                                                      !! ground @tex $(gC m{-2} of ground)$ @endtex variable used for storage

    REAL(r_std), DIMENSION(npts,ncarb,nvm,nbdl)                   :: carbon_old       !! Soil carbon pools: active, slow, or passive, \f$(gC m^{2})$\f used for storage
     REAL(r_std), DIMENSION(npts,ncarb,nvm,ndeep)                   :: carbon_copy,test       !! Soil carbon pools: active, slow, or passive, \f$(gC m^{2})$\f used for storage
    
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nbdl)                   :: carbon_old_buffer!! Soil carbon pools: active, slow, or passive, \f$(gC m^{2})$\f used for storage
    REAL(r_std), DIMENSION(npts)                                  :: Qmax             !! Maximum adsorption capacity for DOC in the Langmuir equation (mg kg¿1)
    REAL(r_std), DIMENSION(npts,ndeep)                             :: Kd               !! distribution coefficient between DOC adsorbed and DOC free (unitless [0-1])
    REAL(r_std), DIMENSION(npts,ndeep)                             :: b                !! desorption term of the Initial Mass (IM) isotherm (mmol C/kg)
    REAL(r_std), DIMENSION(npts,ndeep)                             :: IM               !! sorption affinity of the Initial Mass (IM) isotherm (unitless)

    REAL(r_std), DIMENSION(npts)                                  :: Dif_coef         !! Diffusion coeficient used for bioturbation (m2 dt-1)

    REAL(r_std), DIMENSION(npts)                                  :: Dif_DOC          !! Diffusion coeficient used for DOC diffusion (m2 dt-1)

    REAL(r_std), DIMENSION(npts)                                  :: CUE_coef         !! Partition coefficient for DOC decomposed between CO2 and POC (unitless, 0-1)
                                                                                    
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nbdl)                   :: carbon_flux      !! Soil carbon flux within pools towards the soil column\f$(gC m^{2} dt^{-1})$\f
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nbdl)                   :: carbon_flux_old  !! Soil carbon flux within pools towards the soil column\f$(gC m^{2} dt^{-1})$\f
                                                                                      !! used for storage
    REAL(r_std), DIMENSION(npts,nstm)                             :: soilwater_31mm   !! Soil water content of the first x layers (m)
    REAL(r_std), DIMENSION(npts)                                  :: soilDOC_31mm     !! Soil DOC content of the first x layers (m)
    REAL(r_std), DIMENSION(npts)                                  :: soilDOC_corr     !! Soil DOC content of the first x layers (m)
    REAL(r_std), DIMENSION(npts)                                  :: flux_red         !! Flux reduction in upper meter of the soil
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)            :: check_intern     !! Contains the components of the internal
                                                                                      !! mass balance chech for this routine
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                    :: closure_intern   !! Check closure of internal mass balance

    REAL(r_std), DIMENSION(npts,nvm,nelements)                    :: pool_start       !! Start and end pool of this routine 
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                    :: pool_end         !! Start and end pool of this routine 
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)                    :: pool_end_after   !! Start and end pool of this routine after mass balance correction 
                                                                                      !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    !! TF-DOC
    REAL(r_std), DIMENSION(npts,nvm,nelements)                    :: wet_dep_ground    !! total wet deposition of DOC onto the ground
    REAL(r_std), DIMENSION(npts,nvm,nelements)                    :: wet_dep_flood     !! total wet deposition of DOC onto the flooded
    REAL(r_std), DIMENSION(npts)                                  :: bio_frac          !! total wet deposition of DOC onto the flooded
    REAL(r_std), DIMENSION(npts)                                  :: fastr_corr        !! correction factor based on fast reservoir

    INTEGER(i_std)                                                :: pts
!_ ================================================================================================================================
!
      !
IF ( firstcall_soilcarbon ) THEN

!WRITE(numout,*) 'BOOM firstcall print carbon_32l min/max=', minval(carbon_32l), maxval(carbon_32l)
!WRITE(numout,*) 'BOOM firstcall print DOC min/max=', minval(DOC), maxval(DOC)
      
       !Config Key   = NO_PFROST_DECOMP
       !Config Desc  = whether this is spin-up
       !Config Def   = n 
       !Config If    = OK_PC
       !Config Help  = 
       !Config         
       !Config Units = [flag]
       ! 
       no_pfrost_decomp=.FALSE. 
       CALL getin_p('NO_PFROST_DECOMP',no_pfrost_decomp)
       !
       !Config Key   = cryoturbate
       !Config Desc  = Do we allow for cyoturbation?
       !Config Def   = y 
       !Config If    = OK_PC
       !Config Help  = 
       !Config         
       !Config Units = [flag]
       ! 
       ok_cryoturb=.TRUE.
       CALL getin_p('cryoturbate_doc_POC',ok_cryoturb)
       !
       !Config Key   = cryoturbation_diff_k_in
       !Config Desc  = diffusion constant for cryoturbation 
       !Config Def   = 0.001 
       !Config If    = OK_PC
       !Config Help  = 
       !Config         
       !Config Units = [m2/year]
       !  
       cryoturbation_diff_k_in = .001
       CALL getin_p('cryoturbation_diff_k',cryoturbation_diff_k_in)
       !
       !Config Key   = bioturbation_diff_k_in
       !Config Desc  = diffusion constant for bioturbation 
       !Config Def   = 0.0
       !Config If    = OK_PC
       !Config Help  = 
       !Config         
       !Config Units = [m2/year]
       !  
       bioturbation_diff_k_in = 0.0001
       CALL getin_p('bioturbation_diff_k',bioturbation_diff_k_in)
    !
           !!  Arrays allocations
         ALLOCATE (alt(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in alt allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF

       ALLOCATE (altmax_lastyear(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
!           CALL ipslerr_p(3, 'error in altmax_lastyear allocation', 'We need to STOP', 'ntps * nvm', '')
           WRITE (numout,*) ' error in altmax_lastyear allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF 

       ALLOCATE (alt_ind(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in alt_ind allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF

       ALLOCATE (altmax_ind(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in altmax_ind allocation. We stop. We need',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF

       ALLOCATE (altmax_ind_lastyear(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in altmax_ind allocation. We stop. We need',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF

       ALLOCATE (z_root(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in z_root allocation. We stop. We need',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF

       ALLOCATE (rootlev(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in rootlev allocation. We stop. We need',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF
   
       ALLOCATE (zf_soil_B(0:ndeep),stat=ier)
       IF (ier.NE.0) THEN
    WRITE (numout,*) ' error in zf_soil_B allocation. We stop. We need',ndeep+1,' words = '&
              & , ndeep+1
           STOP 'soilcarbon'
       END IF
   
!          ALLOCATE (altmax_lastyear(npts,nvm),stat=ier)
!       IF (ier.NE.0) THEN
!           WRITE (numout,*) ' error in altmax_lastyear allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
!              & , npts*nvm
!           STOP 'soilcarbon'
!       END IF 
           ALLOCATE (veget_mask_2d(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in veget_mask_2d allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
        END IF
       ALLOCATE(lalo_global(npts,2),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in lalo_global allocation. We stop. We need ',npts,' fois ',2,' words = '&
              & , npts*2
           STOP 'soilcarbon'
       END IF
      ALLOCATE (zi_soil(ndeep),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in zi_soil allocation. We stop. We need',ndeep,' words = '&
              & , ndeep
           STOP 'soilcarbon'
       END IF 

       ALLOCATE (zf_snow(npts,0:nsnow,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in zf_snow allocation. We stop. We need', npts, ' fois ',nsnow+1, ' fois ',nvm,' words = '&
              & , npts*(nsnow+1)*nvm
           STOP 'soilcarbon'
       END IF
           ALLOCATE (zi_snow(npts,nsnow,nvm),stat=ier)
       IF (ier.NE.0) THEN           
           WRITE (numout,*) ' error in zi_snow allocation. We stop. We need', npts, ' fois ',nsnow, ' fois ',nvm,' words = '&
              & , npts*nsnow*nvm
           STOP 'soilcarbon'
       END IF

       ALLOCATE (heights_snow(npts,nvm),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in heights_snow allocation. We stop. We need',npts,' fois ',nvm,' words = '&
              & , npts*nvm
           STOP 'soilcarbon'
       END IF
!!     
!!       
           ALLOCATE (zf_snow_nopftdim(npts,0:nsnow),stat=ier)   
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in zf_snow_nopftdim allocation. We stop. We need', npts, ' fois ',nsnow+1,' words = '&
              & , npts*(nsnow+1)
           STOP 'soilcarbon'
       END IF
       
       ALLOCATE (zi_snow_nopftdim(npts,nsnow),stat=ier)
       IF (ier.NE.0) THEN           
           WRITE (numout,*) ' error in zi_snow_nopftdim allocation. We stop. We need', npts, ' fois ',nsnow,' words = '&
              & , npts*nsnow
           STOP 'soilcarbon'
       END IF
!
       !! assign values for arrays
       yr_len = NINT(one_year)

       veget_max_bg(:,2:nvm) = veget_max(:,2:nvm)
       veget_max_bg(:,1) = MAX((un - SUM(veget_max(:,2:nvm), 2)), zero)
	
	veget_mask_2d = veget_max .GT. min_stomate
!!!       veget_mask_2d(:,:) = .TRUE.
!
       lalo_global(:,:) = lalo(:,:)
       alt(:,:) = 0
       altmax_lastyear(:,:) = 0
       alt_ind(:,:) = 0
       altmax_ind_lastyear(:,:) = 0
       z_root(:,:) = 0
       rootlev(:,:) = 0
       heights_snow(:,:) = zero
       z_soil = zero
       zf_soil_B(:) = zero
       zi_soil(:) = zero
       zf_snow(:,:,:) = zero
       zi_snow(:,:,:) = zero
       zf_snow_nopftdim(:,:) = zero
       zi_snow_nopftdim(:,:) = zero
!
!
!
       ALLOCATE (carbon_32l_pftmean(npts,ncarb,ndeep),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in carbon_32l_pftmean allocation. We stop. We need', npts, ' fois ',ncarb, ' fois ',ndeep,' words = '&
              & , npts*ncarb*ndeep
           STOP 'soilcarbon'
       END IF
       
              ALLOCATE (carbon_32l_conct(npts,ncarb,ndeep),stat=ier)
       IF (ier.NE.0) THEN
           WRITE (numout,*) ' error in carbon_32l_pftmean allocation. We stop. We need', npts, ' fois ',ncarb, ' fois ',ndeep,' words = '&
              & , npts*ncarb*ndeep
           STOP 'soilcarbon'
       END IF
!
!
!        carbon_tau(:)  = zero
!        DOC_tau = zero
        priming_param = zero

        !!SIMON addded call for snowlevels
       DO iv = 1, nvm
          heights_snow(:,iv) = SUM(snowdz(:,1:nsnow), 2)
       ENDDO
!
       CALL snowlevels (npts, snowdz, zi_snow, zf_snow, veget_max_bg)
!!
    !! 1.1 soil levels
    z_soil(0) = zero
    z_soil(1:nbdl) = diaglev(1:nbdl)
!
!    !!!SIMON ADDED MICT
!
       zi_soil(:) = zz_deep(:)
    zf_soil_B(1:ndeep) = zz_coef_deep(:)
     zf_soil_B(0) = 0.
!
!!SIMON ADDED these calls
!
	CALL itau2ymds(itau, dt*one_day, year, month, dayno, scnd)
       dayno = (month-1)*30 + dayno

       CALL altcalc_DOC(npts, dt*one_day, dayno, scnd, tprof, zi_soil, alt, alt_ind, altmax, altmax_ind, &
            altmax_lastyear, altmax_ind_lastyear)   
!!WRITE(numout,*) "L742 after altcalc call, altmax_lastyear=", altmax_lastyear
!WRITE(numout,*) 'L742 maxval alt_ind =', (maxval(alt_ind))
!WRITE(numout,*) 'L742 maxval altmax_ind =', (maxval(altmax_ind))
!WRITE(numout,*) 'L742 maxval altmax_lastyear =', (maxval(altmax_lastyear))
	
       IF (printlev>=3 ) THEN
          WRITE(*,*) 'soilcarbon: finished MICT based firstcall calcs'
       ENDIF



        !! 1.2.2 Residence times in carbon pools (in year converted in days)
        priming_param(iactive)=priming_param_iactive  !Priming parameter for mineralization obtained from optimization of SOM-FOM-DEP see Guenet et al., (In prep)
        priming_param(islow)=priming_param_islow
        priming_param(ipassive)=priming_param_ipassive

        ! The values of DOC_tau are coming from Kalbitz et al 2003 Geoderma
        ! (already in days) 
!        1.1.2 Residence times in carbon pools (days)
         carbon_tau(iactive) = carbon_tau_iactive * one_year       ! 1.5 years. This is same as CENTURY. But, in Parton et al. (1987), it's weighted by moisture and temperature dependences.
         carbon_tau(islow) = carbon_tau_islow * one_year          ! 25 years. This is same as CENTURY. But, in Parton et al. (1987), it's weighted by moisture and temperature dependences.
         carbon_tau(ipassive) = carbon_tau_ipassive * one_year       ! 1000 years. This is same as CENTURY. But, in Parton et al. (1987), it's weighted by moisture and temperature dependences.
!         
       !! 1.3 soil levels
        !! 1.4 Messages : display the residence times  
        carbon_str(iactive) = 'active'
        carbon_str(islow) = 'slow'
        carbon_str(ipassive) = 'passive'
        
!        WRITE(numout,*) 'soilcarbon:'
        
        WRITE(numout,*) '   > minimal carbon residence time in carbon pools (d):'
        DO k = 1, ncarb ! Loop over carbon pools
          WRITE(numout,*) '(1, ::carbon_str(k)):',carbon_str(k),' : (1, ::carbon_tau(k)):',carbon_tau(k)
        ENDDO
        !
!!SIMON added the layer multiplier for forced forcesoil restarts
    IF (DANs_restart) THEN        
  	DO l = 1,ndeep
 	   	carbon_32l(:,:,:,l)=carbon_32l(:,:,:,l) * (zf_soil_B(l)-zf_soil_B(l-1))       
 	ENDDO ! l to ndeep
    DANs_restart = .FALSE.
    ENDIF !DANs_restart
        
        firstcall_soilcarbon = .FALSE.
        
    
   END IF !!!firstcall soilcarbon
    !
    ! output
    DOC_noinfil = zero
    DOC_infil = zero
    DOC_canopy2ground = zero
    DOC_precip2ground = zero
    DOC_precip2canopy = zero
    dry_dep_canopy = zero
    moist_soil = zero
    DOC_EXP = zero
    resp_hetero_soil = zero
    resp_flood_soil = zero

    ! local
    LOM = zero
    DOC_RE = zero
    DOC_FLUX = zero
    IM = zero  
    b = zero
    Kd = zero
    Qmax = zero
    carbon_old_buffer = zero
    carbon_old = zero
    DOC_old_buffer = zero
    DOC_old2 = zero
    DOC_old = zero
    DOC_DRAIN = zero
    DOC_FLOOD = zero
    DOC_RUN = zero
    DOC_FLUX_DIFF_old = zero
    DOC_FLUX_DIFF = zero
    frac_carb  = zero
    litter_tot  = zero
    flux = zero
    fluxtot_DOC_flood = zero
    fluxtot_flood = zero
    fluxtot = zero
    fluxtot_DOC = zero
    fbact_npool = zero
    fbact_ncarb = zero
    fbact_docL = zero
    fbact_docR = zero
    fbact_a = zero
    fbact_s = zero
    fbact_p = zero
    temp = zero
    Dif_DOC = zero
    Dif_coef = zero
    carbon_flux_old = zero
    carbon_flux = zero
    CUE_coef = zero
    fastr_corr = zero
    bio_frac = zero
    wet_dep_ground = zero
    wet_dep_flood = zero
    pool_end_after = zero
    pool_end = zero
    pool_start = zero
    closure_intern = zero
    check_intern = zero
    flux_red = zero
    soilDOC_corr = zero
    soilDOC_31mm = zero
    soilwater_31mm = zero


    !! printlev is the level of diagnostic information, 0 (none) to 4 (full)
    IF (printlev>=3) WRITE(numout,*) 'Entering soilcarbon' 



    CALL itau2ymds(itau, dt*one_day, year, month, dayno, scnd)
                dayno = (month-1)*30 + dayno
  
!WRITE(numout,*) 'soilcarbon L1028'

   CALL altcalc_DOC(npts, dt*one_day, dayno, scnd, tprof, zi_soil, alt, alt_ind, altmax, altmax_ind, &
        altmax_lastyear, altmax_ind_lastyear)  


!! 1. Initializations:::  fbact = (dt/carbon_tau_iactive)* moist *temp ctrls ...SO
    
    fbact_docL = fbact * (carbon_tau_iactive*one_year/DOC_tau_labile) 
    fbact_docR = fbact * (carbon_tau_iactive*one_year/DOC_tau_stable) 
!  WRITE(numout,*) 'fbact max/min =', maxval(fbact),minval(fbact)
!  WRITE(numout,*) 'DOC_tau_labile = ', DOC_tau_labile
!  WRITE(numout,*) 'doc_tau_labile = ', doc_tau_labile
    
    fbact_npool(:,:,:,imetabo) = fbact_docL
    fbact_npool(:,:,:,imetbel) = fbact_docL
    fbact_npool(:,:,:,istrabo) = fbact_docL
    fbact_npool(:,:,:,istrbel) = fbact_docL
    fbact_npool(:,:,:,iact) = fbact_docL
    fbact_npool(:,:,:,ipas) = fbact_docR
    fbact_npool(:,:,:,islo) = fbact_docR
	fbact_a=fbact
    fbact_s = fbact_a *  (carbon_tau_iactive/carbon_tau_islow) 
    fbact_p = fbact_a *  (carbon_tau_iactive/carbon_tau_ipassive)
 
!WRITE(numout,*) 'fbact_npool max/min =', maxval(fbact_npool),minval(fbact_npool) 
!WRITE(numout,*) 'flood_frac max/min =', maxval(flood_frac),minval(flood_frac) 
 
 !!!
	fbact_ncarb(:,:,:,iactive)=fbact_a
	fbact_ncarb(:,:,:,islow)=fbact_s
	fbact_ncarb(:,:,:,ipassive)=fbact_p
    !!!SIMON ADDED Below
!    
!    fbact_npool(:,:,:,iactive)=fbact_a
!    fbact_npool(:,:,:,islow)=fbact_s
!    fbact_npool(:,:,:,ipassive)=fbact_p
!!!SIMON ADDED ORCHIDOC
    !! 1.0 Calculation of soil moisture
    DO l = 1,nbdl
      DO i = 1, nstm
        moist_soil(:,l)= moist_soil(:,l) + soil_mc(:,l,i)*soiltile(:,i)
      ENDDO
    ENDDO
    
!END



    !! DOC flux reduction factor, effects of poor soils
    flux_red(:) = flux_red_sro * (un - poor_soils(:)) + poor_soils(:)

    !! 1.2 TF-DOC: Initialize wet and dry deposition of DOC
    DOC_precip2canopy(:,:,icarbon) = zero
    DOC_precip2ground(:,:,icarbon) = zero
    dry_dep_canopy(:,:,icarbon) = zero
    bio_frac(:)=SUM(veget_max(:,2:13),DIM=2)

    IF (ok_TF_DOC) THEN   
      !! Calculate the DOC depoited onto the ground
      !! TF Calculate thoughfall DOC

      !! TF.1 Flux of DOC with precipitation onto the ground and onto the canopy (interception storage)
      DOC_precip2canopy(:,2:13,icarbon) = precip2canopy(:,2:13) * conc_DOC_rain * 1e-3
      DOC_precip2ground(:,2:13,icarbon) = precip2ground(:,2:13) * conc_DOC_rain * 1e-3
      !! TF.2 Daily contirubitoon of dry deposition and exsuadtes and feces

      DO j = 2,9! Loop over PFTs composed of trees
         DO i = 1, npts! Loop over # of pixels
            IF (veget_max(i,j) .GT. 0) THEN
               dry_dep_canopy(i,j,icarbon) = DOC_incr_per_LEAF_M2 * dt * biomass(i,j,ileaf,icarbon) * veget_max(i,j)
               !! So far, this is only the parametrisation for PFT=2
               !! sa the rate is per day, we have to multiply by dt
            ELSE
               dry_dep_canopy(i,j,icarbon) = zero
            ENDIF
         ENDDO ! Loop over # of pixels
      ENDDO ! # End Loop over # of PFTs
    ENDIF !(ok_TF_DOC)

    !! 1.4 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 14. Initial biomass and harvest pool and all other
    !  relevant pools were set to zero before calling this routine.
    pool_start(:,:,:) = zero
    IF (ld_doc) THEN
      !Starting carbon pool
      DO m = 1, nvm
        DO i = 1,ncarb
          DO l = 1, ndeep
                 pool_start(:,m,icarbon) = pool_start(:,m,icarbon) + &
                           ( carbon_32l(:,i,m,l) ) * veget_max(:,m)
          ENDDO
        ENDDO
        pool_start(:,m,icarbon) = pool_start(:,m,icarbon) + interception_storage(:,m,icarbon)
      ENDDO
      !write (numout,*) 'soilcarbon L435 preDOC pool DOC maxval =', maxval(DOC)
      !Starting DOC pool
      DO m = 1, nvm
        DO i = 1,npool
          DO l = 1, ndeep
            DO j = 1, ndoc
               pool_start(:,m,icarbon) = pool_start(:,m,icarbon) + &
                         ( DOC(:,m,l,j,i,icarbon)) * veget_max(:,m)
            ENDDO
          ENDDO
        ENDDO
      ENDDO
 !     write (numout,*) 'soilcarbon  preDOC pool DOC maxval,minval =', maxval(DOC),minval(DOC)

      !Starting litter below pool
      DO m = 1,nvm
        DO l = 1, ndeep
          pool_start(:,m,icarbon) = pool_start(:,m,icarbon) + &
                      (soilcarbon_input_DOC(:,m,l,imetbel,icarbon)+soilcarbon_input_DOC(:,m,l,istrbel,icarbon)+ &
                      soilcarbon_input_DOC(:,m,l,imetabo,icarbon)+soilcarbon_input_DOC(:,m,l,istrabo,icarbon)) * &
                      veget_max(:,m) * dt
        ENDDO
      ENDDO
      !write (numout,*) 'soilcarbon L459 soilcarbon_input_DOC maxval =', maxval(soilcarbon_input_DOC(:,m,l,istrbel,icarbon))

      !Starting litter above pool
      DO m = 1,nvm
        WHERE (veget_max(:,m) .GT. zero)
          pool_start(:,m,icarbon) = pool_start(:,m,icarbon) + &
          (DOC_to_topsoil(:,idocl) + DOC_to_subsoil(:,idocl) + DOC_to_topsoil(:,idocr) + DOC_to_subsoil(:,idocr) + &
               floodcarbon_input(:,m,iact,icarbon) + floodcarbon_input(:,m,islo,icarbon))*dt*veget_max(:,m) + &
          DOC_precip2canopy(:,m,icarbon) + DOC_precip2ground(:,m,icarbon) + dry_dep_canopy(:,m,icarbon)
        ELSEWHERE
          pool_start(:,m,icarbon) = pool_start(:,m,icarbon) 
        ENDWHERE
      ENDDO
    ENDIF

    !! 1.2 Get soil "constants"
    !! 1.2.1 Flux fractions between carbon pools: depend on clay content, recalculated each time
    ! From active pool: depends on clay content
    frac_carb(:,iactive,iactive) = zero
    frac_carb(:,iactive,ipassive) = frac_carb_ap /(un - metabolic_ref_frac + active_to_pass_clay_frac*clay(:))
    frac_carb(:,iactive,islow) = un - frac_carb(:,iactive,ipassive)  
    ! 1.2.1.2 from slow pool
    frac_carb(:,islow,islow) = zero
    frac_carb(:,islow,iactive) = frac_carb_sa
    frac_carb(:,islow,ipassive) = un - frac_carb(:,islow,iactive) 
    ! From passive pool
    frac_carb(:,ipassive,ipassive) = zero
    frac_carb(:,ipassive,iactive) = frac_carb_pa
    frac_carb(:,ipassive,islow) = un - frac_carb(:,ipassive,iactive)
!!!!SIMON END

    !! 1.1 Get soil "constants"
    !! 1.1.1 Flux fractions between carbon pools: depend on clay content, recalculated each time
    ! From active pool: depends on clay content
    !frac_carb(:,iactive,iactive) = zero
    !frac_carb(:,iactive,ipassive) = frac_carb_ap
    !frac_carb(:,iactive,islow) = un - (metabolic_ref_frac - active_to_pass_clay_frac*clay(:)) - frac_carb(:,iactive,ipassive)

    ! 1.1.1.2 from slow pool

    !frac_carb(:,islow,islow) = zero
    !frac_carb(:,islow,iactive) = frac_carb_sa
    !frac_carb(:,islow,ipassive) = frac_carb_sp

    ! From passive pool
    !frac_carb(:,ipassive,ipassive) = zero
    !frac_carb(:,ipassive,iactive) = frac_carb_pa
    !frac_carb(:,ipassive,islow) = frac_carb_ps

    
!!SIMON added ORCHIDOC
! 1.2.1.3 Initialization
    carbon_flux =  zero
    carbon_flux_old(:,:,:,:) =  zero

!**********************CRASHES AFTER:
!!
!!SIMON: All the below is directly out of MICT deep_carbcycle
!!MICT copy in BEGIN
    ! Prepare values for arrays
    veget_max_bg(:,2:nvm) = veget_max(:,2:nvm)
    veget_max_bg(:,1) = MAX((un - SUM(veget_max(:,2:nvm), 2)), zero)
    ! whether this is a C spin-up; if not, then 

IF ( .NOT. no_pfrost_decomp ) THEN

    
            IF ( ANY(rootlev(:,:) .GT. ndeep) ) THEN
               WRITE(*,*) 'problems with rootlev:', rootlev
               STOP
            ENDIF
  
            DO iv = 1, nvm
                  heights_snow(:,iv) = SUM(snowdz(:,1:nsnow), 2)
           ENDDO
!WRITE(numout,*) 'soilcarbon L1023'

!WRITE(numout,*) "L1049 after altcalc call, altmax_lastyear=", altmax_lastyear
!WRITE(numout,*) 'soilcarbon L1033'
  
            ! list pft-mean alt and altmax for debugging purposes
            IF (printlev>=3) THEN
               alt_pftmean(:) = 0.
               altmax_pftmean(:) = 0.
               tsurf_pftmean(:) = 0.
               DO iv = 1, nvm
                  WHERE ( veget_mask_2d(:,iv) )
                     alt_pftmean(:) = alt_pftmean(:) + alt(:,iv)*veget_max_bg(:,iv)
                     altmax_pftmean(:) = altmax_pftmean(:) + altmax(:,iv)*veget_max_bg(:,iv)
                     tsurf_pftmean(:) = tsurf_pftmean(:) + tprof(:,1,iv)*veget_max_bg(:,iv)
                  END WHERE
               END DO
            END IF
!WRITE(numout,*) 'soilcarbon L1048'
  
            ! Make sure the rooting depth is within the active layer
!WRITE(numout,*) "L1069 after altcalc call, altmax_ind_lastyear=", altmax_ind_lastyear
            
            !need to sort out the rooting depth, by each STOMATE PFT
            WHERE ( altmax_lastyear(:,:) .LT. z_root_max .and. veget_mask_2d(:,:) )
               z_root(:,:) = altmax_lastyear(:,:)
               rootlev(:,:) = altmax_ind_lastyear(:,:) 
            ELSEWHERE ( veget_mask_2d(:,:) )
               z_root(:,:) = z_root_max
               rootlev(:,:) = altmax_ind_lastyear(:,:)  
            ENDWHERE

!IF (minval(DOC) .LT. zero) THEN 
!WRITE(numout,*) 'BOOM DOC negative before cryoturbate'
!ENDIF


!WRITE(numout,*) 'soilcarbon L1060'
!WRITE(numout,*) 'L1083 Layer 12 carbon_32l min/max=', minval(carbon_32l(:,:,:,12)), maxval(carbon_32l(:,:,:,12))
!WRITE(numout,*) 'L1083 Layer 12  DOC min/max=', minval(DOC(:,:,12,:,:,:)), maxval(DOC(:,:,12,:,:,:))

!!!NOTE: SIMON took MICT deepcarb use of diff_k_const which is swapped for cryoturbation_diff_k_in/(one_day*one_year)
!WRITE(numout,*) 'About to enter cryoturbate first call (diffuse)' 
!WRITE (numout,*) 'carbon_32l max before cryoturbate =', maxval(carbon_32l)
carbon_copy=carbon_32l

	DO l=1,ndeep
 		carbon_32l(:,:,:,l)=carbon_32l(:,:,:,l) /(zf_soil_B(l)-zf_soil_B(l-1))
	ENDDO
	DO l=1,ndeep
 		litter_below(:,:,:,l,:)=litter_below(:,:,:,l,:) /(zf_soil_B(l)-zf_soil_B(l-1))
	ENDDO
	DO l=1,ndeep
 		DOC(:,:,l,:,:,:)=DOC(:,:,l,:,:,:) /(zf_soil_B(l)-zf_soil_B(l-1))
	ENDDO

            IF (ok_cryoturb) CALL cryoturbate_doc_POC(npts, dt*one_day, dayno, altmax_ind, DOC, carbon_32l, litter_below, &
       'diffuse', cryoturbation_diff_k_in/(one_day*one_year), bioturbation_diff_k_in/(one_day*one_year), altmax_lastyear, fixed_cryoturbation_depth)
!
!

!WRITE(numout,*) 'About to enter cryoturbate second call (coefficients)'               
            IF (ok_cryoturb) CALL cryoturbate_doc_POC(npts, dt*one_day, dayno, altmax_ind, DOC, carbon_32l, litter_below, &
       'coefficients', cryoturbation_diff_k_in/(one_day*one_year), bioturbation_diff_k_in/(one_day*one_year), altmax_lastyear, fixed_cryoturbation_depth)
	
!	WHERE (carbon_copy .GT. zero) 
!	test=carbon_32l/carbon_copy 
!	ELSEWHERE
!	test=1
!	ENDWHERE
!	IF (ANY(test .GT. 1.01)) write (numout,*) 'BOOM increase GT 1%, test=', test


!WRITE (numout,*) 'carbon_32l max AFTER cryoturbate =', maxval(carbon_32l)

!write (numout,*) '1102 BOOM xe_ALL ', xe_ALL
!write (numout,*) '1102 BOOM xc_cryoturb range', maxval(xc_cryoturb), minval(xc_cryoturb)    
!write (numout,*) '1102 BOOM xd_cryoturb range', maxval(xd_cryoturb), minval(xd_cryoturb) 

	DO l=1,ndeep
 		carbon_32l(:,:,:,l)=carbon_32l(:,:,:,l)*(zf_soil_B(l)-zf_soil_B(l-1))
	ENDDO	
	DO l=1,ndeep
 		litter_below(:,:,:,l,:)=litter_below(:,:,:,l,:)*(zf_soil_B(l)-zf_soil_B(l-1))
	ENDDO
	DO l=1,ndeep
 		DOC(:,:,l,:,:,:)=DOC(:,:,l,:,:,:)*(zf_soil_B(l)-zf_soil_B(l-1))
	ENDDO
 



!IF (maxval(alpha_DOC) .GT. zero) THEN
!write (numout,*) 'L1098 BOOM alpha_DOC gt zero', (maxval(alpha_DOC))   
!ENDIF
!IF (maxval(beta_DOC) .GT. zero) THEN
!write (numout,*) 'L1098 BOOM beta_DOC gt zero', (maxval(beta_DOC)) 
!ENDIF 
!IF (maxval(beta_DOC) .GT. zero) THEN
!write (numout,*) 'L1098 BOOM alpha_C gt zero', (maxval(alpha_C))     
!ENDIF 
!IF (maxval(beta_C) .GT. zero) THEN
!write (numout,*) 'L1098 BOOM beta_C gt zero', (maxval(beta_C))     
!ENDIF
!IF (maxval(xc_cryoturb) .GT. zero) THEN
!write (numout,*) 'L1098 BOOM xc_cryoturb gt zero', (maxval(xc_cryoturb))     
!ENDIF
!IF (maxval(xd_cryoturb) .GT. zero) THEN
!write (numout,*) 'L1098 BOOM xd_cryoturb gt zero', (maxval(xd_cryoturb))     
!ENDIF




!IF (minval(DOC) .LT. zero) THEN 
!WRITE(numout,*) 'BOOM DOC negative after cryoturbate'
!ENDIF
!	IF (minval(carbon_32l) .LT. zero) THEN
!	WRITE(numout,*) 'BOOM right after cryoturbate is negative'
!	ENDIF
!
!WRITE(numout,*) 'soilcarbon L1071'
!
!!!%%%SIMON: SIMON commented this call from MICT because it is done by littercalc
!             CALL carbinput_DOC(kjpindex,time_step,time,no_pfrost_decomp,tprof,tsurf,hslong, dayno,z_root,altmax, &
!!!Same with soilcarbon decomposition? Obviously we calculate decomp elsewhere but CHECK if permafrost_decomp
!!!routine has processes (excluding gas stuff) that should be put in to decomposition calculations
!!!--> Dont include the routine it calculatesthe same stuff as the LEAK code.  Just check clay is included here.
    carbon=SUM(carbon_32l,DIM=4)
    ! define pft-mean soil C profile
    carbon_32l_pftmean(:,:,:) = 0._r_std
    carbon_32l_conct(:,:,:) = 0._r_std
!
!WRITE(numout,*) 'soilcarbon L1082'
!
	DO ip=1,npts
		DO iv=1,nvm
			carbon_32l_pftmean(ip,:,:)=carbon_32l_pftmean(ip,:,:)+carbon_32l(ip,:,iv,:)*veget_max(ip,iv)
		ENDDO
	ENDDO
	DO l=1,ndeep
                     carbon_32l_conct(:,:,l)=carbon_32l_conct(:,:,l) / (zf_soil_B(l)-zf_soil_B(l-1))
	END DO
ENDIF !
!
!WRITE(numout,*) 'soilcarbon L1091'

!!%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    !! 1.5 Initialization
!!!SIMON ADD ORCHIDOC 
    !! 1.5.1 Initialization of parameters
    Dif_coef(:) = (Dif/one_year)*dt
    Dif_DOC(:)=D_DOC*dt        !We divided by one_year  
                                   !because the unit of the Dif
                                   !parameter is m2 yr-1 and the time stept of the model
                                   !is dt (1/48, by default, i.e. the fraction
                                   !of 30 min expressed in days) 
    CUE_coef(:)=CUE
!WRITE(numout,*) 'soilcarbon L1106'
!!!SIMON END
    !! 1.5.2 Set soil respiration and DOC fluxes to zero
    resp_hetero_soil(:,:) = zero
    resp_flood_soil(:,:) = zero

! gmjc
!    resp_hetero_soil_part(:,:,:) = zero
!!!SIMON ADD ORCHIDOC
    DOC_EXP(:,:,:,:,:) = zero
    DOC_RE(:,:,:,:,:) = zero
    DOC_FLUX(:,:,:,:,:) = zero
    DOC_FLUX_DIFF(:,:,:,:,:) = zero
    DOC_FLUX_DIFF_old(:,:,:,:,:) = zero
    DOC_RUN(:,:,:,:)= zero
    DOC_FLOOD(:,:,:,:)= zero
    DOC_DRAIN(:,:,:,:)= zero
    fluxtot(:,:,:,:)= zero
    fluxtot_DOC(:,:,:,:) = zero
    fluxtot_flood(:,:,:,:) = zero
    fluxtot_DOC_flood(:,:,:,:) = zero
    litter_tot(:,:,:) = zero
    DOC_canopy2ground(:,:,:) = zero
    wet_dep_flood(:,:,:) = zero
    wet_dep_ground(:,:,:) = zero
    LOM(:,:,:)= zero
! end gmjc
!WRITE(numout,*) 'soilcarbon L1133'
!! 2. Update the carbon stocks with the different soil carbon input

    IF (ANY(bulk_dens < 0) ) CALL ipslerr_p(3, 'stomate_soilcarbon', 'bulk_dens is negative', '', '')
    ! bulk_density must be in kg m-3 and is generally given in g cm-3
!!!SIMON MUST UNCOMMENT WHEN BULK D IS INCLUDED!
    WHERE (bulk_dens(:) .LT. 500)
      bulk_dens(:) = bulk_dens(:)*1e3
    ENDWHERE
!WRITE(numout,*) 'soilcarbon L1142'
!!!SIMON ADD ORCHIDOC
    !!1.6 Calculating water content of the first 5 layers
    soilwater_31mm(:,:)= zero
    soilwater_31mm(:,:)= soilwater_31mm(:,:) + (z_soil(1) * soil_mc(:,1,:))   
    IF (sro_bottom .GT. 1) THEN
      DO l=2, sro_bottom
         soilwater_31mm(:,:)= soilwater_31mm(:,:) + ((z_soil(l)-z_soil(l-1)) * soil_mc(:,l,:))
      ENDDO
    ENDIF
!WRITE(numout,*) 'soilcarbon L1152'
!     write (numout,*) 'soilcarbon 1152 MU_SOIL =', mu_soil
!     write (numout,*) 'soilcarbon 1152 ZI_SOIL =', zi_soil
     
!!!mu_soil is calculated here and used in cryoturbation
 mu_soil = zi_soil(1) / ( zi_soil(2) - zi_soil(1) )
!
!WRITE(numout,*) 'soilcarbon L1150'

    !! 2. Fluxes between carbon reservoirs, and to the atmosphere (respiration) \n 
    !! 2.1 calculation of TF fluxes
    !! Calculation of TF-Fluxes
!!!SIMON ADD ORCHIDOC
    !! TF.3 Calculate the flow to the ground

    !! Calculate max possible DOC flux, assuming a maximum concentration
    DO j = 2,nvm! Loop over # of PFTs
      DO i = 1, npts! Loop over # of pixels
        IF (veget_max(i,j) > zero) THEN
          DOC_canopy2ground(i,j,icarbon) = min(canopy2ground(i,j)*conc_DOC_max * 1e-3, &
               &                                              interception_storage(i,j,icarbon))
        ELSE
          DOC_canopy2ground(i,j,icarbon) = zero
        ENDIF
      ENDDO ! Loop over # of pixels
    ENDDO ! # End Loop over # of PFTs

    DO j = 1,nvm! Loop over # of PFTs
      wet_dep_flood(:,j,icarbon)  = (DOC_precip2ground(:,j,icarbon) + DOC_canopy2ground(:,j,icarbon)) * flood_frac(:)
      wet_dep_ground(:,j,icarbon) = (DOC_precip2ground(:,j,icarbon) + DOC_canopy2ground(:,j,icarbon)) * (un-flood_frac(:))
    ENDDO

    !! TF.4 Update the canopy storage of soluble OC and DOC

    DO j = 2,nvm! Loop over # of PFTs
      DO i = 1, npts! Loop over # of pixels
        IF (veget_max(i,j) > zero) THEN
          interception_storage(i,j,icarbon) = interception_storage(i,j,icarbon) + dry_dep_canopy(i,j,icarbon) + &
                     &                          DOC_precip2canopy(i,j,icarbon) - DOC_canopy2ground(i,j,icarbon)
        ELSE
          interception_storage(i,j,icarbon) = zero
        ENDIF
      ENDDO ! Loop over # of pixels
    ENDDO ! # End Loop over # of PFTs

    !! TF.5 Calculate the fraction that is infiltrating into the soil
    DOC_infil = zero
    DOC_noinfil = zero
!WRITE(numout,*) 'soilcarbon L1191'

    !! 2.2 Input from litter to DOC and from reinflitration in floodplains
    !! For the variables DOC_to_topsoil and DOC_to_subsoil we multiplied by
    !veget_max because it is not defined per PFT and therefore we redistribute
    !it following veget_max. Moreover we assume the all the DOC coming from
    !floodplains reinfiltration is going to DOC coming from the active pool.

!!    REAL(r_std), DIMENSION(npts,nvm,ndeep,npool,nelements), INTENT(inout)        :: soilcarbon_input_DOC !! Amount of carbon going into the carbon pools 
!!!    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements), INTENT(inout)     :: DOC                !! Dissolved Organic Carbon in soil
    !! TF-DOC
In_Slow(:,:,:)=zero

    DO m = 2, nvm
      DO l = 1,ndeep
        DOC(:,m,l,ifree,imetabo,icarbon) = DOC(:,m,l,ifree,imetabo,icarbon) + soilcarbon_input_DOC(:,m,l,imetabo,icarbon)*dt 
        DOC(:,m,l,ifree,istrabo,icarbon) = DOC(:,m,l,ifree,istrabo,icarbon) + soilcarbon_input_DOC(:,m,l,istrabo,icarbon)*dt
        DOC(:,m,l,ifree,imetbel,icarbon) = DOC(:,m,l,ifree,imetbel,icarbon) + soilcarbon_input_DOC(:,m,l,imetbel,icarbon)*dt
        DOC(:,m,l,ifree,istrbel,icarbon) = DOC(:,m,l,ifree,istrbel,icarbon) + soilcarbon_input_DOC(:,m,l,istrbel,icarbon)*dt
      ENDDO
!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after soilcarbon_input_DOC'      
!ENDIF

      WHERE (veget_max(:,m) .GT. zero)
        DOC(:,m,1,ifree,iact,icarbon) = DOC(:,m,1,ifree,iact,icarbon) + DOC_to_topsoil(:,idocl)*dt/bio_frac(:) &
           + wet_dep_ground(:,m,icarbon) * undemi/veget_max(:,m)
	
        DOC(:,m,nbdl,ifree,iact,icarbon) = DOC(:,m,nbdl,ifree,iact,icarbon) + DOC_to_subsoil(:,idocl)*dt/bio_frac(:)

        DOC(:,m,1,ifree,islo,icarbon) = DOC(:,m,1,ifree,islo,icarbon) + DOC_to_topsoil(:,idocr)*dt/bio_frac(:) &
           + wet_dep_ground(:,m,icarbon) * undemi/veget_max(:,m)

        DOC(:,m,nbdl,ifree,islo,icarbon) = DOC(:,m,nbdl,ifree,islo,icarbon) + DOC_to_subsoil(:,idocr)*dt/bio_frac(:)
      ELSEWHERE
        DOC(:,m,1,ifree,iact,icarbon) = DOC(:,m,1,ifree,iact,icarbon) 
        DOC(:,m,nbdl,ifree,iact,icarbon) = DOC(:,m,nbdl,ifree,iact,icarbon)
      ENDWHERE
!      write (numout,*) 'soilcarbon L1224 DOC  maxval =', maxval(DOC)
!     write (numout,*) 'DOC(:,m,nbdl,ifree,iact,icarbon)  maxval =', maxval(DOC(:,m,nbdl,ifree,iact,icarbon))

    ENDDO
!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after TF+floodinput calc'      
!ENDIF

!!!SIMON END ORCHIDOC

!! 3. Fluxes between carbon reservoirs, and to the atmosphere (respiration) \n

    !! 3.1. Determine the respiration fraction : what's left after
    ! subtracting all the 'pool-to-pool' flux fractions
    ! Diagonal elements of frac_carb are zero
    !    VPP killer:
    !     frac_resp(:,:) = 1. - SUM( frac_carb(:,:,:), DIM=3 )

!!!SIMON ADD ORCHIDOC (commented frac_resp)
!    frac_resp(:,:) = un - frac_carb(:,:,iactive) - frac_carb(:,:,islow) - &
!         frac_carb(:,:,ipassive) 

    !! 3.2. Calculate fluxes
!IF (maxval(DOC_FLUX_DIFF) .GT. zero) THEN
!	WRITE(numout,*) 'L1859  2.3.8 BOOM firstcall print DOC_FLUX_DIFF 11l max=', maxval(DOC_FLUX_DIFF(:,:,11,:,:))
!	WRITE(numout,*) 'L1859  2.3.8 BOOM firstcall print DOC_FLUX_DIFF 12l max=', maxval(DOC_FLUX_DIFF(:,:,12,:,:))
!ENDIF

!WRITE(numout,*) '1850  2.3.8 BOOM firstcall print tprof 12 layer min/max=', minval(tprof(:,12,:)), maxval(tprof(:,12,:))

!    DO m = 2,nvm ! Loop overPFTs
    DO m = 2,nvm
      !calculate total litter available
      litter_tot(:,m,1)=litter_above(:,imetabolic,m,icarbon)+litter_above(:,istructural,m,icarbon)
    
      DO l = 1, nbdl
        litter_tot(:,m,l)=litter_tot(:,m,l) + litter_below(:,imetabolic,m,l,icarbon)+litter_below(:,istructural,m,l,icarbon)
      ENDDO
    ENDDO
    
    DO m = 1,nvm ! Loop overPFTs
      !! 2.3.1. Flux out of pools
      DO k = 1, ncarb ! Loop over carbon pools from which the flux comes
!!!SIMON Changed l=1,nbdl to ndeep below --CHECK consistency
	DO l = 1,ndeep
!
          IF (k == iactive) THEN
             LOM(:,m,l) = litter_tot(:,m,l) + DOC(:,m,l,ifree,imetabo,icarbon) + DOC(:,m,l,ifree,istrabo,icarbon) + DOC(:,m,l,ifree,imetbel,icarbon) + DOC(:,m,l,ifree,istrbel,icarbon)
!
!
          ELSEIF (k == islow) THEN
             LOM(:,m,l) = litter_tot(:,m,l) + carbon_32l(:,iactive,m,l) + DOC(:,m,l,ifree,iact,icarbon) + &
                          DOC(:,m,l,ifree,imetabo,icarbon) + DOC(:,m,l,ifree,istrabo,icarbon) + DOC(:,m,l,ifree,imetbel,icarbon) + DOC(:,m,l,ifree,istrbel,icarbon)

          ELSEIF (k == ipassive) THEN
             LOM(:,m,l) = litter_tot(:,m,l) + carbon_32l(:,iactive,m,l) + carbon_32l(:,islow,m,l) + DOC(:,m,l,ifree,iact,icarbon) + DOC(:,m,l,ifree,islo,icarbon) + &
                          DOC(:,m,l,ifree,imetabo,icarbon) + DOC(:,m,l,ifree,istrabo,icarbon) + DOC(:,m,l,ifree,imetbel,icarbon) + DOC(:,m,l,ifree,istrbel,icarbon)
!
!     write (numout,*) 'soilcarbon L1292 LOM(:,m,l)  maxval =', maxval(LOM(:,m,l))

          ELSE
             WRITE(numout,*) "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
             WRITE(numout,*) "BE CAREFUL SOMETHING STRANGE HAPPENS"
             WRITE(numout,*) "IN STOMATE_SOILCARBON FOR LOM CALCULATION"
             WRITE(numout,*) "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"

          ENDIF

          
    IF (PRIMING) THEN  !!Soil priming default for DOC module = T
       WHERE ((soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero) .AND. (natural(m)))
               fluxtot(:,k,icarbon,l) =fbact_ncarb(:,l,m,k) *(un-exp(-priming_param(k)*LOM(:,m,l))) * carbon_32l(:,k,m,l) * &
               (un-flood_frac(:)) !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) * 
               fluxtot_flood(:,k,icarbon,l) =fbact_ncarb(:,l,m,k)*(un-exp(-priming_param(k)*LOM(:,m,l))) * carbon_32l(:,k,m,l) * &
                   flood_frac(:) * un/trois !! control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) * 
                    
            ! C3 crop
       ELSEWHERE ((soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero) .AND.( (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) ))
               fluxtot(:,k,icarbon,l) =fbact_ncarb(:,l,m,k)*(un-exp(-priming_param(k)*LOM(:,m,l))) * carbon_32l(:,k,m,l) * &
                     flux_tot_coeff(1) * (un-flood_frac(:)) !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
               fluxtot_flood(:,k,icarbon,l) =fbact_ncarb(:,l,m,k)*(un-exp(-priming_param(k)*LOM(:,m,l))) * carbon_32l(:,k,m,l) * &
                     flux_tot_coeff(1) * flood_frac(:) * un/trois !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
            ! C4 crop
       ELSEWHERE ((soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero) .AND.( (.NOT. natural(m)) .AND. (is_c4(m)) )) 
               fluxtot(:,k,icarbon,l) =fbact_ncarb(:,l,m,k)*(un-exp(-priming_param(k)*LOM(:,m,l))) * carbon_32l(:,k,m,l) * &
                     flux_tot_coeff(2) * (un-flood_frac(:)) !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
               fluxtot_flood(:,k,icarbon,l) =fbact_ncarb(:,l,m,k)*(un-exp(-priming_param(k)*LOM(:,m,l))) * carbon_32l(:,k,m,l) * &
                    flux_tot_coeff(2) * flood_frac(:) * un/trois !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) * 
       ELSEWHERE
               fluxtot(:,k,icarbon,l) = zero
               fluxtot_flood(:,k,icarbon,l) = zero
       ENDWHERE
!!!ELSE	
!   
   ELSE !!NO priming effect
       WHERE ((soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero) .AND. (natural(m)))
               fluxtot(:,k,icarbon,l) =fbact_ncarb(:,l,m,k) * carbon_32l(:,k,m,l) * &
               (un-flood_frac(:)) !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
               fluxtot_flood(:,k,icarbon,l) =fbact_ncarb(:,l,m,k) * carbon_32l(:,k,m,l) * &
                   flood_frac(:) * un/trois !! control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *

            ! C3 crop
       ELSEWHERE ((soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero) .AND.( (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) ))
               fluxtot(:,k,icarbon,l) =fbact_ncarb(:,l,m,k) * carbon_32l(:,k,m,l) * &
                     flux_tot_coeff(1) * (un-flood_frac(:)) !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
               fluxtot_flood(:,k,icarbon,l) =fbact_ncarb(:,l,m,k)* carbon_32l(:,k,m,l) * &
                     flux_tot_coeff(1) * flood_frac(:) * un/trois !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
            ! C4 crop
       ELSEWHERE ((soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero) .AND.( (.NOT. natural(m)) .AND. (is_c4(m)) ))
               fluxtot(:,k,icarbon,l) =fbact_ncarb(:,l,m,k)* carbon_32l(:,k,m,l) * &
                     flux_tot_coeff(2) * (un-flood_frac(:)) !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
               fluxtot_flood(:,k,icarbon,l) =fbact_ncarb(:,l,m,k) * carbon_32l(:,k,m,l) * &
                    flux_tot_coeff(2) * flood_frac(:) * un/trois !!!control_moist_below(:,l,m) * control_temp_below(:,l,k+4,m) *
       ELSEWHERE
               fluxtot(:,k,icarbon,l) = zero
               fluxtot_flood(:,k,icarbon,l) = zero
       ENDWHERE
 ENDIF !!! priming effect
	!IF (maxval(fluxtot(:,k,icarbon,l)) .GT. 1000) THEN
!		IF (minval(fluxtot(:,k,icarbon,l)) .LT. zero) THEN  
!
!	WRITE(numout,*) ' maxvalfluxtot(:,k,icarbon,l) =', maxval(fluxtot(:,k,icarbon,l))	
!	WRITE(numout,*) ' maxval carbon_32l(:,k,m,l) =', maxval(carbon_32l(:,k,m,l))	
!	WRITE(numout,*) ' max fbact_ncarb(:,l,m,k) =', maxval(fbact_ncarb(:,l,m,k))
!	WRITE(numout,*) ' maxval LOM(:,m,l) =', maxval(LOM(:,m,l))	
!
!	WRITE(numout,*) ' minvalfluxtot(:,k,icarbon,l) =', minval(fluxtot(:,k,icarbon,l))	
!	WRITE(numout,*) ' minval carbon_32l(:,k,m,l) =', minval(carbon_32l(:,k,m,l))	
!	WRITE(numout,*) ' minval fbact_ncarb(:,l,m,k) =', minval(fbact_ncarb(:,l,m,k))	
!	WRITE(numout,*) ' minval LOM(:,m,l) =', minval(LOM(:,m,l))	
!
!	WRITE(numout,*) 'k,m,l = ', k,m,l
!
!	ENDIF
	
!!!SIMON END
          ! END - S.L. Piao 2006/05/05 - for crop multiply tillage factor of decomposition

!!!SIMON ADD ORCHIDOC -CHECK IF COMMENT BELOW IS VALID OR NEEDS ADAPTATION (above/below(carbon/carbon_32l) 
!   !       IF ( natural(m) ) THEN
!   !          fluxtot(:,k,icarbon) = dt/carbon_tau(k) * carbon(:,k,m) * &
!   !                control_moist(:,ibelow) * control_temp(:,ibelow)
!   !       ! C3 crop
!   !        ELSEIF ( (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) ) THEN
!   !           fluxtot(:,k,icarbon) = dt/carbon_tau(k) * carbon(:,k,m) * &
!   !                control_moist(:,ibelow) * control_temp(:,ibelow) * flux_tot_coeff(1)
!   !        ! C4 Crop   
!   !        ELSEIF ( (.NOT. natural(m)) .AND. is_c4(m) ) THEN
!   !           fluxtot(:,k,icarbon) = dt/carbon_tau(k) * carbon(:,k,m) * &
!   !                control_moist(:,ibelow) * control_temp(:,ibelow) * flux_tot_coeff(2)
!   !      ENDIF
    !    ! END - S.L. Piao 2006/05/05 - for crop multiply tillage factor of decomposition

        ! Carbon flux from active pools depends on clay content
        IF ( k .EQ. iactive ) THEN
              fluxtot(:,k,icarbon,l) = fluxtot(:,k,icarbon,l) * ( un - flux_tot_coeff(3) * clay(:) )
              fluxtot_flood(:,k,icarbon,l) = fluxtot_flood(:,k,icarbon,l) * ( un - flux_tot_coeff(3) * clay(:) )
        ENDIF
!     write (numout,*) 'soilcarbon L1349'


      ENDDO ! End of loop over soil layers

    ENDDO ! End of loop over carbon pools
      
      !! 2.3.2 respiration
      !! 2.3.2 respiration
      !BE CAREFUL: Here resp_hetero_soil is divided by dt to have a value which corresponds to
      ! the sechiba time step but then in stomate.f90 resp_hetero_soil is multiplied by dt.
      ! Perhaps it could be simplified. Moreover, we must totally adapt the routines to the dtradia/one_day
      ! time step and avoid some constructions that could create bug during future developments.
      !
      !       VPP killer:
      !       resp_hetero_soil(:,m) = SUM( frac_resp(:,:) * fluxtot(:,:), DIM=2 ) / dt
!!! SIMON ADD ORCHIDOC
      DO l = 1,ndeep !Loop over soil layers
         resp_hetero_soil(:,m) = resp_hetero_soil(:,m)+&
            ( (un-CUE_coef(:)) * fluxtot(:,iactive,icarbon,l) + &
            (un-CUE_coef(:)) * fluxtot(:,islow,icarbon,l) + &
            (un-CUE_coef(:))* fluxtot(:,ipassive,icarbon,l)  ) / dt
         resp_flood_soil(:,m) = resp_flood_soil(:,m)+&
            ( (un-CUE_coef(:)) * fluxtot_flood(:,iactive,icarbon,l) + &
            (un-CUE_coef(:)) * fluxtot_flood(:,islow,icarbon,l) + &
            (un-CUE_coef(:))* fluxtot_flood(:,ipassive,icarbon,l)  ) / dt
!	IF (minval(resp_hetero_soil(:,m)) .LT. zero) THEN  
 !write (numout, *) '1372 fluxtot(:,iactive,icarbon,l) maxval=', maxval(fluxtot(:,iactive,icarbon,l))
 !write (numout, *) '1373 fluxtot(:,islow,icarbon,l) maxval=', maxval(fluxtot(:,islow,icarbon,l))
 !write (numout, *) '1374 fluxtot(:,ipassive,icarbon,l) maxval=', maxval(fluxtot(:,ipassive,icarbon,l))
!write (numout, *) '1372 fluxtot(:,iactive,icarbon,l) minval=', minval(fluxtot(:,iactive,icarbon,l))
! write (numout, *) '1373 fluxtot(:,islow,icarbon,l) minval=', minval(fluxtot(:,islow,icarbon,l))
! write (numout, *) '1374 fluxtot(:,ipassive,icarbon,l) minval=', minval(fluxtot(:,ipassive,icarbon,l))
 
! write (numout, *) '1375 resp_flood_soil(:,m) maxval=', maxval(resp_flood_soil(:,m))
! write (numout, *) '1376 resp_hetero_soil(:,m) maxval=', maxval(resp_hetero_soil(:,m))
! write (numout, *) '1376 CUE_coef(:) maxval=', maxval(CUE_coef(:))
! write (numout, *) '1375 resp_flood_soil(:,m) minval=', minval(resp_flood_soil(:,m))
! write (numout, *) '1376 resp_hetero_soil(:,m) minval=', minval(resp_hetero_soil(:,m))
! write (numout, *) '1376 CUE_coef(:) minval=', minval(CUE_coef(:))
! 
! write (numout, *) '1376 m,l', m,l
!
!	END IF
      ENDDO !End loop over soil layers

!       resp_hetero_soil(:,m) = &
!          ( frac_resp(:,iactive) * fluxtot(:,iactive,icarbon) + &
!          frac_resp(:,islow) * fluxtot(:,islow,icarbon) + &
!          frac_resp(:,ipassive) * fluxtot(:,ipassive,icarbon)  ) / dt
!gmjc
!       resp_hetero_soil_part(:,iactive,m) = &
!            frac_resp(:,iactive) * fluxtot(:,iactive,icarbon)/dt
!       resp_hetero_soil_part(:,islow,m) = &
!            frac_resp(:,islow) * fluxtot(:,islow,icarbon)/dt
!       resp_hetero_soil_part(:,ipassive,m) = &
!            frac_resp(:,ipassive) * fluxtot(:,ipassive,icarbon)/dt
!end gmjc      
      !! 3.2.3 add fluxes to active, slow, and passive pools
      !       VPP killer:
      !       carbon(:,:,m) = carbon(:,:,m) + SUM( flux(:,:,:), DIM=2 )
!
!
!!!SIMON ADD ORCHIDOC
!!!%%%%%%%%%%%%%%%-------LARGE ORCHIDOC ADDITION HERE-------%%%%%%%%%%%%%!!!
!!!
      !! 2.3.3 DOC calculation
      !! 2.3.3.1 DOC decomposition
      ! In the Kalbitz et al., 2003 Geoderma as well
      ! as in Qualls and Haines 1992 SSAJ
      ! the equation are defined with exponential but for the time
      ! step we used we can approximate the values with 1st order kinetics.
!  
!    DO i = 1,npts !!This one for DOC_tau, to loop over all its dimensions
     DO j = 1,npool  !Loop over pools
       DO l = 1,ndeep !Loop over soil layers
        WHERE (soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero) 
                 fluxtot_DOC(:,m,l,j) = fbact_npool(:,l,m,j)* (un - flood_frac(:)) * &
                                         DOC(:,m,l,ifree,j,icarbon)	!!*control_temp_below(:,l,npool+j,m) 
                 fluxtot_DOC_flood(:,m,l,j) = fbact_npool(:,l,m,j)* flood_frac(:) * un/trois * &
                                         DOC(:,m,l,ifree,j,icarbon) 	!!*control_temp_below(:,l,npool+j,m)
         ELSEWHERE
                 fluxtot_DOC(:,m,l,j) = zero
                 fluxtot_DOC_flood(:,m,l,j) = zero
         ENDWHERE
          resp_hetero_soil(:,m) = resp_hetero_soil(:,m)+&
             (un-CUE_coef(:))*fluxtot_DOC(:,m,l,j)/dt
!
          resp_flood_soil(:,m) = resp_flood_soil(:,m)+&
             (un-CUE_coef(:))*fluxtot_DOC_flood(:,m,l,j)/dt
      ENDDO !End loop over soil layers!
     ENDDO !End loop over soil pool!
!   ENDDO !i=npts
    
      !! 2.3.4 Pools update
      !! 2.3.4.1 For POC
      DO l= 1, ndeep ! Loop over soil layers
!write (numout, *) 'soilcarbon L1424'
       carbon_32l(:,iactive,m,l) = carbon_32l(:,iactive,m,l)+ &
                               frac_carb(:,ipassive,iactive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,ipas) + &
                               frac_carb(:,islow,iactive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,islo) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,imetbel) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,istrbel) * (un - lignin_struc_below(:,m,l)) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,imetabo) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,istrabo) * (un - lignin_struc_above(:,m)) + &
                               frac_carb(:,ipassive,iactive)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,ipas) + &
                               frac_carb(:,islow,iactive)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,islo) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,imetbel) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,istrbel) * (un - lignin_struc_below(:,m,l)) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,imetabo) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,istrabo) * (un - lignin_struc_above(:,m))
!WRITE (numout, *) 'Flux Slow DOC to Active SOC =', (frac_carb(:,islow,iactive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,islo)) 
!WRITE (numout, *) 'Flux Passive DOC to Active SOC =', (frac_carb(:,ipassive,iactive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,ipas)) 

       carbon_32l(:,islow,m,l) = carbon_32l(:,islow,m,l)+ &
                               frac_carb(:,ipassive,islow)*(CUE_coef(:))*fluxtot_DOC(:,m,l,ipas) + &
                               frac_carb(:,iactive,islow)*(CUE_coef(:))*fluxtot_DOC(:,m,l,iact) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,istrbel) *lignin_struc_below(:,m,l) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,istrabo) *lignin_struc_above(:,m) + &
                               frac_carb(:,ipassive,islow)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,ipas) + &
                               frac_carb(:,iactive,islow)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,iact) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,istrbel) *lignin_struc_below(:,m,l) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,istrabo) *lignin_struc_above(:,m)

			In_Slow(:,m,l) = frac_carb(:,ipassive,islow)*(CUE_coef(:))*fluxtot_DOC(:,m,l,ipas) + &
                               frac_carb(:,iactive,islow)*(CUE_coef(:))*fluxtot_DOC(:,m,l,iact) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,istrbel) *lignin_struc_below(:,m,l) + &
                               (CUE_coef(:))*fluxtot_DOC(:,m,l,istrabo) *lignin_struc_above(:,m) + &
                               frac_carb(:,ipassive,islow)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,ipas) + &
                               frac_carb(:,iactive,islow)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,iact) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,istrbel) *lignin_struc_below(:,m,l) + &
                               (CUE_coef(:))*fluxtot_DOC_flood(:,m,l,istrabo) *lignin_struc_above(:,m)

!WRITE (numout, *) 'Flux Active DOC to Slow SOC =', (frac_carb(:,iactive,islow)*(CUE_coef(:))*fluxtot_DOC(:,m,l,iact)) 
!
!write (numout, *) '1449 carbon_32l(:,islow,m,l) maxval=', maxval(carbon_32l(:,islow,m,l))

       carbon_32l(:,ipassive,m,l) = carbon_32l(:,ipassive,m,l)+ &
                               frac_carb(:,iactive,ipassive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,iact) + &
                               frac_carb(:,islow,ipassive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,islo) + &
                               frac_carb(:,iactive,ipassive)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,iact) + &
                               frac_carb(:,islow,ipassive)*(CUE_coef(:))*fluxtot_DOC_flood(:,m,l,islo)

!WRITE (numout, *) 'Flux Slow DOC to Passive SOC =', (frac_carb(:,islow,ipassive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,islo))
!WRITE (numout, *) 'Flux Active DOC to Passive SOC =', (frac_carb(:,iactive,ipassive)*(CUE_coef(:))*fluxtot_DOC(:,m,l,iact))



!	IF (minval(carbon_32l) .LT. zero) THEN
!	WRITE (numout,*) 'BOOM after fluxtot_DOC line 1456'
!	write (numout, *) '1456 carbon_32l(:,ipassive,m,l) maxval=', maxval(carbon_32l(:,ipassive,m,l))
!	write (numout, *) 'fluxtot_DOC maxval=', maxval(fluxtot_DOC(:,m,l,:))
!	write (numout, *) 'fluxtot_DOC minval=', minval(fluxtot_DOC(:,m,l,:))
!	write (numout, *) 'lignin_struc_below(:,m,l) maxval=', maxval(lignin_struc_below(:,m,l))
!	write (numout, *) 'lignin_struc_above(:,m maxval=', maxval(lignin_struc_above(:,m))
!	write (numout, *) 'max/min frac_carb', maxval(frac_carb),minval(frac_carb)
!	ENDIF      
      ENDDO ! End loop over soil layers


      DO k = 1, ncarb ! Loop over carbon pools from which the flux comes
        DO l = 1, ndeep !Loop over soil layers

         carbon_32l(:,k,m,l) = carbon_32l(:,k,m,l) - fluxtot(:,k,icarbon,l) - fluxtot_flood(:,k,icarbon,l)
!IF (minval(carbon_32l) .LT. zero) write (numout, *) 'BOOM after fluxtot subtract' 

        ENDDO ! End of loop over soil layers
      ENDDO ! End of loop over carbon pools

      !! 2.3.4.2 For DOC
      DO l = 1,ndeep !Loop over soil layers
         DO kk = 1,nelements !Loop over elements
           IF (l .GT. sro_bottom) THEN
           !For Active pool
              DOC(:,m,l,ifree,iact,kk) = DOC(:,m,l,ifree,iact,kk) + (CUE_coef(:)) * fluxtot(:,iactive,kk,l) + &
                   (CUE_coef(:)) * fluxtot_flood(:,iactive,kk,l)

              !For Slow pool    
              DOC(:,m,l,ifree,islo,kk) = DOC(:,m,l,ifree,islo,kk) + (CUE_coef(:)) * fluxtot(:,islow,kk,l) + &
                   (CUE_coef(:)) * fluxtot_flood(:,islow,kk,l)

              !For Passive pool
              DOC(:,m,l,ifree,ipas,kk) = DOC(:,m,l,ifree,ipas,kk) + (CUE_coef(:)) * fluxtot(:,ipassive,kk,l) + &
                   (CUE_coef(:)) * fluxtot_flood(:,ipassive,kk,l)
!	write (numout,*) 'stomate_soilcarbon L961 DOC total maxval = ', maxval(DOC)
           ELSE
              !For Active pool
           DOC(:,m,l,ifree,iact,kk) = DOC(:,m,l,ifree,iact,kk) + (CUE_coef(:)) * fluxtot(:,iactive,kk,l)

           !For Slow pool    

           DOC(:,m,l,ifree,islo,kk) = DOC(:,m,l,ifree,islo,kk) + (CUE_coef(:)) * fluxtot(:,islow,kk,l)

           !For Passive pool

           DOC(:,m,l,ifree,ipas,kk) = DOC(:,m,l,ifree,ipas,kk) + (CUE_coef(:)) * fluxtot(:,ipassive,kk,l)
!	write (numout,*) 'stomate_soilcarbon L1505 DOC total maxval,minval = ', maxval(DOC),minval(DOC)
           ENDIF
      ENDDO !End loop over elements
    ENDDO !End loop over soil layers
!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after fluxtot add'      
!ENDIF

     DO j = 1,npool  !Loop over pools
       DO l = 1,ndeep !Loop over soil layers

          DOC(:,m,l,ifree,j,icarbon) = DOC(:,m,l,ifree,j,icarbon) - fluxtot_DOC(:,m,l,j) - fluxtot_DOC_flood(:,m,l,j)
!write (numout,*) 'stomate_soilcarbon L1507 DOC total maxval,minval = ', maxval(DOC),minval(DOC)

       ENDDO !End loop over soil layers!
     ENDDO !End loop over soil pool!
!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after fluxtot_DOC subtr'      
!ENDIF
      !! 2.3.5 Adsorption/resorption of DOC using linear Mass isotherm assuming
      !that DOC from native soil is zero (b parameters in Nodvin et al., 1986,
      !Soil Science)
      !For the moment, fixed parameters taken from Kaiser and others,1996, b in mg/g soil
     Kd(:,:)=kd_ads
     !IM(:,)=m_ads
     !b(:,:)=b_ads
!write (numout,*) 'soilcarbon L1519'
     Kd(:,1) = dix**(-3.1+0.2*log10(clay(:)*cent)+log10(z_soil(1)*cent/deux))
     DO l = 2,nbdl
        ! IM, b or Kd calculating following the statistical model obtained in
        ! Camino Serrano et al. in prep
        Kd(:,l) = dix**(-3.1+0.2*log10(clay(:)*cent)+log10((z_soil(l)+z_soil(l-1))/deux))
        ! Kd(:,l) = 0.001225841-0.000212346*soil_ph(:)+0.003739918*clay(:)
     ENDDO
     Kd(:,:)=MAX(Kd(:,:),dix**(-3.2))
     Kd(:,:)=MIN(Kd(:,:),dix**(-1.5))
!write (numout,*) 'soilcarbon L1529'

     ! Calculation of amount adsorbed/desorbed based on 
     ! Ota et al., 2013 JGR
     DO l = 1,ndeep !Loop over soil layers
        DO kk = 1,nelements !Loop over elements
           DO kkk = 1,npool !Loop over soil and litter pool
              DOC_RE(:,m,l,kkk,kk) = (Kd(:,l)*bulk_dens(:))/(Kd(:,l)*bulk_dens(:)+un)* &
                   (DOC(:,m,l,ifree,kkk,kk)+DOC(:,m,l,iadsorbed,kkk,kk)) 
           ENDDO !End loop over soil and litter pool
        ENDDO !End loop over elements
     ENDDO !End loop over soil layers
!!write (numout,*) 'stomate_soilcarbon L0116 after adsorbed DOC calc DOC maxval = ', maxval(DOC)

     DOC_old(:,m,:,:,:,:)= zero
     DOC_old(:,m,:,:,:,:)=DOC(:,m,:,:,:,:)
     DO l = 1,ndeep !Loop over soil layers
        DO kk = 1,nelements !Loop over elements
           DO kkk = 1,npool !Loop over soil and litter pool
              WHERE (((DOC_RE(:,m,l,kkk,kk)-DOC_old(:,m,l,iadsorbed,kkk,kk)) .GT. zero) .AND. (abs(DOC_RE(:,m,l,kkk,kk)-DOC_old(:,m,l,iadsorbed,kkk,kk)) .GT. DOC_old(:,m,l,ifree,kkk,kk)))
                 !To avoid adsorbing more DOC than existing (when there is not enough free carbon)
                 DOC(:,m,l,iadsorbed,kkk,kk)= DOC_old(:,m,l,iadsorbed,kkk,kk)+ DOC_old(:,m,l,ifree,kkk,kk)
                 DOC(:,m,l,ifree,kkk,kk)= zero           
              ELSEWHERE (((DOC_RE(:,m,l,kkk,kk)-DOC_old(:,m,l,iadsorbed,kkk,kk)) .LT. zero) .AND. (abs(DOC_RE(:,m,l,kkk,kk)-DOC_old(:,m,l,iadsorbed,kkk,kk)) .GT. DOC_old(:,m,l,iadsorbed,kkk,kk)))
               !To avoid desorbing more DOC than existing (when there is not enough adsorbed carbon)
                 DOC(:,m,l,iadsorbed,kkk,kk)= zero 
                 DOC(:,m,l,ifree,kkk,kk)= DOC_old(:,m,l,iadsorbed,kkk,kk)+ DOC_old(:,m,l,ifree,kkk,kk)        
              ELSEWHERE ((DOC_RE(:,m,l,kkk,kk)-DOC_old(:,m,l,iadsorbed,kkk,kk)) .NE. zero) 
                !The normal functioning of the distribution coefficient to reach equilibrium between DOC adsorbed and DOC free   
                 DOC(:,m,l,ifree,kkk,kk)=DOC_old(:,m,l,ifree,kkk,kk)-(DOC_RE(:,m,l,kkk,kk)-DOC_old(:,m,l,iadsorbed,kkk,kk))  
                 DOC(:,m,l,iadsorbed,kkk,kk)= DOC_old(:,m,l,iadsorbed,kkk,kk)+(DOC_RE(:,m,l,kkk,kk)-DOC_old(:,m,l,iadsorbed,kkk,kk))
              ELSEWHERE
                 DOC(:,m,l,ifree,kkk,kk)=DOC_old(:,m,l,ifree,kkk,kk)
                 DOC(:,m,l,iadsorbed,kkk,kk)= DOC_old(:,m,l,iadsorbed,kkk,kk)
              ENDWHERE
           ENDDO !End loop over soil and litter pool
        ENDDO !End loop over elements
     ENDDO !End loop over soil layers
!
!
!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after adsorp/resorb'      
!ENDIF
!	write (numout,*) 'stomate_soilcarbon L1575 DOC  maxval = ', maxval(DOC)

     !! 2.3.6 Transport of DOC with the water fluxes following Futter et al.,
     ! (2007) Water ressources research. 
     
     !! 2.3.6.1 Change DOC free from gC m-2 of soil to g C m^-3 of water  
     DO kk = 1, nelements !Loop over elements
        
        DO k = 1, npool ! Loop over soil and litter pools
           
           WHERE( soil_mc(:,1,pref_soil_veg(m)) .GT. zero)

              DOC(:,m,1,ifree,k,kk) = DOC(:,m,1,ifree,k,kk) / (zf_soil_B(1)*soil_mc(:,1,pref_soil_veg(m)))

           ELSEWHERE
              DOC(:,m,1,ifree,k,kk) = DOC(:,m,1,ifree,k,kk)
           ENDWHERE
!write (numout,*) 'stomate_soilcarbon L1593 DOC total maxval = ', maxval(DOC)

        ENDDO ! End loop over soil and litter pools

     ENDDO !End loop over elements

!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after stocks to conc calc'      
!ENDIF

     DO l = 2,ndeep !Loop over soil layers

        DO kk = 1, nelements !Loop over elements

           DO k = 1, npool ! Loop over soil and litter pools

              WHERE( soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero)

                 DOC(:,m,l,ifree,k,kk) = DOC(:,m,l,ifree,k,kk) / ((zf_soil_B(l)-zf_soil_B(l-1))*soil_mc_32l(:,l,pref_soil_veg(m)))

              ELSEWHERE
                 DOC(:,m,l,ifree,k,kk) = DOC(:,m,l,ifree,k,kk)
              ENDWHERE
!write (numout,*) 'stomate_soilcarbon L1080 DOC total maxval = ', maxval(DOC)

           ENDDO ! End loop over soil and litter pools

        ENDDO !End loop over elements

     ENDDO !End loop over soil layers 
     
     ! DOC fluxes between layers calculated (we multiplied by
     ! 1E-3 to convert from kg/m2 to m3/m2)
    
   !!!SIMON CHECK bloew is consistent(ndeep)
   
     DO l = 1,nbdl-1 !Loop over soil layers
        DO kk = 1,nelements !Loop over elements
           DO kkk = 1,npool !Loop over soil and litter pool
              WHERE ((wat_flux(:,l,pref_soil_veg(m)) .GT. 0.) .AND. (soil_mc(:,l,pref_soil_veg(m)) .GT. zero))
                 DOC_FLUX(:,m,l,kkk,kk) = DOC(:,m,l,ifree,kkk,kk)*wat_flux(:,l,pref_soil_veg(m))*1E-3*flux_red(:)
              ELSEWHERE ((wat_flux(:,l,pref_soil_veg(m)) .LT. 0.) .AND. (soil_mc(:,l,pref_soil_veg(m)) .GT. zero))
                 DOC_FLUX(:,m,l,kkk,kk) = DOC(:,m,l+1,ifree,kkk,kk)*abs(wat_flux(:,l,pref_soil_veg(m)))*1E-3*flux_red(:)
              ELSEWHERE 
                 DOC_FLUX(:,m,l,kkk,kk) = zero
              ENDWHERE
           ENDDO !End loop over soil and litter pool
        ENDDO !End loop over elements
     ENDDO !End loop over soil layers
!write (numout,*) 'stomate_soilcarbon L1637 DOC total maxval,minval = ', maxval(DOC),minval(DOC)

     !For the last layer
     DO kk = 1,nelements !Loop over elements
        DO kkk = 1,npool !Loop over soil and litter pool
           WHERE ((wat_flux(:,nbdl,pref_soil_veg(m)) .GT. 0.) .AND. (soil_mc(:,nbdl,pref_soil_veg(m)) .GT. zero))
              DOC_FLUX(:,m,nbdl,kkk,kk) = DOC(:,m,nbdl,ifree,kkk,kk)*wat_flux(:,nbdl,pref_soil_veg(m))*1E-3*flux_red(:)
           ELSEWHERE 
              DOC_FLUX(:,m,nbdl,kkk,kk) = zero
           ENDWHERE
        ENDDO !End loop over soil and litter pool
     ENDDO !End loop over elements


!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after advection calc'      
!ENDIF

     !! 2.3.7 DOC is reconverted from gC m^⁻3 of water in gC m^-2 of ground to exchange it with the other carbon pools
     !For the first layer

     DO k = 1, npool ! Loop over soil and litter pools
        DO kk = 1,nelements !Loop over elements
           WHERE(soil_mc_32l(:,1,pref_soil_veg(m)) .GT. zero)              
              DOC(:,m,1,ifree,k,kk)=DOC(:,m,1,ifree,k,kk) * soil_mc_32l(:,1,pref_soil_veg(m)) * zf_soil_B(1)
           ELSEWHERE
              DOC(:,m,1,ifree,k,kk)=DOC(:,m,1,ifree,k,kk) 
           ENDWHERE
        ENDDO !End loop over elements
     ENDDO ! End loop over soil and litter pools
!write (numout,*) 'stomate_soilcarbon L1662 DOC total maxval,minval = ', maxval(DOC),minval(DOC)

     !For the other layers

     DO l= 2, ndeep ! Loop over soil layers
        DO k = 1, npool ! Loop over soil and litter pools
           DO kk = 1,nelements !Loop over elements
              WHERE(soil_mc_32l(:,l,pref_soil_veg(m)) .GT. zero)
                 DOC(:,m,l,ifree,k,kk)=DOC(:,m,l,ifree,k,kk) * soil_mc_32l(:,l,pref_soil_veg(m)) * (zf_soil_B(l)-zf_soil_B(l-1))
              ELSEWHERE
                 DOC(:,m,l,ifree,k,kk)=DOC(:,m,l,ifree,k,kk) 
              ENDWHERE
           ENDDO !End loop over elements
        ENDDO ! End loop over soil and litter pools
     ENDDO ! End loop over soil layers
!write (numout,*) 'stomate_soilcarbon L1677 DOC total maxval,minval = ', maxval(DOC),minval(DOC)
!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after  conc back to stock calc'      
!ENDIF
     ! DOC update
     !For the first layer
     DOC_old(:,m,:,:,:,:)= zero
     DOC_old(:,m,:,:,:,:)=DOC(:,m,:,:,:,:)

     DO kk = 1,nelements !Loop over elements
        DO kkk = 1,npool !Loop over soil and litter pool
              
           WHERE (wat_flux(:,1,pref_soil_veg(m)) .GT. 0.) 
              DOC(:,m,1,ifree,kkk,kk) = DOC_old(:,m,1,ifree,kkk,kk) - MIN(DOC_old(:,m,1,ifree,kkk,kk),DOC_FLUX(:,m,1,kkk,kk)) 
           ELSEWHERE (wat_flux(:,1,pref_soil_veg(m)) .LT. 0.) 
              DOC(:,m,1,ifree,kkk,kk) = DOC_old(:,m,1,ifree,kkk,kk) + MIN(DOC_old(:,m,2,ifree,kkk,kk),DOC_FLUX(:,m,1,kkk,kk))
           ELSEWHERE
              DOC(:,m,1,ifree,kkk,kk) = DOC_old(:,m,1,ifree,kkk,kk)
           ENDWHERE
        ENDDO !End loop over soil and litter pool
     ENDDO !End loop over elements
!write (numout,*) 'stomate_soilcarbon L1696 DOC total maxval,minval = ', maxval(DOC),minval(DOC)

     ! For the other layers
     DO l = 2,nbdl-1 !Loop over soil layers
        DO kk = 1,nelements !Loop over elements
           DO kkk = 1,npool !Loop over soil and litter pool
              WHERE ((wat_flux(:,l,pref_soil_veg(m)) .GT. 0.) .AND. (wat_flux(:,l-1,pref_soil_veg(m)) .GT. 0.))
                 DOC(:,m,l,ifree,kkk,kk) = DOC_old(:,m,l,ifree,kkk,kk) - MIN(DOC_FLUX(:,m,l,kkk,kk),DOC_old(:,m,l,ifree,kkk,kk)) + MIN(DOC_FLUX(:,m,l-1,kkk,kk),DOC_old(:,m,l-1,ifree,kkk,kk))
              ELSEWHERE ((wat_flux(:,l,pref_soil_veg(m)) .LT. 0.) .AND. (wat_flux(:,l-1,pref_soil_veg(m)) .LT. 0.))
                 DOC(:,m,l,ifree,kkk,kk) = DOC_old(:,m,l,ifree,kkk,kk) + MIN(DOC_FLUX(:,m,l,kkk,kk),DOC_old(:,m,l+1,ifree,kkk,kk)) - MIN(DOC_FLUX(:,m,l-1,kkk,kk),DOC_old(:,m,l,ifree,kkk,kk))
              ELSEWHERE ((wat_flux(:,l,pref_soil_veg(m)) .GT. 0.) .AND. (wat_flux(:,l-1,pref_soil_veg(m)) .LT. 0.))
                 DOC(:,m,l,ifree,kkk,kk) = DOC_old(:,m,l,ifree,kkk,kk) - MIN(DOC_FLUX(:,m,l,kkk,kk),DOC_old(:,m,l,ifree,kkk,kk)) - MIN(DOC_FLUX(:,m,l-1,kkk,kk),DOC_old(:,m,l,ifree,kkk,kk))
              ELSEWHERE ((wat_flux(:,l,pref_soil_veg(m)) .LT. 0.) .AND. (wat_flux(:,l-1,pref_soil_veg(m)) .GT. 0.))
                 DOC(:,m,l,ifree,kkk,kk) = DOC_old(:,m,l,ifree,kkk,kk) + MIN(DOC_FLUX(:,m,l,kkk,kk),DOC_old(:,m,l+1,ifree,kkk,kk)) + MIN(DOC_FLUX(:,m,l-1,kkk,kk),DOC_old(:,m,l-1,ifree,kkk,kk))
              ELSEWHERE 
                 DOC(:,m,l,ifree,kkk,kk) = DOC_old(:,m,l,ifree,kkk,kk)
              ENDWHERE
           ENDDO !End loop over soil and litter pool
        ENDDO !End loop over elements
     ENDDO !End loop over soil layers
!write (numout,*) 'stomate_soilcarbon L1716 DOC total maxvalminval = ', maxval(DOC),minval(DOC)

     !For the last layer
     DO kk = 1,nelements !Loop over elements
        DO kkk = 1,npool !Loop over soil and litter pool
           WHERE (wat_flux(:,nbdl-1,pref_soil_veg(m)) .GT. 0.)
              DOC(:,m,nbdl,ifree,kkk,kk) = DOC_old(:,m,nbdl,ifree,kkk,kk) + MIN(DOC_FLUX(:,m,nbdl-1,kkk,kk),DOC_old(:,m,nbdl-1,ifree,kkk,kk))
           ELSEWHERE (wat_flux(:,nbdl-1,pref_soil_veg(m)) .LT. 0.)
              DOC(:,m,nbdl,ifree,kkk,kk) = DOC_old(:,m,nbdl,ifree,kkk,kk) - MIN(DOC_FLUX(:,m,nbdl-1,kkk,kk),DOC_old(:,m,nbdl,ifree,kkk,kk))
           ELSEWHERE
              DOC(:,m,nbdl,ifree,kkk,kk) = DOC_old(:,m,nbdl,ifree,kkk,kk)
           ENDWHERE
        ENDDO !End loop over soil and litter pool
     ENDDO !End loop over elements
!write (numout,*) 'stomate_soilcarbon L1730 DOC total maxval,minval = ', maxval(DOC),minval(DOC)
!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after advection'      
!ENDIF
     !! 2.3.8 DOC transport by diffusion
     !! 2.3.8.1 Calculation of the DOC fluxes by diffusion
     ! Here we use the DOC concentration in the soil and not in water to
     ! stabilize the calcul of
     ! water distribution is continuous in the soil column (no dry places)

     DOC_old2(:,m,:,:,:,:)=zero
     DOC_old_buffer(:,m,:,:,:,:)=zero
     DOC_old2(:,m,:,:,:,:)=DOC(:,m,:,:,:,:)


     DO kkk = 1, npool ! Loop over soil and litter pools
        DO kk = 1, nelements ! Loop over elements	
 !!!SIMON ADDED temp threshold on DOC diffusion at frozen_resp_func CASE (1) of -1degC for 'respiration' cutoff i.e. DOC activity below this.   
	   WHERE ((tprof(:,1,m) .GT. 272.15) .AND. (tprof(:,2,m) .GT. 272.15))
           DOC_FLUX_DIFF(:,m,1,kkk,kk) = Dif_DOC(:)*ABS(DOC_old2(:,m,1,ifree,kkk,kk)/(zf_soil_B(1))-DOC_old2(:,m,2,ifree,kkk,kk)/(zf_soil_B(2)-zf_soil_B(1)))/(zf_soil_B(2)-zf_soil_B(1))
!

	   ELSEWHERE 
		DOC_FLUX_DIFF(:,m,1,kkk,kk) = zero
	   ENDWHERE
!
           DO l= 2, ndeep-1 ! Loop over soil layers
	  	WHERE((tprof(:,l,m) .GT. 272.15) .AND. (tprof(:,l+1,m) .GT. 272.15))
              DOC_FLUX_DIFF(:,m,l,kkk,kk) = Dif_DOC(:)*ABS(DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1))-DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l)))/(zf_soil_B(l+1)-zf_soil_B(l))
           	ELSEWHERE
		   DOC_FLUX_DIFF(:,m,l,kkk,kk) = zero
	   	ENDWHERE

	   ENDDO

!WRITE(numout,*) '1895  max DOC_FLUX_DIFF 12l=', maxval(DOC_FLUX_DIFF(:,:,12,:,:))
!WRITE(numout,*) '1895  max DOC_FLUX_DIFF 11l=', maxval(DOC_FLUX_DIFF(:,:,11,:,:))
!WRITE(numout,*) 'maxval Dif_DOC(:)=', maxval(Dif_DOC(:))
!WRITE(numout,*) '1895 max DOC_old2 11l=', maxval(DOC_old2(:,:,11,:,:,:))
!WRITE(numout,*) '1895 max DOC_old2 12l=', maxval(DOC_old2(:,:,12,:,:,:))


       !Below we checked if in case that, in a given layer, you have diffusion
       !in the above and below litters, both fluxes are not higher than the
       !stocks of the given layer.
           DO l =1, ndeep-2
              WHERE (DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1)) .LT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l)) .AND. &
                   DOC_old2(:,m,l+2,ifree,kkk,kk)/(zf_soil_B(l+2)-zf_soil_B(l+1)) .LT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l)) .AND. &
                   DOC_FLUX_DIFF(:,m,l,kkk,kk) + DOC_FLUX_DIFF(:,m,l+1,kkk,kk) .GT. DOC_old2(:,m,l+1,ifree,kkk,kk))
                 DOC_FLUX_DIFF(:,m,l,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk)*(DOC_FLUX_DIFF(:,m,l,kkk,kk)/(DOC_FLUX_DIFF(:,m,l,kkk,kk)+DOC_FLUX_DIFF(:,m,l+1,kkk,kk)))
                 DOC_FLUX_DIFF(:,m,l+1,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk)*(DOC_FLUX_DIFF(:,m,l+1,kkk,kk)/(DOC_FLUX_DIFF(:,m,l,kkk,kk)+DOC_FLUX_DIFF(:,m,l+1,kkk,kk)))
              ELSEWHERE (DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1)) .GE. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l)) .AND. &
                   DOC_old2(:,m,l+2,ifree,kkk,kk)/(zf_soil_B(l+2)-zf_soil_B(l+1)) .LT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l)) .AND. &
                   DOC_old2(:,m,l+1,ifree,kkk,kk) + DOC_FLUX_DIFF(:,m,l,kkk,kk) - DOC_FLUX_DIFF(:,m,l+1,kkk,kk) .LE. min_stomate)
                 DOC_FLUX_DIFF(:,m,l,kkk,kk) = DOC_FLUX_DIFF(:,m,l,kkk,kk)
                 DOC_FLUX_DIFF(:,m,l+1,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk) + DOC_FLUX_DIFF(:,m,l,kkk,kk)
              ELSEWHERE (DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1)) .LT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l)) .AND. &
                   DOC_old2(:,m,l+2,ifree,kkk,kk)/(zf_soil_B(l+2)-zf_soil_B(l+1)) .GE. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l)) .AND. &
                   DOC_old2(:,m,l+1,ifree,kkk,kk) - DOC_FLUX_DIFF(:,m,l,kkk,kk) + DOC_FLUX_DIFF(:,m,l+1,kkk,kk) .LE. min_stomate)
                 DOC_FLUX_DIFF(:,m,l,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk) + DOC_FLUX_DIFF(:,m,l+1,kkk,kk)
                 DOC_FLUX_DIFF(:,m,l+1,kkk,kk) = DOC_FLUX_DIFF(:,m,l+1,kkk,kk)
              ELSEWHERE
                 DOC_FLUX_DIFF(:,m,l,kkk,kk) = DOC_FLUX_DIFF(:,m,l,kkk,kk)
                 DOC_FLUX_DIFF(:,m,l+1,kkk,kk) = DOC_FLUX_DIFF(:,m,l+1,kkk,kk)
              ENDWHERE
           ENDDO

!WRITE(numout,*) '1928  max DOC_FLUX_DIFF 12l=', maxval(DOC_FLUX_DIFF(:,:,12,:,:))
!WRITE(numout,*) '1929  max DOC_FLUX_DIFF 11l=', maxval(DOC_FLUX_DIFF(:,:,11,:,:))

                 write (numout,*) 'where loop begins'

           DO l= 1, ndeep-1 ! Loop over soil layers
              WHERE ((DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1)) .LT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l))) .AND. &
                   ((DOC_FLUX_DIFF(:,m,l,kkk,kk) - DOC_old2(:,m,l+1,ifree,kkk,kk)) .GE. min_stomate))
		 DOC(:,m,l,ifree,kkk,kk) = DOC_old2(:,m,l,ifree,kkk,kk) + DOC_old2(:,m,l+1,ifree,kkk,kk) + DOC_old_buffer(:,m,l,ifree,kkk,kk)
                 DOC_old_buffer(:,m,l+1,ifree,kkk,kk) = - DOC_old2(:,m,l+1,ifree,kkk,kk) 
                 DOC(:,m,l+1,ifree,kkk,kk) = zero
              ELSEWHERE ((DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1)) .LT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l))) .AND. &
                   ((DOC_old2(:,m,l+1,ifree,kkk,kk) - DOC_FLUX_DIFF(:,m,l,kkk,kk)) .GT. min_stomate))
		 DOC(:,m,l,ifree,kkk,kk) = DOC_old2(:,m,l,ifree,kkk,kk) + DOC_FLUX_DIFF(:,m,l,kkk,kk) + DOC_old_buffer(:,m,l,ifree,kkk,kk)
                 DOC_old_buffer(:,m,l+1,ifree,kkk,kk) =  - DOC_FLUX_DIFF(:,m,l,kkk,kk)
                 DOC(:,m,l+1,ifree,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk) - DOC_FLUX_DIFF(:,m,l,kkk,kk)
              ELSEWHERE ((DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1)) .GT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l))) .AND. &
                   ((DOC_FLUX_DIFF(:,m,l,kkk,kk) - DOC_old2(:,m,l,ifree,kkk,kk)) .GE. min_stomate))
		 DOC(:,m,l,ifree,kkk,kk) = zero + DOC_old_buffer(:,m,l,ifree,kkk,kk)
                 DOC_old_buffer(:,m,l+1,ifree,kkk,kk) = DOC_old2(:,m,l,ifree,kkk,kk)
                 DOC(:,m,l+1,ifree,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk) + DOC_old2(:,m,l,ifree,kkk,kk)
              ELSEWHERE ((DOC_old2(:,m,l,ifree,kkk,kk)/(zf_soil_B(l)-zf_soil_B(l-1)) .GT. DOC_old2(:,m,l+1,ifree,kkk,kk)/(zf_soil_B(l+1)-zf_soil_B(l))) .AND. &
                   ((DOC_old2(:,m,l,ifree,kkk,kk) - DOC_FLUX_DIFF(:,m,l,kkk,kk)) .GT. min_stomate))
                 DOC(:,m,l,ifree,kkk,kk) = DOC_old2(:,m,l,ifree,kkk,kk) - DOC_FLUX_DIFF(:,m,l,kkk,kk) + DOC_old_buffer(:,m,l,ifree,kkk,kk)
                 DOC_old_buffer(:,m,l+1,ifree,kkk,kk) = DOC_FLUX_DIFF(:,m,l,kkk,kk)
                 DOC(:,m,l+1,ifree,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk) + DOC_FLUX_DIFF(:,m,l,kkk,kk)
              ELSEWHERE
                 DOC(:,m,l,ifree,kkk,kk) = DOC_old2(:,m,l,ifree,kkk,kk)
                 DOC(:,m,l+1,ifree,kkk,kk) = DOC_old2(:,m,l+1,ifree,kkk,kk)
              ENDWHERE
           ENDDO ! End loop over soil layers
        ENDDO ! End loop over elements
     ENDDO ! End loop over DOC pools
!write (numout,*) '1974 AFter DOC where loop update'


!IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after diffusion flux'      
!ENDIF


!IF (maxval(DOC_FLUX_DIFF) .GT. zero) THEN
!write (numout,*) 'L1940 BOOM DOC_FLUX_DIFF greater than zero after diffusion flux', (maxval(DOC_FLUX_DIFF))     
!ENDIF


        !! 2.3.9 DOC export 
        !! Calculate total DOC going out of the system (the factor 1E-3 is
        !! applied to convert runoff, floodout, drainage from kg/m2 to m3/m2) 
        !! We assume that runoff and floodout affects the first layer of soil
        !! and drainage only the last layer. 
        !! Drainage and runoff are given by soil type by hydrol.f90 but not floodout (it is a weighed mean) explaining why 
        !! the calculation is a bit different between DOC_RUN and DOC_FLOOD. Moreover, floodout is computed as evaporation-precipitation,
        !! so it represent an export to the rivers only if floodout is negative explaing why we have to multiply by -un.
        !! Be careful we do not * by dt because it is already done in hydrol.f90

        !! based on fast reservoir, we calculate a reduction factor. The calculation is based on the assumption that there 
        !! head waters that is the main source of DOC to inland waters, while areas more remote from the head waters have a neglibile 
        !! effect on the DOC in streams. We assume that the export from soils to head waters is proportional to the 
        !! square root of the fast reservoir storage. This is consitnet with empirical finding relating stream width to discharge. As we
        !! do not directly consider a stream surface area, we simply assign a reference fast reservoir for which we assume that
        !! all top soils export DOC into the head waters (which can include streamlets). Note that very high runoff appears where swamps
        !! are pesent.

!write (numout,*) 'Entering DOC export DOC total maxval,minval = ', maxval(DOC),minval(DOC)

        fastr_corr(:) = (fastr(:)**undemi)/(fastr_ref**undemi)
        WHERE (fastr_corr(:) .LT. zero)
           fastr_corr(:) = zero
        ELSEWHERE
           fastr_corr(:) = fastr_corr(:)
        ENDWHERE

        !! Based on maximum export concentration, we calculate a correction factor
         soilDOC_corr(:) = zero
         soilDOC_31mm(:) = zero
         DO kk = 1,nelements !Loop over elements
            DO kkk = 1,npool !Loop over soil and litter pool
               soilDOC_31mm(:) = soilDOC_31mm + DOC(:,m,1,ifree,kkk,kk)
               IF (sro_bottom .GT. 1) THEN
                  DO l=2,sro_bottom
                     soilDOC_31mm(:) = soilDOC_31mm(:) + DOC(:,m,l,ifree,kkk,kk)
                  ENDDO !l=2,sro_bottom
               ENDIF !(sro_bottom .GT. 1)
            ENDDO !kkk = 1,npool
         ENDDO !kk = 1,nelements
! write (numout,*) '2032 after DOC correction factor'

         !
         soilDOC_corr(:) = un
         DO pts=1, npts
           IF (soilwater_31mm(pts, pref_soil_veg(m)) > min_stomate) THEN
             IF (soilDOC_31mm(pts)*flux_red(pts)*fastr_corr(pts)/soilwater_31mm(pts,pref_soil_veg(m)) .GT. DOCexp_max) THEN
             
!             WHERE (soilDOC_31mm(:)*flux_red(:)*fastr_corr(:)/soilwater_31mm(:,pref_soil_veg(m)) .GT. DOCexp_max)
                soilDOC_corr(pts) = DOCexp_max/(soilDOC_31mm(pts)*flux_red(pts)*fastr_corr(pts)/soilwater_31mm(pts,pref_soil_veg(m)))
!             ELSEWHERE
!                soilDOC_corr(:) = un
!             ENDWHERE
             ENDIF
           ENDIF
         ENDDO
         soilDOC_corr(:) = soilDOC_corr(:) * flux_red(:) * fastr_corr(:)
         !
!write (numout,*) '20150 before runoff DOC calc'             
         DO kk = 1,nelements !Loop over elements
            DO kkk = 1,npool !Loop over soil and litter pool
!write(numout,*) 'now entering pool number', kkk
               !DOC lost through runoff for the first layer
               WHERE (soilwater_31mm(:,pref_soil_veg(m)) .GT. zero)
                  DOC_RUN(:,m,kkk,kk) = DOC_RUN(:,m,kkk,kk) + MIN(DOC(:,m,1,ifree,kkk,kk),soilDOC_corr(:) &
                       &                                     * (runoff_per_soil(:,pref_soil_veg(m))*1E-3/soilwater_31mm(:,pref_soil_veg(m))) &
                       &              * DOC(:,m,1,ifree,kkk,kk))
               ELSEWHERE
                  DOC_RUN(:,m,kkk,kk) = DOC_RUN(:,m,kkk,kk)
               ENDWHERE
               !
               IF (kkk .EQ. imetabo) THEN
                  DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,imetabo,kk) 
                  DOC_RUN(:,m,imetabo,kk) = zero
               ELSEIF (kkk .EQ. istrabo) THEN                  
                  DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,istrabo,kk) * (un - lignin_struc_above(:,m))        
                  DOC_RUN(:,m,islo,kk) = DOC_RUN(:,m,islo,kk) + DOC_RUN(:,m,istrabo,kk) * lignin_struc_above(:,m) 
                  DOC_RUN(:,m,istrabo,kk) = zero
               ELSEIF (kkk .EQ. istrbel) THEN
                  DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,istrbel,kk) * (un - lignin_struc_below(:,m,1))        
                  DOC_RUN(:,m,islo,kk) = DOC_RUN(:,m,islo,kk) + DOC_RUN(:,m,istrbel,kk) * lignin_struc_below(:,m,1) 
                  DOC_RUN(:,m,istrbel,kk) = zero  
               ELSEIF (kkk .EQ. imetbel) THEN              
                  DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,imetbel,kk) 
                  DOC_RUN(:,m,imetbel,kk) = zero
               ELSE
                  DOC_RUN(:,m,kkk,kk) = DOC_RUN(:,m,kkk,kk)
               ENDIF
               !
               !And for the layers 2-? layers
               IF (sro_bottom .GT. 1) THEN
                  DO l=2,sro_bottom
                     WHERE (soilwater_31mm(:,pref_soil_veg(m)) .GT. zero)
                        DOC_RUN(:,m,kkk,kk) = DOC_RUN(:,m,kkk,kk) + MIN(DOC(:,m,l,ifree,kkk,kk), &
                             &      soilDOC_corr(:) * (runoff_per_soil(:,pref_soil_veg(m))*1E-3/soilwater_31mm(:,pref_soil_veg(m))) &
                             &    * DOC(:,m,l,ifree,kkk,kk))
                     ELSEWHERE
                        DOC_RUN(:,m,kkk,kk) = DOC_RUN(:,m,kkk,kk)
                     ENDWHERE
                     !
                     IF (kkk .EQ. imetabo) THEN
                        DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,imetabo,kk) 
                        DOC_RUN(:,m,imetabo,kk) = zero
                     ELSEIF (kkk .EQ. istrabo) THEN                  
                        DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,istrabo,kk) * (un - lignin_struc_above(:,m))        
                        DOC_RUN(:,m,islo,kk) = DOC_RUN(:,m,islo,kk) + DOC_RUN(:,m,istrabo,kk) * lignin_struc_above(:,m) 
                        DOC_RUN(:,m,istrabo,kk) = zero
                     ELSEIF (kkk .EQ. istrbel) THEN
                        DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,istrbel,kk) * (un - lignin_struc_below(:,m,l))        
                        DOC_RUN(:,m,islo,kk) = DOC_RUN(:,m,islo,kk) + DOC_RUN(:,m,istrbel,kk) * lignin_struc_below(:,m,l) 
                        DOC_RUN(:,m,istrbel,kk) = zero  
                     ELSEIF (kkk .EQ. imetbel) THEN              
                        DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk) + DOC_RUN(:,m,imetbel,kk) 
                        DOC_RUN(:,m,imetbel,kk) = zero
                     ELSE
                        DOC_RUN(:,m,kkk,kk) = DOC_RUN(:,m,kkk,kk)
                     ENDIF
                  ENDDO
               ENDIF !sro_bottom .GT. 1

!write (numout,*) 'Just before floodplain DOC calc'                     !
               !Now the floodplains and streams
               !
              ! DO l=1,sro_bottom
              !    DOC_FLOOD(:,m,kkk,kk) = DOC_FLOOD(:,m,kkk,kk) + flux_red_sro * DOC(:,m,l,ifree,kkk,kk) * flood_frac(:)
              ! ENDDO !l=1,sro_bottom
               !
!write(numout,*) 'for pool number ', kkk,' DOC(:,m,nbdl,ifree,kkk,kk) ', DOC(:,m,nbdl,ifree,kkk,kk)
!write(numout,*) 'Drainage is ', drainage_per_soil(:,pref_soil_veg(m))	       
!
!
               WHERE (soil_mc(:,nbdl,pref_soil_veg(m)) .GT. zero)
                  DOC_DRAIN(:,m,kkk,kk) = DOC_DRAIN(:,m,kkk,kk) + MIN(DOC(:,m,nbdl,ifree,kkk,kk),&
                       &                                         DOC(:,m,nbdl,ifree,kkk,kk)*drainage_per_soil(:,pref_soil_veg(m))*1E-3 &
                       &                                         /((zf_soil_B(nbdl)-zf_soil_B(nbdl-1)) * soil_mc_32l(:,nbdl,pref_soil_veg(m))))
               ELSEWHERE
                  DOC_DRAIN(:,m,kkk,kk) = DOC_DRAIN(:,m,kkk,kk)
               ENDWHERE
               !
!write(numout,*) 'lignin_struc_above(:,m) =', lignin_struc_above(:,m)
!write(numout,*) 'lignin_struc_below(:,m,nbdl) =', lignin_struc_below(:,m,nbdl)
               IF (kkk .EQ. imetabo) THEN
                  DOC_DRAIN(:,m,iact,kk) = DOC_DRAIN(:,m,iact,kk) + DOC_DRAIN(:,m,imetabo,kk) 
                  DOC_DRAIN(:,m,imetabo,kk) = zero
               ELSEIF (kkk .EQ. istrabo) THEN                  
                  DOC_DRAIN(:,m,iact,kk) = DOC_DRAIN(:,m,iact,kk) + DOC_DRAIN(:,m,istrabo,kk) * (un - lignin_struc_above(:,m))        
                  DOC_DRAIN(:,m,islo,kk) = DOC_DRAIN(:,m,islo,kk) + DOC_DRAIN(:,m,istrabo,kk) * lignin_struc_above(:,m) 
                  DOC_DRAIN(:,m,istrabo,kk) = zero
               ELSEIF (kkk .EQ. istrbel) THEN
                  DOC_DRAIN(:,m,iact,kk) = DOC_DRAIN(:,m,iact,kk) + DOC_DRAIN(:,m,istrbel,kk) * (un - lignin_struc_below(:,m,nbdl))        
                  DOC_DRAIN(:,m,islo,kk) = DOC_DRAIN(:,m,islo,kk) + DOC_DRAIN(:,m,istrbel,kk) * lignin_struc_below(:,m,nbdl) 
                  DOC_DRAIN(:,m,istrbel,kk) = zero  
               ELSEIF (kkk .EQ. imetbel) THEN              
                  DOC_DRAIN(:,m,iact,kk) = DOC_DRAIN(:,m,iact,kk) + DOC_DRAIN(:,m,imetbel,kk) 
                  DOC_DRAIN(:,m,imetbel,kk) = zero
               ELSE
                  DOC_DRAIN(:,m,kkk,kk) = DOC_DRAIN(:,m,kkk,kk)
               ENDIF
!write(numout,*) 'for pool number ', kkk, 'DOC_DRAIN is ', DOC_DRAIN(:,m,kkk,kk)	       
               !
            ENDDO !End loop over npool
                  !! TF-DOC
!write(numout,*) 'going into doc_flood throughfall'
            WHERE (veget_max(:,m) .GT. zero)
               DOC_FLOOD(:,m,iact,kk) = DOC_FLOOD(:,m,iact,kk) + wet_dep_flood(:,m,kk)*undemi/veget_max(:,m)
               DOC_FLOOD(:,m,islo,kk) = DOC_FLOOD(:,m,islo,kk) + wet_dep_flood(:,m,kk)*undemi/veget_max(:,m)
               DOC_FLOOD(:,m,iact,kk) = DOC_FLOOD(:,m,iact,kk) + floodcarbon_input(:,m,iact,kk)*dt
               DOC_FLOOD(:,m,islo,kk) = DOC_FLOOD(:,m,islo,kk) + floodcarbon_input(:,m,islo,kk)*dt
            ELSEWHERE
               DOC_RUN(:,m,iact,kk) = DOC_RUN(:,m,iact,kk)
               DOC_FLOOD(:,m,iact,kk) = DOC_FLOOD(:,m,iact,kk)
            ENDWHERE
            !
            DO l=1,sro_bottom
               WHERE (veget_max(:,m) .GT. zero)
                  DOC_FLOOD(:,m,islo,kk) = DOC_FLOOD(:,m,islo,kk) + CUE_coef(:) * fluxtot_flood(:,islow,kk,l)
                  DOC_FLOOD(:,m,ipas,kk) = DOC_FLOOD(:,m,ipas,kk) + CUE_coef(:) * fluxtot_flood(:,ipassive,kk,l)
                  DOC_FLOOD(:,m,iact,kk) = DOC_FLOOD(:,m,iact,kk) + CUE_coef(:) * fluxtot_flood(:,iactive,kk,l)
               ELSEWHERE
                  DOC_FLOOD(:,m,iact,kk) = DOC_FLOOD(:,m,iact,kk)
               ENDWHERE
            ENDDO !l=1,sro_bottom
            !
         ENDDO !End loop over elements

         DO kk = 1,nelements !Loop over elements
            DOC_EXP(:,m,irunoff,:,kk) = DOC_RUN(:,m,:,kk)/dt
            DOC_EXP(:,m,iflooded,:,kk) = DOC_FLOOD(:,m,:,kk)/dt
            DOC_EXP(:,m,idrainage,:,kk) = DOC_DRAIN(:,m,:,kk)/dt
         ENDDO !End loop over elements
! We substract runoff, floodout and drainage from DOC pools
         DO kkk = 1,npool !Loop over soil and litter pool
            DO kk = 1, nelements
               
               !For the first layer
               WHERE (soilwater_31mm(:,pref_soil_veg(m)) .GT. zero)
                  DOC(:,m,1,ifree,kkk,kk) = DOC(:,m,1,ifree,kkk,kk) - MIN(DOC(:,m,1,ifree,kkk,kk), &
                       &                    soilDOC_corr(:) * (runoff_per_soil(:,pref_soil_veg(m))*1E-3/soilwater_31mm(:,pref_soil_veg(m))) &
                       &                  * DOC(:,m,1,ifree,kkk,kk))
               ELSEWHERE
                  DOC(:,m,1,ifree,kkk,kk) = DOC(:,m,1,ifree,kkk,kk)
               ENDWHERE
               !And for the layers 2-5
               IF (sro_bottom .GT. 1) THEN
                  DO l=2,sro_bottom
                     WHERE (soilwater_31mm(:,pref_soil_veg(m)) .GT. zero)
                        DOC(:,m,l,ifree,kkk,kk) = DOC(:,m,l,ifree,kkk,kk) - MIN(DOC(:,m,l,ifree,kkk,kk), &
                             &                    soilDOC_corr(:) * (runoff_per_soil(:,pref_soil_veg(m))*1E-3/soilwater_31mm(:,pref_soil_veg(m))) &
                             &                  * DOC(:,m,l,ifree,kkk,kk))
                     ELSEWHERE
                        DOC(:,m,l,ifree,kkk,kk) = DOC(:,m,l,ifree,kkk,kk)
                     ENDWHERE
                  ENDDO
               ENDIF !sro_bottom .GT. 1
               !For the last layer
               WHERE (soil_mc(:,nbdl,pref_soil_veg(m)) .GT. zero)
                  DOC(:,m,nbdl,ifree,kkk,kk) = DOC(:,m,nbdl,ifree,kkk,kk) - &
                       &                                        MIN(DOC(:,m,nbdl,ifree,kkk,kk),&
                       &                                            DOC(:,m,nbdl,ifree,kkk,kk)*drainage_per_soil(:,pref_soil_veg(m))*1E-3 &
                       &                                            /((z_soil(nbdl)-z_soil(nbdl-1)) * soil_mc(:,nbdl,pref_soil_veg(m))))
               ELSEWHERE
                  DOC(:,m,nbdl,ifree,kkk,kk) = DOC(:,m,nbdl,ifree,kkk,kk)
               ENDWHERE
            ENDDO ! End loop over elements
         ENDDO !End loop over soil and litter pool
        ENDDO !end loop over nvm (DO m)
        
        
WRITE (numout,*) 'In_Slow at pix3 itau', itau, '=', SUM(SUM(In_Slow(3,:,:),DIM=2),DIM=1) 

!IF (maxval(DOC_FLUX_DIFF) .GT. zero) THEN
!	WRITE(numout,*) 'L2150  2.3.8  BOOM firstcall print DOC_FLUX_DIFF 2l max=', maxval(DOC_FLUX_DIFF(:,:,2,:,:))
!ENDIF
!IF (maxval(DOC_FLUX_DIFF) .GT. zero) THEN
!	WRITE(numout,*) 'L2150  2.3.8  BOOM firstcall print DOC_FLUX_DIFF 11l max=', maxval(DOC_FLUX_DIFF(:,:,11,:,:))
!ENDIF
!IF (maxval(DOC_FLUX_DIFF) .GT. zero) THEN
!	WRITE(numout,*) 'L2150  2.3.8 BOOM firstcall print DOC_FLUX_DIFF 12l max=', maxval(DOC_FLUX_DIFF(:,:,12,:,:))
!ENDIF
!IF (maxval(tprof(:,12,:)) .GT. 273.15) THEN
!WRITE(numout,*) 'tprof 11 layer min/max=', minval(tprof(:,11,:)), maxval(tprof(:,11,:))
!ENDIF
!IF (maxval(tprof(:,12,:)) .GT. 273.15) THEN
!WRITE(numout,*) 'tprof 12 layer min/max=', minval(tprof(:,12,:)), maxval(tprof(:,12,:))
!END IF
	
!	IF (minval(DOC) .LT. zero) THEN
!write (numout,*) 'BOOM DOC less than zero after lateral exp doc'      
!ENDIF
	!!HERE SIMON CALLS IMPLEMENTED cryoturbate diffusion scheme from MICT
	!!TO REPLACE the original LEAK diffusion scheme
	!!	
!!	CALL 	cryoturbate_doc_POC(npts, dt, dayno, altmax_ind, DOC, carbon_32l, litter_below, &
!!     		'diffuse', cryoturbation_diff_k_in/(one_day*one_year), bioturbation_diff_k_in/(one_day*one_year), altmax_lastyear, fixed_cryoturbation_depth)
!
!
!
!write (numout,*) 'stomate_soilcarbon L2028 DOC total maxval = ', maxval(DOC)
!
!      !! 2.3.10 Transport of particulate organic C (i.e. active, slow and passive
!      !! pools) following the second Fick's Law (O¿Brien and Stout, 1978; Wynn
!      !! et al., 2005). Represent the bioturbation and is generally associated
!      !! with transport  plant debris (Elzein and Balesdent, 1995; Bruun et al., 
!      !! 2007; Braakhekke et al., 2011, Guenet et al., 2013) 
!
!      ! Definition of the diffusion rate from Guenet et al., (2013) BG for the
!      ! moment but must depends on clay, ph organic C, or whatever...
!      
!
!carbon_old(:,:,m,:)=zero
!carbon_old_buffer(:,:,m,:)=zero
!carbon_old(:,:,m,:)=carbon_32l(:,:,m,:)
!
!     DO kk = 1, ncarb ! Loop over soil and litter pools
!
!        carbon_flux(:,kk,m,1) = Dif_coef(:)*ABS(carbon_old(:,kk,m,1)/(zf_soil(1))-carbon_old(:,kk,m,2)/(zf_soil(2)-zf_soil(1)))/(zf_soil(2)-zf_soil(1))
!
!      DO l= 2, ndeep-1 ! Loop over soil layers
!        carbon_flux(:,kk,m,l) = Dif_coef(:)*ABS(carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1))-carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l)))/(zf_soil(l+1)-zf_soil(l))
!      ENDDO
!        carbon_flux_old(:,kk,m,:) = carbon_flux(:,kk,m,:)
!
!       !Below we checked if in case that, in a given layer, you have diffusion
!       !in the above and below litters, both fluxes are not higher than the
!       !stocks of the given layer.
!
!      DO l =1, ndeep-2
!           WHERE (carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1)) .LT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l)) .AND. &
!                  carbon_old(:,kk,m,l+2)/(zf_soil(l+2)-zf_soil(l+1)) .LT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l)) .AND. &
!                  carbon_flux(:,kk,m,l)+carbon_flux(:,kk,m,l+1) .GT. carbon_old(:,kk,m,l+1))
!            carbon_flux(:,kk,m,l) = carbon_old(:,kk,m,l+1)*(carbon_flux(:,kk,m,l)/(carbon_flux(:,kk,m,l)+carbon_flux(:,kk,m,l+1)))
!            carbon_flux(:,kk,m,l+1) = carbon_old(:,kk,m,l+1)*(carbon_flux(:,kk,m,l+1)/(carbon_flux(:,kk,m,l)+carbon_flux(:,kk,m,l+1)))
!           ELSEWHERE (carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1)) .GE. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l)) .AND. &
!                      carbon_old(:,kk,m,l+2)/(zf_soil(l+2)-zf_soil(l+1)) .LT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l)) .AND. &
!                      carbon_old(:,kk,m,l+1) + carbon_flux(:,kk,m,l) - carbon_flux(:,kk,m,l+1) .LE. min_stomate)
!            carbon_flux(:,kk,m,l) = carbon_flux(:,kk,m,l)
!            carbon_flux(:,kk,m,l+1) = carbon_old(:,kk,m,l+1) + carbon_flux(:,kk,m,l)
!           ELSEWHERE (carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1)) .LT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l)) .AND. &
!                      carbon_old(:,kk,m,l+2)/(zf_soil(l+2)-zf_soil(l+1)) .GE. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l)) .AND. &
!                      carbon_old(:,kk,m,l+1) - carbon_flux(:,kk,m,l) + carbon_flux(:,kk,m,l+1) .LE. min_stomate)
!            carbon_flux(:,kk,m,l) = carbon_old(:,kk,m,l+1) + carbon_flux(:,kk,m,l+1)
!            carbon_flux(:,kk,m,l+1) = carbon_flux(:,kk,m,l+1)
!           ELSEWHERE
!            carbon_flux(:,kk,m,l) = carbon_flux(:,kk,m,l)
!            carbon_flux(:,kk,m,l+1) = carbon_flux(:,kk,m,l+1)
!           ENDWHERE
!      ENDDO
!
!      DO l= 1, ndeep-1 ! Loop over soil layers
!           WHERE ((carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1)) .LT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l))) .AND. &
!                 ((carbon_flux(:,kk,m,l) - carbon_old(:,kk,m,l+1)) .GE. min_stomate))
!            carbon_32l(:,kk,m,l) = carbon_old(:,kk,m,l) + carbon_old(:,kk,m,l+1) + carbon_old_buffer(:,kk,m,l)
!            carbon_old_buffer(:,kk,m,l+1) = -carbon_old(:,kk,m,l+1) 
!            carbon_32l(:,kk,m,l+1) = zero
!           ELSEWHERE ((carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1)) .LT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l))) .AND. &
!                  ((carbon_old(:,kk,m,l+1) - carbon_flux(:,kk,m,l)) .GT. min_stomate))
!            carbon_32l(:,kk,m,l) = carbon_old(:,kk,m,l) + carbon_flux(:,kk,m,l) + carbon_old_buffer(:,kk,m,l)
!            carbon_old_buffer(:,kk,m,l+1) =  - carbon_flux(:,kk,m,l)
!            carbon_32l(:,kk,m,l+1) = carbon_old(:,kk,m,l+1) - carbon_flux(:,kk,m,l)
!           ELSEWHERE ((carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1)) .GT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l))) .AND. &
!                  ((carbon_flux(:,kk,m,l) - carbon_old(:,kk,m,l)) .GE. min_stomate))
!            carbon_32l(:,kk,m,l) =zero + carbon_old_buffer(:,kk,m,l)
!            carbon_old_buffer(:,kk,m,l+1) = carbon_old(:,kk,m,l)
!            carbon_32l(:,kk,m,l+1) = carbon_old(:,kk,m,l+1) + carbon_old(:,kk,m,l)
!           ELSEWHERE ((carbon_old(:,kk,m,l)/(zf_soil(l)-zf_soil(l-1)) .GT. carbon_old(:,kk,m,l+1)/(zf_soil(l+1)-zf_soil(l))) .AND. &
!                  ((carbon_old(:,kk,m,l) - carbon_flux(:,kk,m,l)) .GT. min_stomate))
!            carbon_32l(:,kk,m,l) = carbon_old(:,kk,m,l) - carbon_flux(:,kk,m,l) + carbon_old_buffer(:,kk,m,l)
!            carbon_old_buffer(:,kk,m,l+1) = carbon_flux(:,kk,m,l)
!            carbon_32l(:,kk,m,l+1) = carbon_old(:,kk,m,l+1) + carbon_flux(:,kk,m,l)
!           ELSEWHERE
!            carbon_32l(:,kk,m,l) = carbon_old(:,kk,m,l)
!            carbon_32l(:,kk,m,l+1) = carbon_old(:,kk,m,l+1)
!           ENDWHERE
!        ENDDO ! End loop over soil layers
!
!
!
!      ENDDO ! End loop over carbon pools
!  ENDDO ! End loop over PFTs
!
!!!END SIMON COMMENT
!write (numout,*) '2319 mass balance closure'


!! 3. Check mass balance closure
IF (ld_doc) THEN    
    !! 3.1 Calculate components of the mass balance
    pool_end(:,:,:) = zero 

!Ending carbon pool
    DO m = 1, nvm
       DO i = 1,ncarb
          DO l = 1, ndeep
                 pool_end(:,m,icarbon) = pool_end(:,m,icarbon) + &
                           ( carbon_32l(:,i,m,l)* veget_max(:,m) ) 
          ENDDO
       ENDDO
    ENDDO

!Ending DOC pool
    DO m = 1, nvm
       DO i = 1,npool
          DO l = 1, ndeep
             DO j = 1, ndoc
                    pool_end(:,m,icarbon) = pool_end(:,m,icarbon) + &
                              ( DOC(:,m,l,j,i,icarbon) * veget_max(:,m)) 
             ENDDO
          ENDDO
       ENDDO
       pool_end(:,m,icarbon) = pool_end(:,m,icarbon) +  interception_storage(:,m,icarbon)
    ENDDO

!WRITE(numout,*) " pool_start(:,m,icarbon) soil=",  pool_start(:,4,icarbon)
!WRITE(numout,*) " pool_end(:,m,icarbon) soil=",  pool_end(:,4,icarbon)

!write (numout,*) '2353 calculate mass balance'

 !! 3.2 Calculate mass balance
    !  Note that soilcarbon is transfered to other pools but that the pool
    !  was not updated. We should not account for it in ::pool_end
    check_intern(:,:,:,:) = zero
    check_intern(:,:,iatm2land,icarbon) = zero

    
    check_intern(:,:,iland2atm,icarbon) = -un * (resp_hetero_soil(:,:)+resp_flood_soil(:,:)) * &
         veget_max(:,:) * dt 
    DO i=1,npool
       check_intern(:,:,ilat2out,icarbon) = check_intern(:,:,ilat2out,icarbon) - un * (DOC_EXP(:,:,irunoff,i,icarbon)+ &
            &         DOC_EXP(:,:,iflooded,i,icarbon)+ DOC_EXP(:,:,idrainage,i,icarbon)) * veget_max(:,:) * dt
    ENDDO !i=1,npool 
    check_intern(:,:,ilat2in,icarbon) = un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - &
         pool_start(:,:,icarbon))
    

    closure_intern = zero

    DO i = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + &
           check_intern(:,:,i,icarbon)
    ENDDO
!write (numout,*) '2379 before write verdict'

!! 3.3 Write verdict
    DO i = 1,npts
       IF (SUM(closure_intern(i,:,icarbon)) .GT. min_stomate .OR. &
            SUM(closure_intern(i,:,icarbon)) .LT. -min_stomate) THEN
          WRITE(numout,*) 'Error: mass balance is not closed in stomate_soilcarbon.f90 at pixel ', i
          WRITE(numout,*) '   Difference is, ', SUM(closure_intern(i,:,icarbon))
          WRITE(numout,*) '   Vegetation is, ', veget_max(i,:)

          WRITE(numout,*) '   deposition, ', SUM(wet_dep_ground(i,:,icarbon))
          WRITE(numout,*) '   deposition flood, ', SUM(wet_dep_flood(i,:,icarbon))

          WRITE(numout,*) '   soilcarbon_input, ', SUM(SUM(soilcarbon_input_DOC(i,:,:,:,icarbon),DIM=2),DIM=2)

          WRITE(numout,*) '   DOC_EXP, ',  SUM(SUM(DOC_EXP(i,:,:,:,icarbon),DIM=2),DIM=2)
          WRITE(numout,*) '   FLOOD_carbon_input, ', SUM(floodcarbon_input(i,:,:,icarbon),DIM=2)

          WRITE(numout,*) '   resp_hetero_soil(:,:), ', SUM(resp_hetero_soil(i,:))
          WRITE(numout,*) '   resp_flood_soil(:,:), ', SUM(resp_flood_soil(i,:))
       ENDIF
    ENDDO !i = 1,npts
ENDIF

!! 4 Writing history files

    CALL xios_orchidee_send_field("dry_dep_canopy",dry_dep_canopy(:,:,icarbon)/dt)
    CALL xios_orchidee_send_field("DOC_precip2canopy",DOC_precip2canopy(:,:,icarbon)/dt)
    CALL xios_orchidee_send_field("DOC_precip2ground",DOC_precip2ground(:,:,icarbon)/dt)
    CALL xios_orchidee_send_field("DOC_canopy2ground",DOC_canopy2ground(:,:,icarbon)/dt)
    CALL xios_orchidee_send_field("EXP_DOC_RUNOFF_ref",DOC_EXP(:,:,irunoff,islo,kk)+DOC_EXP(:,:,irunoff,ipas,kk))
    CALL xios_orchidee_send_field("EXP_DOC_RUNOFF_lab",DOC_EXP(:,:,irunoff,iact,kk))
    CALL xios_orchidee_send_field("EXP_DOC_DRAIN_ref",DOC_EXP(:,:,idrainage,islo,kk)+DOC_EXP(:,:,idrainage,ipas,kk))
    CALL xios_orchidee_send_field("EXP_DOC_DRAIN_lab",DOC_EXP(:,:,idrainage,iact,kk))
    CALL xios_orchidee_send_field("EXP_DOC_FLOOD_ref",DOC_EXP(:,:,iflooded,islo,kk)+DOC_EXP(:,:,iflooded,ipas,kk))
    CALL xios_orchidee_send_field("EXP_DOC_FLOOD_lab",DOC_EXP(:,:,iflooded,iact,kk))
    CALL xios_orchidee_send_field("interception_storage",interception_storage(:,:,icarbon))
    CALL xios_orchidee_send_field("precip2canopy",precip2canopy(:,:)/dt)
    CALL xios_orchidee_send_field("precip2ground",precip2ground(:,:)/dt)
    CALL xios_orchidee_send_field("canopy2ground",canopy2ground(:,:)/dt)
    CALL xios_orchidee_send_field("runoff_forest",runoff_per_soil(:,2)/dt)
    CALL xios_orchidee_send_field("CLAY",clay(:))
    CALL xios_orchidee_send_field("SOIL_pH",soil_pH(:))
    CALL xios_orchidee_send_field("KD",Kd(:,:))
    CALL xios_orchidee_send_field("POOR_SOILS",poor_soils(:))

    IF (printlev>=4) WRITE(numout,*) 'Leaving soilcarbon'
!    
  END SUBROUTINE soilcarbon
!
 !!!! MICT ORIGINAL BELOW     
!       DO k = 1, ncarb ! Loop over carbon pools
!         carbon(:,k,m) = carbon(:,k,m) + &
!            flux(:,iactive,k,icarbon) + flux(:,ipassive,k,icarbon) + flux(:,islow,k,icarbon)
!       ENDDO ! Loop over carbon pools
!       
!     ENDDO ! End loop over PFTs
!     
!  
 !!!SIMON REMOVED ANALYTIC SPINUP FOR SIMPLICITY
 !! 4. (Quasi-)Analytical Spin-up
    
    !! 4.1.1 Finish to fill MatrixA with fluxes between soil pools
    
!     IF (spinup_analytic) THEN
! 
!        DO m = 2,nvm 
! 
!           ! flux leaving the active pool
!           MatrixA(:,m,iact,iact) = moins_un * &
!                dt/carbon_tau(iactive) * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) * &
!                ( 1. - flux_tot_coeff(3) * clay(:)) 
! 
!           ! flux received by the active pool from the slow pool
!           MatrixA(:,m,iact,islo) =  frac_carb(:,islow,iactive)*dt/fbact_s * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)
! 
!           ! flux received by the active pool from the passive pool
!           MatrixA(:,m,iact,ipas) =  frac_carb(:,ipassive,iactive)*dt/fbact_p * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) 
! 
!           ! flux received by the slow pool from the active pool
!           MatrixA(:,m,islo,iact) =  frac_carb(:,iactive,islow) *&
!                dt/carbon_tau(iactive) * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) * &
!                ( 1. - flux_tot_coeff(3) * clay(:) ) 
! 
!           ! flux leaving the slow pool
!           MatrixA(:,m,islo,islo) = moins_un * &
!                dt/fbact_s * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)
! 
!           ! flux received by the passive pool from the active pool
!           MatrixA(:,m,ipas,iact) =  frac_carb(:,iactive,ipassive)* &
!                dt/carbon_tau(iactive) * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) *&
!                ( 1. - flux_tot_coeff(3) * clay(:) )
! 
!           ! flux received by the passive pool from the slow pool
!           MatrixA(:,m,ipas,islo) =  frac_carb(:,islow,ipassive) * &
!                dt/fbact_s * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)
! 
!           ! flux leaving the passive pool
!           MatrixA(:,m,ipas,ipas) =  moins_un * &
!                dt/fbact_p * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)      
! 
! 
!           IF ( (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) ) THEN ! C3crop
! 
!              ! flux leaving the active pool
!              MatrixA(:,m,iact,iact) = MatrixA(:,m,iact,iact) * &
!                   flux_tot_coeff(1)  
! 
!              ! flux received by the active pool from the slow pool
!              MatrixA(:,m,iact,islo)= MatrixA(:,m,iact,islo) * &
!                   flux_tot_coeff(1) 
! 
!              ! flux received by the active pool from the passive pool
!              MatrixA(:,m,iact,ipas) = MatrixA(:,m,iact,ipas) * &
!                   flux_tot_coeff(1)    
! 
!              ! flux received by the slow pool from the active pool
!              MatrixA(:,m,islo,iact) =  MatrixA(:,m,islo,iact) * &
!                   flux_tot_coeff(1) 
! 
!              ! flux leaving the slow pool
!              MatrixA(:,m,islo,islo) = MatrixA(:,m,islo,islo) * &
!                   flux_tot_coeff(1)     
! 
!              ! flux received by the passive pool from the active pool
!              MatrixA(:,m,ipas,iact) = MatrixA(:,m,ipas,iact) * &
!                   flux_tot_coeff(1)
! 
!              ! flux received by the passive pool from the slow pool
!              MatrixA(:,m,ipas,islo) = MatrixA(:,m,ipas,islo) * &
!                   flux_tot_coeff(1)
! 
!              ! flux leaving the passive pool
!              MatrixA(:,m,ipas,ipas) =  MatrixA(:,m,ipas,ipas) *&
!                   flux_tot_coeff(1)
! 
!           ENDIF ! (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) 
! 
! 
!           IF ( (.NOT. natural(m)) .AND. is_c4(m) ) THEN ! C4crop
! 
!              ! flux leaving the active pool
!              MatrixA(:,m,iact,iact) = MatrixA(:,m,iact,iact) * &
!                   flux_tot_coeff(2)  
! 
!             ! flux received by the active pool from the slow pool
!              MatrixA(:,m,iact,islo)= MatrixA(:,m,iact,islo) * &
!                   flux_tot_coeff(2) 
! 
!              ! flux received by the active pool from the passive pool
!              MatrixA(:,m,iact,ipas) = MatrixA(:,m,iact,ipas) * &
!                   flux_tot_coeff(2)    
! 
!              ! flux received by the slow pool from the active pool
!              MatrixA(:,m,islo,iact) =  MatrixA(:,m,islo,iact) * &
!                   flux_tot_coeff(2) 
! 
!              ! flux leaving the slow pool
!              MatrixA(:,m,islo,islo) = MatrixA(:,m,islo,islo) * &
!                   flux_tot_coeff(2)     
! 
!              ! flux received by the passive pool from the active pool
!              MatrixA(:,m,ipas,iact) = MatrixA(:,m,ipas,iact) * &
!                   flux_tot_coeff(2)
! 
!              ! flux received by the passive pool from the slow pool
!              MatrixA(:,m,ipas,islo) = MatrixA(:,m,ipas,islo) * &
!                   flux_tot_coeff(2)
! 
!              ! flux leaving the passive pool
!              MatrixA(:,m,ipas,ipas) =  MatrixA(:,m,ipas,ipas) * &
!                   flux_tot_coeff(2)
! 
!           ENDIF ! (.NOT. natural(m)) .AND. is_c4(m) 
! 
!           IF (printlev>=4) WRITE(numout,*)'Finish to fill MatrixA'
! 
!        ENDDO ! Loop over # PFTS
! 
! 
!        ! 4.2 Add Identity for each submatrix(7,7) 
! 
!        DO j = 1,npool
!           MatrixA(:,:,j,j) = MatrixA(:,:,j,j) + un 
!        ENDDO
! 
!     ENDIF ! (spinup_analytic)
! 
!     IF (printlev>=4) WRITE(numout,*) 'Leaving soilcarbon'
!     
!   END SUBROUTINE soilcarbon
!
!END MODULE stomate_soilcarbon


 !!!! MICT ORIGINAL BELOW     
!       DO k = 1, ncarb ! Loop over carbon pools
!         carbon(:,k,m) = carbon(:,k,m) + &
!            flux(:,iactive,k,icarbon) + flux(:,ipassive,k,icarbon) + flux(:,islow,k,icarbon)
!       ENDDO ! Loop over carbon pools
!       
!     ENDDO ! End loop over PFTs
!     
!  
 !!!SIMON REMOVED ANALYTIC SPINUP FOR SIMPLICITY
 !! 4. (Quasi-)Analytical Spin-up
    
    !! 4.1.1 Finish to fill MatrixA with fluxes between soil pools
    
!     IF (spinup_analytic) THEN
! 
!        DO m = 2,nvm 
! 
!           ! flux leaving the active pool
!           MatrixA(:,m,iact,iact) = moins_un * &
!                dt/carbon_tau(iactive) * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) * &
!                ( 1. - flux_tot_coeff(3) * clay(:)) 
! 
!           ! flux received by the active pool from the slow pool
!           MatrixA(:,m,iact,islo) =  frac_carb(:,islow,iactive)*dt/fbact_s * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)
! 
!           ! flux received by the active pool from the passive pool
!           MatrixA(:,m,iact,ipas) =  frac_carb(:,ipassive,iactive)*dt/fbact_p * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) 
! 
!           ! flux received by the slow pool from the active pool
!           MatrixA(:,m,islo,iact) =  frac_carb(:,iactive,islow) *&
!                dt/carbon_tau(iactive) * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) * &
!                ( 1. - flux_tot_coeff(3) * clay(:) ) 
! 
!           ! flux leaving the slow pool
!           MatrixA(:,m,islo,islo) = moins_un * &
!                dt/fbact_s * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)
! 
!           ! flux received by the passive pool from the active pool
!           MatrixA(:,m,ipas,iact) =  frac_carb(:,iactive,ipassive)* &
!                dt/carbon_tau(iactive) * &
!                control_moist(:,ibelow) * control_temp(:,ibelow) *&
!                ( 1. - flux_tot_coeff(3) * clay(:) )
! 
!           ! flux received by the passive pool from the slow pool
!           MatrixA(:,m,ipas,islo) =  frac_carb(:,islow,ipassive) * &
!                dt/fbact_s * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)
! 
!           ! flux leaving the passive pool
!           MatrixA(:,m,ipas,ipas) =  moins_un * &
!                dt/fbact_p * &
!                control_moist(:,ibelow) * control_temp(:,ibelow)      
! 
! 
!           IF ( (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) ) THEN ! C3crop
! 
!              ! flux leaving the active pool
!              MatrixA(:,m,iact,iact) = MatrixA(:,m,iact,iact) * &
!                   flux_tot_coeff(1)  
! 
!              ! flux received by the active pool from the slow pool
!              MatrixA(:,m,iact,islo)= MatrixA(:,m,iact,islo) * &
!                   flux_tot_coeff(1) 
! 
!              ! flux received by the active pool from the passive pool
!              MatrixA(:,m,iact,ipas) = MatrixA(:,m,iact,ipas) * &
!                   flux_tot_coeff(1)    
! 
!              ! flux received by the slow pool from the active pool
!              MatrixA(:,m,islo,iact) =  MatrixA(:,m,islo,iact) * &
!                   flux_tot_coeff(1) 
! 
!              ! flux leaving the slow pool
!              MatrixA(:,m,islo,islo) = MatrixA(:,m,islo,islo) * &
!                   flux_tot_coeff(1)     
! 
!              ! flux received by the passive pool from the active pool
!              MatrixA(:,m,ipas,iact) = MatrixA(:,m,ipas,iact) * &
!                   flux_tot_coeff(1)
! 
!              ! flux received by the passive pool from the slow pool
!              MatrixA(:,m,ipas,islo) = MatrixA(:,m,ipas,islo) * &
!                   flux_tot_coeff(1)
! 
!              ! flux leaving the passive pool
!              MatrixA(:,m,ipas,ipas) =  MatrixA(:,m,ipas,ipas) *&
!                   flux_tot_coeff(1)
! 
!           ENDIF ! (.NOT. natural(m)) .AND. (.NOT. is_c4(m)) 
! 
! 
!           IF ( (.NOT. natural(m)) .AND. is_c4(m) ) THEN ! C4crop
! 
!              ! flux leaving the active pool
!              MatrixA(:,m,iact,iact) = MatrixA(:,m,iact,iact) * &
!                   flux_tot_coeff(2)  
! 
!             ! flux received by the active pool from the slow pool
!              MatrixA(:,m,iact,islo)= MatrixA(:,m,iact,islo) * &
!                   flux_tot_coeff(2) 
! 
!              ! flux received by the active pool from the passive pool
!              MatrixA(:,m,iact,ipas) = MatrixA(:,m,iact,ipas) * &
!                   flux_tot_coeff(2)    
! 
!              ! flux received by the slow pool from the active pool
!              MatrixA(:,m,islo,iact) =  MatrixA(:,m,islo,iact) * &
!                   flux_tot_coeff(2) 
! 
!              ! flux leaving the slow pool
!              MatrixA(:,m,islo,islo) = MatrixA(:,m,islo,islo) * &
!                   flux_tot_coeff(2)     
! 
!              ! flux received by the passive pool from the active pool
!              MatrixA(:,m,ipas,iact) = MatrixA(:,m,ipas,iact) * &
!                   flux_tot_coeff(2)
! 
!              ! flux received by the passive pool from the slow pool
!              MatrixA(:,m,ipas,islo) = MatrixA(:,m,ipas,islo) * &
!                   flux_tot_coeff(2)
! 
!              ! flux leaving the passive pool
!              MatrixA(:,m,ipas,ipas) =  MatrixA(:,m,ipas,ipas) * &
!                   flux_tot_coeff(2)
! 
!           ENDIF ! (.NOT. natural(m)) .AND. is_c4(m) 
! 
!           IF (printlev>=4) WRITE(numout,*)'Finish to fill MatrixA'
! 
!        ENDDO ! Loop over # PFTS
! 
! 
!        ! 4.2 Add Identity for each submatrix(7,7) 
! 
!        DO j = 1,npool
!           MatrixA(:,:,j,j) = MatrixA(:,:,j,j) + un 
!        ENDDO
! 
!     ENDIF ! (spinup_analytic)
! 
!     IF (printlev>=4) WRITE(numout,*) 'Leaving soilcarbon'
!     
!   END SUBROUTINE soilcarbon
! 
!================================================================================================================================
!! SUBROUTINE   : altcalc_DOC
!!
!>\BRIEF        This routine calculate active layer thickness
!!
!! DESCRIPTION : 
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : alt
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART11    : None
!! \n
!_
!================================================================================================================================  
  SUBROUTINE altcalc_DOC(npts,dt,dayno,scnd, tprof, zprof, alt, alt_ind, altmax, altmax_ind, &
        altmax_lastyear, altmax_ind_lastyear)

  !! 0. Variable and parameter declaration

    !! 0.1  Input variables

    INTEGER(i_std), INTENT(in) 			                 :: npts
    REAL(r_std), INTENT(in)                                       :: dt                  !timestep
!    REAL(r_std), INTENT(in)                                      :: time_step           !! time step in seconds
    INTEGER(i_std), INTENT(in)			                 :: dayno               !! number of the day in the current year
    REAL(r_std), INTENT(in)			                 :: scnd                !! model time & time step
!    REAL(r_std), DIMENSION(npts,ndeep, nvm), INTENT(in)      :: temp                !! soil temperature
    REAL(r_std), DIMENSION(npts,ndeep,nvm),INTENT (in)	    :: tprof             !! deep temperature profile    
    REAL(r_std), DIMENSION(ndeep), INTENT(in)   	         :: zprof               !! soil depths (m)

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts, nvm), INTENT(out)	         :: alt	                !! active layer thickness  
    INTEGER, DIMENSION(npts, nvm), INTENT(out)	         :: alt_ind	        !! active layer index  
    
    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts, nvm),INTENT(inout)	         :: altmax_lastyear     !! Maximum active-layer thickness
    REAL(r_std), DIMENSION(npts, nvm),INTENT(inout)          :: altmax              !! Maximum active-layer thickness
    INTEGER(i_std), DIMENSION(npts, nvm),INTENT(inout)	 :: altmax_ind          !! Maximum over the year active-layer index
    INTEGER(i_std), DIMENSION(npts, nvm),INTENT(inout)       :: altmax_ind_lastyear !! Maximum over the year active-layer index

    !! 0.4 Local variables
!!    LOGICAL, DIMENSION(npts,nvm)		:: veget_mask_2d	!!SIMON added this local variable so that =TRUE

    INTEGER				                         :: ix,iz,il,iv         !! grid indices
    LOGICAL, SAVE 				                 :: firstcall = .TRUE.
    INTEGER, save                                                :: tcounter
    INTEGER(i_std), SAVE  			                 :: id, id2
    LOGICAL, SAVE			                         :: check = .FALSE.
    LOGICAL, SAVE			                         :: newaltcalc = .FALSE.
    LOGICAL, DIMENSION(npts,nvm)                             :: inalt, bottomlevelthawed
    CHARACTER(LEN=16)			                         :: buf  
    INTEGER                                                      :: lev
!WRITE(numout,*) 'entering altcalc'
!WRITE(numout,*) 'altcalc checking veget_mask values =', (veget_mask_2d(:,:))
    
    IF ( firstcall )  THEN
!WRITE(numout,*) 'alt firstcall'
       ! calculate altmax_ind from altmax
       altmax_ind(:,:) = 0
       DO ix = 1, npts
          DO iv = 1, nvm
             IF ( veget_mask_2d(ix,iv) ) THEN	!SIMON uncommented this 
                DO il=1,ndeep
                   IF ( altmax(ix,iv) .GE. zprof(il) ) THEN
                      altmax_ind(ix,iv) = altmax_ind(ix,iv) + 1
                   END IF
                END DO
             END IF
          END DO
       END DO
       altmax_lastyear(:,:) = altmax(:,:)
       altmax_ind_lastyear(:,:) = altmax_ind(:,:)
       firstcall = .FALSE.
!WRITE(numout,*) 'alt firstcall altmax maxval=', (maxval(altmax))
!WRITE(numout,*) 'alt firstcall altmax_ind maxval=', (maxval(altmax_ind))
!WRITE(numout,*) 'alt firstcall zprof=', zprof

       !Config Key   = newaltcalc
       !Config Desc  = calculate alt ?
       !Config Def   = n
       !Config If    = OK_PC
       !Config Help  = 
       !Config Unit  = [flag]
       CALL getin_p('newaltcalc', newaltcalc)

    ELSE
!WRITE(numout,*) 'altcalc not firstcall'    !doesn't get here
!!       veget_mask_2d(:,:) = .TRUE.

       ! all other timesteps
       IF ( .NOT. newaltcalc ) THEN
!WRITE(numout,*) 'altcalc not newaltcalc'	!doesn't get here

          DO ix = 1, npts
             DO iv = 1, nvm
                IF ( veget_mask_2d(ix,iv) ) THEN
                   iz = 1
                   DO WHILE( tprof(ix,iz,iv) > ZeroCelsius .AND. iz < ndeep )
                      iz = iz + 1	  
                   END DO
                   IF( iz == 1 ) THEN 
                      ! it means that all is frozen
                      alt(ix,iv) = zero
		   ELSE
                      alt(ix,iv) = zprof(iz-1)
                   END IF
!!		      IF (alt(ix,iv) .GT. zero) THEN
!!!		   	WRITE(numout,*) 'alt(ix,iv)=', alt(ix,iv)
!!!			ELSE
!!!			WRITE(numout,*) 'still didnt work'
!!!		      ENDIF
                   alt_ind(ix,iv) = iz-1
                END IF
             END DO
          END DO


!WRITE(numout,*) 'altcalc not newaltcalc alt_ind maxval=', (maxval(alt_ind))	!!doesn't get here
!WRITE(numout,*) 'altcalc not newaltcalc alt maxval=', (maxval(alt))
!WRITE(numout,*) 'altcalc not newaltcalc zprof maxval=', (maxval(zprof))



       ELSE          
          ! initialize for pfts that don't exist
          alt(:,:) = zprof(ndeep)  
          bottomlevelthawed(:,:) = .FALSE.
          ! start from bottom and work up instead
          WHERE (tprof(:,ndeep,:) > ZeroCelsius ) 
             bottomlevelthawed(:,:) = .TRUE.
             alt(:,:) = zprof(ndeep)
             alt_ind(:,:) = ndeep
          END WHERE
          inalt(:,:) = .FALSE.
          DO iz = 1, ndeep - 1
             lev = ndeep - iz
             WHERE ( tprof(:,lev,:) > ZeroCelsius .AND. .NOT. inalt(:,:) .AND. .NOT. bottomlevelthawed(:,:) )
                inalt(:,:) = .TRUE.
                alt(:,:) = zprof(lev)
                alt_ind(:,:) = lev
             ELSEWHERE ( tprof(:,lev,:) <= ZeroCelsius .AND. inalt(:,:) .AND. .NOT. bottomlevelthawed(:,:) )
                inalt(:,:) = .FALSE.
             END WHERE
          END DO
          WHERE ( .NOT. inalt .AND. .NOT. bottomlevelthawed(:,:) ) 
             alt(:,:) = zero
             alt_ind(:,:) = 0
          END WHERE
!WRITE(numout,*) 'altcalc yes newaltcalc alt_ind maxval=', (maxval(alt_ind))
!WRITE(numout,*) 'altcalc yes newaltcalc alt maxval=', (maxval(alt))
!WRITE(numout,*) 'altcalc yes newaltcalc inalt =', inalt
       ENDIF

       ! debug
       IF ( check ) THEN
          IF (ANY(alt(:,:) .GT. zprof(ndeep))) THEN
             WRITE(*,*) 'error: alt greater than soil depth.'
          ENDIF
       ENDIF

       ! Maximum over the year active layer thickness
       WHERE ( ( alt(:,:) .GT. altmax(:,:) ) .AND. veget_mask_2d(:,:))   
          altmax(:,:) = alt(:,:)
          altmax_ind(:,:) = alt_ind(:,:)
       ENDWHERE
       
       IF ( .NOT. soilc_isspinup ) THEN
             ! do it on the second timestep, that way when we are writing restart files it is not done before that!
             ! now we are doing daily permafrost calcs, so just run it on the second day.
          IF ( ( dayno .EQ. 2) ) THEN  
             ! Reinitialize ALT_max
             altmax_lastyear(:,:) = altmax(:,:)
             altmax_ind_lastyear(:,:) = altmax_ind(:,:)
             altmax(:,:) = alt(:,:)
             altmax_ind(:,:) = alt_ind(:,:)
          END IF
       ELSE
!WRITE(numout,*) 'altcalc not soilc_isspinup alt_ind maxval=', (maxval(alt_ind))
!WRITE(numout,*) 'altcalc not soilc_isspinup alt maxval=', (maxval(alt))
!WRITE(numout,*) 'altcalc not soilc_isspinup altmax maxval=', (maxval(altmax))
             ! for spinup, best to set altmax_lastyear to altmax, and not boter to reset since every year is the same, 
             ! and if you try to do so, it doesn't work properly --  06 may 2010
          altmax_lastyear(:,:) = altmax(:,:)
          altmax_ind_lastyear(:,:) = altmax_ind(:,:)
       END IF
    END IF
!WRITE(numout,*) 'leaving altcalc_DOC'
    IF (printlev>=3) WRITE(*,*) 'leaving  altcalc_DOC'
  END SUBROUTINE altcalc_DOC
  

!!================================================================================================================================
!! SUBROUTINE   : cryoturbate_doc_POC
!!
!>\BRIEF        This routine calculates cryoturbation process
!!
!! DESCRIPTION : 
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : 
!! 
!! REFERENCE(S) : None
!!
!! FLOWCHART11    : None
!! \n
!_
!================================================================================================================================     


  SUBROUTINE cryoturbate_doc_POC(npts, dt, dayno, altmax_ind, DOC, carbon_32l, &
       litter_below, action, diff_k_const, bio_diff_k_const, altmax_lastyear, fixed_cryoturbation_depth)

  !! 0. Variable and parameter declaration

    !! 0.1  Input variables     

    INTEGER(i_std), INTENT(in)				       :: npts         !! domain size
    REAL(r_std), INTENT(in)                                    :: dt        !! time step in seconds
    INTEGER(i_std), INTENT(in)			               :: dayno            !! number of the day in the current year
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(in)	       :: altmax_ind       !! Maximum over the year active-layer index
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)            :: altmax_lastyear  !! Maximum over the year active-layer thickness
    CHARACTER(LEN=*), INTENT(in)                               :: action           !! what to do
    REAL(r_std), INTENT(in)                                    :: diff_k_const
    REAL(r_std), INTENT(in)                                    :: bio_diff_k_const

    !! 0.2 Output variables 

    !! 0.3 Modified variables
    REAL(r_std), DIMENSION(npts,ncarb,nvm,ndeep), INTENT(inout)  			:: carbon_32l           !! Soil carbon pools: active, slow, or passive, \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements), INTENT(inout)     :: DOC                !! Dissolved Organic Carbon in soil
	REAL(r_std), DIMENSION(npts,nlitt,nvm,ndeep,nelements), INTENT(inout) :: litter_below 
    REAL(r_std), DIMENSION(npts,nvm),INTENT(inout)         :: fixed_cryoturbation_depth  !! depth to hold cryoturbation to for fixed runs
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm), INTENT(inout)  :: deepC_a          !! soil carbon (g/m**3) active
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm), INTENT(inout)  :: deepC_s          !! soil carbon (g/m**3) slow
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm), INTENT(inout)  :: deepC_p          !! soil carbon (g/m**3) passive
!
!
!
    !! 0.4 Local variables
!!!    LOGICAL, DIMENSION(npts,nvm)		:: veget_mask_2d	
    LOGICAL, SAVE			                       :: firstcall = .TRUE.
    LOGICAL, SAVE			                       :: use_new_cryoturbation
    INTEGER, SAVE			                       :: cryoturbation_method
    REAL(r_std), DIMENSION(npts,ncarb,nvm,ndeep)  		:: carbon_32l_old
    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements)  :: DOC_old
    REAL(r_std), DIMENSION(npts,nlitt,nvm,ndeep,nelements) :: litter_below_old
!This is declared in the beginning of the module (soilcarbon)
!	REAL(r_std), DIMENSION(npts,ncarb,nvm,ndeep), ALLOCATABLE, SAVE  :: alpha_C
!	REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool), ALLOCATABLE, SAVE  :: alpha_DOC
    REAL(r_std), DIMENSION(npts,ncarb,nvm,ndeep)  				:: carbon_32l_old_buffer
    REAL(r_std), DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements)  :: DOC_old_buffer
    REAL(r_std), DIMENSION(npts,nlitt,nvm,ndeep,nelements) :: litter_below_old_buffer 
 !!Bertrand Diff scheme changes END
    REAL(r_std), DIMENSION(npts,ncarb,nvm)                       :: altC_C_old       !! soil carbon (g/m**2) active integrated over active layer before cryoturbation
    REAL(r_std), DIMENSION(npts,nvm,ndoc,npool)                       :: altC_DOC_old       !! soil carbon (g/m**2) active integrated over active layer before cryoturbation
    REAL(r_std), DIMENSION(npts,nlitt,nvm) 							:: altC_litter_below_old 

    REAL(r_std), DIMENSION(npts,ncarb,nvm)                       :: altC_C
    REAL(r_std), DIMENSION(npts,nvm,ndoc,npool)                       :: altC_DOC
    REAL(r_std), DIMENSION(npts,nlitt,nvm) 							:: altC_litter_below 

!

    INTEGER(i_std), PARAMETER                                  :: n_totakefrom = 3 !! how many surface layers to subtract from in mass balance
    REAL(r_std), DIMENSION(npts,ncarb,nvm)                       :: surfC_totake_C   !! active soil carbon to subtract from surface layers to maintain mass balance (g/m**3) 
    REAL(r_std), DIMENSION(npts,nvm,ndoc,npool)                       :: surfC_totake_DOC   !! active soil carbon to subtract from surface layers to maintain mass balance (g/m**3) 
    REAL(r_std), DIMENSION(npts,nlitt,nvm)                       :: surfC_totake_litter_below   !! active soil carbon to subtract from surface layers to maintain mass balance (g/m**3) 

    CHARACTER(LEN=20), SAVE                                    :: last_action = 'not called'
    INTEGER(i_std)                                             :: cryoturb_date
    REAL(r_std), SAVE                                          :: max_cryoturb_alt
    REAL(r_std), SAVE                                          :: min_cryoturb_alt
    REAL(r_std), SAVE                                          :: bioturbation_depth
    LOGICAL, SAVE			                       :: reset_fixed_cryoturbation_depth = .FALSE.
    LOGICAL, SAVE			                       :: use_fixed_cryoturbation_depth = .FALSE.
!!SIMON added bertrand
!    LOGICAL, SAVE                                              :: adjust_method_new = .TRUE.
    LOGICAL, SAVE                                              :: use_new_diffusion_scheme = .FALSE.
    REAL(r_std), DIMENSION(npts,nvm)	               :: cryoturbation_depth
 INTEGER(i_std)                                                :: ip,iv,il,ier,ilt, ido, ic, ipo  !! Indices (unitless)

 !!SIMON REMOVED 
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_a_old_buffer      !! soil carbon (g/m**3) active before timestep variable used to close the mass balance
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_s_old_buffer      !! soil carbon (g/m**3) slow before timestep variable used to close the mass balance
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_p_old_buffer      !! soil carbon (g/m**3) passive before timestep variable used to close the mass balance
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_a_flux     !! soil carbon (g/m**3/tstep) active flux between each layers
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_s_flux     !! soil carbon (g/m**3/tstep) slow flux between each layers
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_p_flux     !! soil carbon (g/m**3/tstep) passive flux between each layers
!
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: surfC_totake_a   !! active soil carbon to subtract from surface layers to maintain mass balance (g/m**3) 
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: surfC_totake_s   !! slow soil carbon to subtract from surface layers to maintain mass balance (g/m**3) 
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: surfC_totake_p   !! passive soil carbon to subtract from surface layers to maintain mass balance (g/m**3) 
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: error_a
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: error_s
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: error_p
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: altC_a
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: altC_s
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: altC_p
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: altC_a_old       !! soil carbon (g/m**2) active integrated over active layer before cryoturbation
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: altC_s_old       !! soil carbon (g/m**2) slow integrated over active layer before cryoturbation
!     REAL(r_std), DIMENSION(kjpindex,nvm)                       :: altC_p_old       !! soil carbon (g/m**2) passive integrated over active layer before cryoturbation
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_a_old      !! soil carbon (g/m**3) active before timestep
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_s_old      !! soil carbon (g/m**3) slow before timestep
!    REAL(r_std), DIMENSION(kjpindex,ndeep,nvm)                 :: deepC_p_old      !! soil carbon (g/m**3) passive before timestep
    ! 1. ensure that we do not repeat actions
    !
!!WRITE(numout,*) 'Entering cryoturbate'    
IF (printlev>=3) WRITE(numout,*) 'Entering cryoturbate' 

    IF ( action .EQ. last_action ) THEN
       !
       WRITE(*,*) 'CANNOT TAKE THE SAME ACTION TWICE in CRYOTURBATE_doc_POC: ',TRIM(action)
       STOP
       !
    ENDIF
 IF ( action .EQ. 'diffuse' ) THEN
       IF (firstcall) THEN
!!!SIMON removed the .not. from the IF loop below
          ! 2. faire les trucs du debut
 !      IF (use_new_diffusion_scheme) THEN   
          ! 2.1 allocation des variables
!WRITE(numout,*) 'Cryoturbate: 1 Entering allocation at firstcall'               
          ALLOCATE (xe_ALL(npts,nvm),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in xe_ALL allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
              & , npts*nvm
             STOP 'soilcarbon'
          END IF
	xe_ALL(:,:)=0.
          ALLOCATE (xc_cryoturb(npts,ndeep,nvm),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in xc_cryoturb allocation. We stop. We need ',npts,' fois ',ndeep,' fois ',nvm,' words = '&
              & , npts*ndeep*nvm
             STOP 'soilcarbon'
          END IF
	xc_cryoturb(:,:,:)=0.
          ALLOCATE (xd_cryoturb(npts,ndeep,nvm),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in xd_cryoturb allocation. We stop. We need ',npts,' fois ',ndeep,' fois ',nvm,' words = '&
              & , npts*ndeep*nvm
             STOP 'soilcarbon'
          END IF
	xd_cryoturb(:,:,:)=0.
!!!SIMON MADE new variable
          ALLOCATE (alpha_C(npts,ncarb,nvm,ndeep),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in alpha_C allocation. We stop. We need ',npts,' fois ',ncarb,' fois ',nvm,' fois ',ndeep,' words = '&
              & , npts*ncarb*nvm*ndeep
             STOP 'soilcarbon'
          END IF
	  alpha_C(:,:,:,:)=0.

          ALLOCATE (alpha_DOC(npts,nvm,ndeep,ndoc,npool),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in alpha_DOC allocation. We stop. We need ',npts,' fois ',nvm,' fois ',ndeep,' fois ',ndoc,' fois ',npool,' words = '&
              & , npts*nvm*ndeep*ndoc*npool
             STOP 'soilcarbon'
          END IF
	  alpha_DOC(:,:,:,:,:)=0.

          ALLOCATE (alpha_litter_below(npts,nlitt,nvm,ndeep),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in alpha_litter_below allocation. We stop. We need ',npts,' fois ',nlitt,' fois ',nvm,' fois ',ndeep,' fois ',nelements,' words = '&
              & , npts*nlitt*nvm*ndeep
             STOP 'soilcarbon'
          END IF
	  alpha_litter_below(:,:,:,:)=0.



          ALLOCATE (beta_C(npts,ncarb,nvm,ndeep),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in beta_C allocation. We stop. We need ',npts,' fois ',ncarb,' fois ',nvm,' fois ',ndeep,' words = '&
              & , npts*ncarb*nvm*ndeep
             STOP 'soilcarbon'
          END IF
	  beta_C(:,:,:,:)=0.

          ALLOCATE (beta_DOC(npts,nvm,ndeep,ndoc,npool),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in beta_DOC allocation. We stop. We need ',npts,' fois ',nvm,' fois ',ndeep,' fois ',ndoc,' fois ',npool,' words = '&
              & , npts*nvm*ndeep*ndoc*npool
             STOP 'soilcarbon'
          END IF
	  beta_DOC(:,:,:,:,:)=0.

          ALLOCATE (beta_litter_below(npts,nlitt,nvm,ndeep),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in beta_litter_below allocation. We stop. We need ',npts,' fois ',nlitt,' fois ',nvm,' fois ',ndeep,' fois ',' words = '&
              & , npts*nlitt*nvm*ndeep
             STOP 'soilcarbon'
          END IF
	  beta_litter_below(:,:,:,:)=0.
!END IF

          ALLOCATE (diff_k(npts,ndeep,nvm),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in diff_k allocation. We stop. We need ',npts,' fois ',ndeep,' fois ',nvm,' words = '&
              & , npts*ndeep*nvm
             STOP 'soilcarbon'
          END IF
          diff_k=0.
          ALLOCATE (cryoturb_location(npts,nvm),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in cryoturb_location allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
              & , npts*nvm
             STOP 'soilcarbon'
          END IF
	cryoturb_location=.FALSE.
          ALLOCATE (bioturb_location(npts,nvm),stat=ier)
          IF (ier.NE.0) THEN
             WRITE (numout,*) ' error in bioturb_location allocation. We stop. We need ',npts,' fois ',nvm,' words = '&
              & , npts*nvm
             STOP 'soilcarbon'
          END IF
	  bioturb_location=.FALSE.

!WRITE(numout,*) 'Cryoturbate: 2 leaving allocation at firstcall'               
	            
          cryoturb_location(:,:) = .false.
          use_new_cryoturbation = .false.
          !
          !Config Key   = use_new_cryoturbation
          !Config Desc  = 
          !Config Def   = n
          !Config If    = OK_PC
          !Config Help  = 
          !Config Units = [flag]
          CALL getin_p('use_new_cryoturbation', use_new_cryoturbation)
          !
          !Config Key   = cryoturbation_method
          !Config Desc  = 
          !Config Def   = 1
          !Config If    =  OK_PC 
          !Config Help  = 
          !Config Units = []
          cryoturbation_method = 4
          CALL getin_p('cryoturbation_method', cryoturbation_method)
          !
          !Config Key   = max_cryoturb_alt
          !Config Desc  = 
          !Config Def   = 1
          !Config If    = OK_PC
          !Config Help  = 
          !Config Units = []
          max_cryoturb_alt = 3.
          CALL getin_p('max_cryoturb_alt',max_cryoturb_alt)
          !
          !Config Key   = min_cryoturb_alt
          !Config Desc  = 
          !Config Def   = 1
          !Config If    = OK_PC
          !Config Help  = 
          !Config Units = []
          min_cryoturb_alt = 0.01
          CALL getin_p('min_cryoturb_alt',min_cryoturb_alt)
          !
          !Config Key   = reset_fixed_cryoturbation_depth
          !Config Desc  = 
          !Config Def   = n
          !Config If    = OK_PC
          !Config Help  = 
          !Config Units = [flag]
          CALL getin_p('reset_fixed_cryoturbation_depth',reset_fixed_cryoturbation_depth)
          IF (reset_fixed_cryoturbation_depth) THEN
             fixed_cryoturbation_depth = altmax_lastyear
          ENDIF
          !
          !Config Key   = use_fixed_cryoturbation_depth
          !Config Desc  = 
          !Config Def   = n
          !Config If    = OK_PC
          !Config Help  = 
          !Config Units = [flag]
          CALL getin_p('use_fixed_cryoturbation_depth',use_fixed_cryoturbation_depth)
          bioturb_location(:,:) = .false.
          !
          !Config Key   = bioturbation_depth
          !Config Desc  = maximum bioturbation depth 
          !Config Def   = 2
          !Config If    = ok_pc
          !Config Help  = 
          !Config Units = m
          bioturbation_depth = 2.
          CALL getin_p('bioturbation_depth',bioturbation_depth)
          !
          !Config Key   = use_new_diffusion_scheme
          !Config Desc  = 
          !Config Def   = n
          !Config If    = ok_pc
          !Config Help  = 
          !Config Units = [flag]
          CALL getin_p('use_new_diffusion_scheme',use_new_diffusion_scheme)
          
          firstcall = .FALSE.

       ELSE
!WRITE(numout,*) 'Cryoturbate: 3 after ELSE Before Alayer TSC'               

          ! 1. calculate the total soil carbon in the active layer
          carbon_32l_old = carbon_32l
	  DOC_old = DOC
	  litter_below_old = litter_below
!!!!          deepC_s_old = deepC_s
!!!!          deepC_p_old = deepC_p
 carbon_32l_old_buffer = zero
	DOC_old_buffer = zero
litter_below_old_buffer = zero
!          deepC_s_old_buffer = zero
!          deepC_p_old_buffer = zero
!!SIMON initialise vars.    
		altC_C(:,:,:) = zero
		altC_DOC(:,:,:,:) = zero
		altC_litter_below(:,:,:) = zero
!!
		altC_C_old(:,:,:) = zero
		altC_DOC_old(:,:,:,:) = zero
		altC_litter_below_old(:,:,:) = zero		

!!       veget_mask_2d(:,:) = .TRUE.
!BELOW SIMON commented *(zf_soil_B(il)-zf_soil_B(il-1)) because carbon_32l is in g/m2 but deepC_A/s/p is in g/m3 which needed this depth conversion
!!WRITE(numout,*) 'Cryoturbate: 4 Before Alayer TSC loop'               
 	DO ip = 1, npts
!!WRITE(numout,*) 'Cryoturbate: 4 do ip(:)', ip               	
		DO iv = 1, nvm
!!WRITE(numout,*) 'Cryoturbate: 4 do iv(:)', iv 
!!WRITE(numout,*) 'cryoturb_location =', cryoturb_location(ip,iv)            	
!!WRITE(numout,*) 'bioturb_location =', bioturb_location(ip,iv)            	

			IF ( cryoturb_location(ip,iv) .OR. bioturb_location(ip,iv) )THEN
!!WRITE(numout,*) 'Cryoturbate: 4 if turb loop begins'               	
			
                   	! 1. calculate the total soil carbon in the active layer
                   	 DO il = 1, ndeep
                     	 altC_C_old(ip,:,iv) = altC_C_old(ip,:,iv) + carbon_32l(ip,:,iv,il)*(zf_soil_B(il)-zf_soil_B(il-1))
!!WRITE(numout,*) 'Cryoturbate: 4a : soil'               			 
                     	 altC_DOC_old(ip,iv,:,:) = altC_DOC_old(ip,iv,:,:) + DOC(ip,iv,il,:,:,icarbon)*(zf_soil_B(il)-zf_soil_B(il-1))
!!WRITE(numout,*) 'Cryoturbate: 4b :DOC'               
                    	 altC_litter_below_old(ip,:,iv) = altC_litter_below_old(ip,:,iv)+ litter_below(ip,:,iv,il,icarbon)*(zf_soil_B(il)-zf_soil_B(il-1))
!!WRITE(numout,*) 'Cryoturbate: 4c: litter'               
			 ENDDO
!!!!
!!WRITE(numout,*) 'Cryoturbate: 5 Before 2nd use_new_diffusion scheme tag/diffuse soil carbon'               
 !              		    IF (use_new_diffusion_scheme) THEN !This is code from an old Bertrand adaptation to diffusion 
			 			           
                   		! 2. diffuse the soil carbon
!		WRITE(numout,*) 'SIMON 3266 carbon_32l range=',maxval(carbon_32l), minval(carbon_32l)
!		WRITE(numout,*) 'SIMON 3266 mu_soil range=', mu_soil
!		WRITE(numout,*) 'SIMON 3266 beta_C range=',maxval(beta_C), minval(beta_C)
!		WRITE(numout,*) 'SIMON 3266 (1.+mu_soil*(1.-alpha_C(ip,:,iv,1))) val=',(1.+mu_soil*(1.-alpha_C(ip,:,iv,1)))				
       				carbon_32l(ip,:,iv,1) = (carbon_32l(ip,:,iv,1)+mu_soil*beta_C(ip,:,iv,1)) / (1.+mu_soil*(1.-alpha_C(ip,:,iv,1)))
!
!		WRITE(numout,*) 'SIMON 3266 DOC range=',maxval(DOC), minval(DOC)
!		WRITE(numout,*) 'SIMON 3266 beta_DOC range=',maxval(beta_DOC), minval(beta_DOC)
!		WRITE(numout,*) 'SIMON 3266 (1.+mu_soil*(1.-alpha_DOC(ip,iv,1,:,:))) val=',(1.+mu_soil*(1.-alpha_DOC(ip,iv,1,:,:)))				
        			DOC(ip,iv,1,:,:,icarbon) = (DOC(ip,iv,1,:,:,icarbon)+mu_soil*beta_DOC(ip,iv,1,:,:)) / (1.+mu_soil*(1.-alpha_DOC(ip,iv,1,:,:)))
!
!		WRITE(numout,*) 'SIMON 3266 litter_below range=',maxval(litter_below), minval(litter_below)
!		WRITE(numout,*) 'SIMON 3266 beta_DOC range=',maxval(beta_litter_below), minval(beta_litter_below)
!		WRITE(numout,*) 'SIMON 3266 (1.+mu_soil*(1.-alpha_litter_below(ip,:,iv,1))) val=',(1.+mu_soil*(1.-alpha_litter_below(ip,:,iv,1)))				
       				litter_below(ip,:,iv,1,icarbon) = (litter_below(ip,:,iv,1,icarbon)+mu_soil*beta_litter_below(ip,:,iv,1)) / (1.+mu_soil*(1.-alpha_litter_below(ip,:,iv,1)))
!!
!!
!!WRITE(numout,*) 'Cryoturbate: 6 Before deeper layer TSC loop'               
               	  	
!!!				DO il = 2, ndeep
!!!                   		carbon_32l(ip,:,iv,il) = (carbon_32l(ip,:,iv,il)+mu_soil*beta_C(ip,:,iv,il)) / (1.+mu_soil*(1.-alpha_C(ip,:,iv,il)))
!!!                   		DOC(ip,iv,il,:,:,icarbon) = (DOC(ip,iv,il,:,:,icarbon)+mu_soil*beta_DOC(ip,iv,il,:,:)) / (1.+mu_soil*(1.-alpha_DOC(ip,iv,il,:,:)))
!!!		   		litter_below(ip,:,iv,il,icarbon) = (litter_below(ip,:,iv,il,icarbon)+mu_soil*beta_litter_below(ip,:,iv,il)) / (1.+mu_soil*(1.-alpha_litter_below(ip,:,iv,il)))
!!!                   		ENDDO
!!!SIMON found bug above related to calculation of ndeep diffusion
!
                   DO il = 2, ndeep
                      carbon_32l(ip,:,iv,il) = alpha_C(ip,:,iv,il-1)*carbon_32l(ip,:,iv,il-1) + beta_C(ip,:,iv,il-1)
                      DOC(ip,iv,il,:,:,icarbon) = alpha_DOC(ip,iv,il-1,:,:)*DOC(ip,iv,il-1,:,:,icarbon) + beta_DOC(ip,iv,il-1,:,:)
                      litter_below(ip,:,iv,il,icarbon) = alpha_litter_below(ip,:,iv,il-1)*litter_below(ip,:,iv,il-1,icarbon) + beta_litter_below(ip,:,iv,il-1)
                   ENDDO
!!WRITE(numout,*) '3077carbon_32l 11l=', maxval(carbon_32l(:,:,:,11))
!!WRITE(numout,*) '3077 carbon_32l 12l=', maxval(carbon_32l(:,:,:,12))
!!WRITE(numout,*) '3077 alpha_C 11l=', maxval(alpha_C(:,:,:,11))
!!WRITE(numout,*) '3077 alpha_C 12l=', maxval(alpha_C(:,:,:,12))

!WRITE(numout,*) '3077 beta_C 11l=', maxval(beta_C(:,:,:,11))
!WRITE(numout,*) '3077 beta_C 12l=', maxval(beta_C(:,:,:,12))
!
!WRITE(numout,*) 'DOC 11l=', maxval(DOC(:,:,11,:,:,:))
!WRITE(numout,*) '3077 DOC 12l=', maxval(DOC(:,:,12,:,:,:))
!WRITE(numout,*) '3077 alpha_DOC 11l=', maxval(alpha_DOC(:,:,11,:,:))
!WRITE(numout,*) '3077 alpha_DOC 12l=', maxval(alpha_DOC(:,:,12,:,:))
!WRITE(numout,*) '3077 beta_DOC 11l=', maxval(beta_DOC(:,:,11,:,:))
!WRITE(numout,*) '3077 beta_DOC 12l=', maxval(beta_DOC(:,:,12,:,:))


                   	!!!!SIMON comments (*(zf_soil_B(il)-zf_soil_B(il-1))) as we don't need to convert to m2 by this multiplication if carbon_32l is already m2
!WRITE(numout,*) 'Cryoturbate:  7 Before reacalculating Alayer TSC'               
				! 3. recalculate the total soil carbon in the active layer
                   		DO il = 1, ndeep
                      		altC_C(ip,:,iv) = altC_C(ip,:,iv) + carbon_32l(ip,:,iv,il)*(zf_soil_B(il)-zf_soil_B(il-1))
                      		altC_DOC(ip,iv,:,:) = altC_DOC(ip,iv,:,:) + DOC(ip,iv,il,:,:,icarbon)*(zf_soil_B(il)-zf_soil_B(il-1))
                     		altC_litter_below(ip,:,iv) = altC_litter_below(ip,:,iv) + litter_below(ip,:,iv,il,icarbon)*(zf_soil_B(il)-zf_soil_B(il-1))
                  		ENDDO
!
!WRITE(numout,*) 'Cryoturbate: 8 Before subtracting top layer SC so that sum C conserved (surfC_totake_C)'               

                   		! 4. subtract the soil carbon in the top layer(s) so that the total carbon content of the active layer is conserved.             
                   		! for now remove this correction term...
                      		surfC_totake_C(ip,:,iv) = (altC_C(ip,:,iv)-altC_C_old(ip,:,iv))/(zf_soil_B(altmax_ind(ip,iv))-zf_soil_B(0))
                      		surfC_totake_DOC(ip,iv,:,:) = (altC_DOC(ip,iv,:,:)-altC_DOC_old(ip,iv,:,:))/(zf_soil_B(altmax_ind(ip,iv))-zf_soil_B(0))
                      		surfC_totake_litter_below(ip,:,iv) = (altC_litter_below(ip,:,iv)-altC_litter_below_old(ip,:,iv))/(zf_soil_B(altmax_ind(ip,iv))-zf_soil_B(0))
!WRITE(numout,*) 'Cryoturbate: 9 for soil carbon'               

   				  DO ic = 1,ncarb
                      		  carbon_32l(ip,ic,iv,1:altmax_ind(ip,iv))=carbon_32l(ip,ic,iv,1:altmax_ind(ip,iv))-surfC_totake_C(ip,ic,iv)
   				  ENDDO
 			  	
!WRITE(numout,*) 'Cryoturbate: 10 for DOC'               

				  DO ido = 1,ndoc
     				  	DO ipo = 1,npool
                      				DOC(ip,iv,1:altmax_ind(ip,iv),ido,ipo,icarbon) = DOC(ip,iv,1:altmax_ind(ip,iv),ido,ipo,icarbon) - surfC_totake_DOC(ip,iv,ido,ipo)
  					ENDDO
     				  ENDDO
!WRITE(numout,*) 'Cryoturbate: 11 For litter'               
				  DO ilt = 1,nlitt
                         		litter_below(ip,ilt,iv,1:altmax_ind(ip,iv),icarbon) = litter_below(ip,ilt,iv,1:altmax_ind(ip,iv),icarbon) - surfC_totake_litter_below(ip,ilt,iv)
  				  ENDDO
!WRITE(numout,*) 'Cryoturbate: 12 ensuring no negative values'               

                   		  ! if negative values appear, we don't subtract the delta-C from top layers
				  DO ic = 1,ncarb
                   			IF (ANY(carbon_32l(ip,ic,iv,1:altmax_ind(ip,iv)) .LT. zero) ) THEN
                      				carbon_32l(ip,ic,iv,1:altmax_ind(ip,iv))=carbon_32l(ip,ic,iv,1:altmax_ind(ip,iv))+surfC_totake_C(ip,ic,iv)
				                 	IF (altC_C(ip,ic,iv) .GT. zero) THEN
                         					carbon_32l(ip,ic,iv,:)=carbon_32l(ip,ic,iv,:)*altC_C_old(ip,ic,iv)/altC_C(ip,ic,iv)
                      					ENDIF
                   			ENDIF
				  ENDDO
				  DO ido = 1,ndoc
					DO ipo = 1,npool                   
                   				IF (ANY(DOC(ip,iv,1:altmax_ind(ip,iv),ido,ipo,icarbon) .LT. zero) ) THEN
                      					DOC(ip,iv,1:altmax_ind(ip,iv),ido,ipo,icarbon)=DOC(ip,iv,1:altmax_ind(ip,iv),ido,ipo,icarbon)+surfC_totake_DOC(ip,iv,ido,ipo)
                      					IF (altC_DOC(ip,iv,ido,ipo) .GT. zero) THEN
                         				DOC(ip,iv,:,ido,ipo,icarbon)=DOC(ip,iv,:,ido,ipo,icarbon)*altC_DOC_old(ip,iv,ido,ipo)/altC_DOC(ip,iv,ido,ipo)
                      					ENDIF
                   				ENDIF
        				ENDDO
    				  ENDDO
    
    				  DO ilt = 1,nlitt
                   			IF (ANY(litter_below(ip,ilt,iv,1:altmax_ind(ip,iv),icarbon) .LT. zero) ) THEN	
					litter_below(ip,ilt,iv,1:altmax_ind(ip,iv),icarbon)=litter_below(ip,ilt,iv,1:altmax_ind(ip,iv),icarbon)+surfC_totake_litter_below(ip,ilt,iv)
			                    IF (altC_litter_below(ip,ilt,iv) .GT. zero) THEN
                         			litter_below(ip,ilt,iv,:,icarbon)=litter_below(ip,ilt,iv,:,icarbon)*altC_litter_below_old(ip,ilt,iv)/altC_litter_below(ip,ilt,iv)
                      			    ENDIF
                   			ENDIF
				  ENDDO !nlitt   
               			! Consistency check. Potentially add to STRICT_CHECK flag
				  DO ic = 1,ncarb
                  			IF ( ANY(carbon_32l(ip,ic,iv,:) .LT. zero) ) THEN
                      				WRITE (numout,*) 'cryoturbate_doc_POC: carbon_32l <0','ip=',ip,'iv=',iv,'ic=',ic,'carbon_32l  =',carbon_32l(ip,ic,iv,:)
                      				CALL ipslerr_p (3,'cryoturbate_doc_POC','','','')                            
                   			ENDIF
				  ENDDO
!
				  DO ido = 1,ndoc
					DO ipo = 1,npool
                   				IF ( ANY(DOC(ip,iv,:,ido,ipo,icarbon) .LT. zero) ) THEN
                      				WRITE (numout,*) 'cryoturbate_doc_POC: DOC <0','ip=',ip,'iv=',iv,'ido=',ido,'ipo=',ipo,'icarbon=',icarbon,'DOC  =',DOC(ip,iv,:,ido,ipo,icarbon)
                     				CALL ipslerr_p (3,'cryoturbate_doc_POC','','','')                            
                   				ENDIF
					ENDDO
				  ENDDO   
!!!	
				  DO ilt = 1,nlitt
                   			IF ( ANY(litter_below(ip,ilt,iv,:,icarbon) .LT. zero) ) THEN
                      			WRITE (numout,*) 'cryoturbate_doc_POC: litter_below <0','ip=',ip,'ilt=',ilt,'iv=',iv,'icarbon=',icarbon,'litter_below  =',litter_below(ip,ilt,iv,:,icarbon)
                      			CALL ipslerr_p (3,'cryoturbate_doc_POC','','','')                            
                   			ENDIF
	    			  ENDDO
      				

!			ENDIF !!	
      		ENDIF   !!cryoturb_location(ip,iv)	
	 ENDDO !iv
     ENDDO !ip
ENDIF

!!WRITE(numout,*) 'Cryoturbate: 13 entering COEFFICIENT segment of cryoturbate'               

    	ELSEIF ( action .EQ. 'coefficients' ) THEN
       		IF (firstcall) THEN
          		WRITE(*,*) 'error: initilaizations have to happen before coefficients calculated. we stop.'
          		STOP
       		ENDIF
		!In the former vertical discretization scheme the first level was at 0.016 cm; now it's 
		!only 0.00048 so we set an equivalent threshold directly as a fixed depth of 1 cm,
		
		
      	 	cryoturb_location(:,:) =  ( altmax_lastyear(:,:) .LT. max_cryoturb_alt ) &
       		.AND. ( altmax_lastyear(:,:) .GE. min_cryoturb_alt ) .AND. veget_mask_2d(:,:)
!WRITE(numout,*) 'altmax_lastyear(:,:) =',(altmax_lastyear(:,:))               
!WRITE(numout,*) 'Cryoturbate: 14 before if used fixed cryoturbation depth'               
!WRITE(numout,*) 'cryoturb_location =', cryoturb_location(ip,iv)            	
       		IF (use_fixed_cryoturbation_depth) THEN
          		cryoturbation_depth(:,:) = fixed_cryoturbation_depth(:,:)
       			ELSE
          		cryoturbation_depth(:,:) = altmax_lastyear(:,:)
       		ENDIF
       		bioturb_location(:,:) = ( ( altmax_lastyear(:,:) .GE. max_cryoturb_alt ) .AND. veget_mask_2d(:,:) )
!WRITE(numout,*) 'bioturb_location =', bioturb_location(ip,iv)            	
!!!!%%%%%%%%%%% IF ( action .EQ. 'diffuse' ) STILL GOING
!WRITE(numout,*) 'Cryoturbate: 15 Before case select'               

       DO ip = 1, npts
          DO iv = 1,nvm
             IF ( cryoturb_location(ip,iv) ) THEN
                !
                IF (use_new_cryoturbation) THEN
                   SELECT CASE(cryoturbation_method)
                   CASE(1)
                      !
                      DO il = 1, ndeep ! linear dropoff to zero between alt and 2*alt
                         IF ( zi_soil(il) .LE. cryoturbation_depth(ip,iv) ) THEN
                            diff_k(ip,il,iv) = diff_k_const
                         ELSE
                            diff_k(ip,il,iv) = diff_k_const*(un-MAX(MIN((zi_soil(il)/cryoturbation_depth(ip,iv))-un,un),zero))
                         ENDIF
                      END DO
                      !
                   CASE(2)
                      !
                      DO il = 1, ndeep ! exponential dropoff with e-folding distace = alt, below the active layer
                         IF ( zi_soil(il) .LE. cryoturbation_depth(ip,iv) ) THEN
                            diff_k(ip,il,iv) = diff_k_const
                         ELSE
                            diff_k(ip,il,iv) = diff_k_const*(EXP(-MAX((zi_soil(il)/cryoturbation_depth(ip,iv)-un),zero)))
                         ENDIF
                      END DO
                      !
                   CASE(3)
                      !
                      ! exponential dropoff with e-folding distace = alt, starting at surface
                      diff_k(ip,:,iv) = diff_k_const*(EXP(-(zi_soil(:)/cryoturbation_depth(ip,iv))))
                      !
                   CASE(4)
                      !
                      DO il = 1, ndeep ! linear dropoff to zero between alt and 3*alt
                         IF ( zi_soil(il) .LE. cryoturbation_depth(ip,iv) ) THEN
                            diff_k(ip,il,iv) = diff_k_const
                         ELSE
                            diff_k(ip,il,iv) = diff_k_const*(un-MAX(MIN((zi_soil(il)-cryoturbation_depth(ip,iv))/ &
                                 (2.*cryoturbation_depth(ip,iv)),un),zero))
                         ENDIF
                         IF ( zi_soil(il) .GT. max_cryoturb_alt ) THEN
                            diff_k(ip,il,iv) = zero
                         ENDIF
!!END
                      END DO
                      ! 
                      IF (printlev>=3) WRITE(*,*) 'cryoturb method 4: ip, iv, diff_k(ip,:,iv): ', ip, iv, diff_k(ip,:,iv)
                   CASE(5)
                      !
                      DO il = 1, ndeep ! linear dropoff to zero between alt and 3m
                         IF ( zi_soil(il) .LE. cryoturbation_depth(ip,iv) ) THEN
                            diff_k(ip,il,iv) = diff_k_const
                         ELSE
                            diff_k(ip,il,iv) = diff_k_const*(un-MAX(MIN((zi_soil(il)-cryoturbation_depth(ip,iv))/ &
                                 (3.-cryoturbation_depth(ip,iv)),un),zero))
                         ENDIF
                      END DO
                      !
                IF (printlev>=3) WRITE(*,*) 'cryoturb method 5: ip, iv, diff_k(ip,:,iv): ', ip, iv, diff_k(ip,:,iv)
                   END SELECT

!!WRITE(numout,*) 'Cryoturbate: 16 Before old cryoturbation scheme'               
  
                   ELSE ! old cryoturbation scheme 
                   !
                   diff_k(ip,1:altmax_ind(ip,iv),iv) = diff_k_const
                   diff_k(ip, altmax_ind(ip,iv)+1,iv) = diff_k_const/10.
                   diff_k(ip, altmax_ind(ip,iv)+2,iv) = diff_k_const/100.
                   diff_k(ip,(altmax_ind(ip,iv)+3):ndeep,iv) = zero
                ENDIF
             
	     ELSE IF ( bioturb_location(ip,iv) ) THEN
                DO il = 1, ndeep
                   IF ( zi_soil(il) .LE. bioturbation_depth ) THEN
                      diff_k(ip,il,iv) = bio_diff_k_const
                   ELSE
                      diff_k(ip,il,iv) = zero
                   ENDIF !zi_soil
                END DO
	     ELSE
                diff_k(ip,:,iv) = zero
             END IF
	 END DO !iv
    END DO !ip
    
!!WRITE(numout,*) 'Cryoturbate: 17 Before use new diffusion scheme iffff'               
!!SIMON check this IF (/.not. )
!!  IF (use_new_diffusion_scheme) THEN
!write(numout,*) 'BOOM zf_soil_B  =', zf_soil_B  
!write(numout,*) 'BOOM dt  =', dt  
!write(numout,*) 'BOOM zi_soil  =', zi_soil  
 
       DO il = 1,ndeep-1
          WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:) )
             xc_cryoturb(:,il,:) = (zf_soil_B(il)-zf_soil_B(il-1))  / dt
             xd_cryoturb(:,il,:) = diff_k(:,il,:) / (zi_soil(il+1)-zi_soil(il))
          ENDWHERE
       ENDDO
       
       WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:)  )
          xc_cryoturb(:,ndeep,:) = (zf_soil_B(ndeep)-zf_soil_B(ndeep-1))  / dt
          !bottom
          xe_ALL(:,:) = xc_cryoturb(:,ndeep,:)+xd_cryoturb(:,ndeep-1,:)
      END WHERE    
	DO ic = 1,ncarb
       		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:)  )
          	alpha_C(:,ic,:,ndeep-1) = xd_cryoturb(:,ndeep-1,:) / xe_ALL(:,:)
      		END WHERE    
	ENDDO
!	
	DO ido = 1,ndoc
		DO ipo = 1,npool
       			WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:)  )
		        alpha_DOC(:,:,ndeep-1,ido,ipo) = xd_cryoturb(:,ndeep-1,:) / xe_ALL(:,:)
      			END WHERE    
		ENDDO
	ENDDO
!	
	DO ilt = 1,nlitt
          	WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:)  )
          	alpha_litter_below(:,ilt,:,ndeep-1) = xd_cryoturb(:,ndeep-1,:) / xe_ALL(:,:)
      		END WHERE    
	ENDDO
!
	DO ic = 1,ncarb
       		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:)  )
          	beta_C(:,ic,:,ndeep-1) = xc_cryoturb(:,ndeep,:)*carbon_32l(:,ic,:,ndeep) / xe_ALL(:,:)
      		END WHERE
	ENDDO
!
	DO ilt = 1,nlitt
       		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:)  ) 
         	beta_litter_below(:,ilt,:,ndeep-1) = xc_cryoturb(:,ndeep,:)*litter_below(:,ilt,:,ndeep,icarbon) / xe_ALL(:,:)
		END WHERE
	ENDDO
!!
	DO ido = 1,ndoc
		DO ipo = 1,npool
       			WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:)  )
          		beta_DOC(:,:,ndeep-1,ido,ipo) = xc_cryoturb(:,ndeep,:)*DOC(:,:,ndeep,ido,ipo,icarbon) / xe_ALL(:,:)
			END WHERE
		ENDDO
	ENDDO
     !other levels
       	DO il = ndeep-2,1,-1
         	DO ic = 1,ncarb
          		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:) )
             		xe_ALL(:,:) = xc_cryoturb(:,il+1,:) + (1.-alpha_C(:,ic,:,il+1))*xd_cryoturb(:,il+1,:) + xd_cryoturb(:,il,:)
             		alpha_C(:,ic,:,il) = xd_cryoturb(:,il,:) / xe_ALL(:,:)
			END WHERE
		ENDDO !ic
	
	
		DO ido = 1,ndoc
			DO ipo = 1,npool
                  		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:) )
	     			alpha_DOC(:,:,il,ido,ipo) = xd_cryoturb(:,il,:) / xe_ALL(:,:)
		  		END WHERE
			ENDDO
		ENDDO	
!
		DO ilt = 1,nlitt
	    		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:) )
             		alpha_litter_below(:,ilt,:,il) = xd_cryoturb(:,il,:) / xe_ALL(:,:)
	    		END WHERE
		ENDDO

		DO ic = 1,ncarb
           		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:) ) 
	     		beta_C(:,ic,:,il) = (xc_cryoturb(:,il+1,:)*carbon_32l(:,ic,:,il+1)+xd_cryoturb(:,il+1,:)*beta_C(:,ic,:,il+1)) / xe_ALL(:,:)
	   		END WHERE
		ENDDO
!
		DO ido = 1,ndoc
			DO ipo = 1,npool
	    			WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:) )
             			beta_DOC(:,:,il,ido,ipo) = (xc_cryoturb(:,il+1,:)*DOC(:,:,il+1,ido,ipo,icarbon)+xd_cryoturb(:,il+1,:)*beta_DOC(:,:,il+1,ido,ipo)) / xe_ALL(:,:)
	    			END WHERE
	  		ENDDO
		ENDDO
!!
		DO ilt = 1,nlitt
	    		WHERE ( cryoturb_location(:,:) .OR. bioturb_location(:,:) )
			beta_litter_below(:,ilt,:,il) = (xc_cryoturb(:,il+1,:)*litter_below(:,ilt,:,il+1,icarbon)+xd_cryoturb(:,il+1,:)*beta_litter_below(:,ilt,:,il+1)) / xe_ALL(:,:)
	  		END WHERE
		ENDDO !ilt
      	ENDDO !il
!WRITE(numout,*) 'Cryoturbate: 18 After calculation of old diffusion scheme'               
 
 
       DO ip = 1, npts
          DO iv = 1,nvm
             IF ( cryoturb_location(ip,iv) .AND. .NOT. firstcall) THEN
 
!write (numout,*) 'L3300 BOOM alpha_DOC maxval', (maxval(alpha_DOC(ip,iv,:,:,:)))   
!write (numout,*) 'L3300 BOOM beta_DOC maxval', (maxval(beta_DOC(ip,iv,:,:,:))) 
!write (numout,*) 'L3300 BOOM alpha_C maxval', (maxval(alpha_C(ip,:,iv,:)))      
!write (numout,*) 'L3300 BOOM beta_C maxval', (maxval(beta_C(ip,:,iv,:)))
!write (numout,*) 'L3300 BOOM xe_ALL ', (xe_ALL(ip,iv))
!write (numout,*) 'L3300 BOOM xc_cryoturb gt zero', (maxval(xc_cryoturb(ip,:,iv)))     
!write (numout,*) 'L3300 BOOM xd_cryoturb gt zero', (maxval(xd_cryoturb(ip,:,iv)))     
!write (numout,*) 'L3300 BOOM alpha_DOC minval', (minval(alpha_DOC(ip,iv,:,:,:)))   
!write (numout,*) 'L3300 BOOM beta_DOC minval', (minval(beta_DOC(ip,iv,:,:,:))) 
!write (numout,*) 'L3300 BOOM alpha_C minval', (minval(alpha_C(ip,:,iv,:)))      
!write (numout,*) 'L3300 BOOM beta_C minval', (minval(beta_C(ip,:,iv,:)))
!write (numout,*) 'L3300 BOOM xc_cryoturb minval', (minval(xc_cryoturb(ip,:,iv)))     
!write (numout,*) 'L3300 BOOM xd_cryoturb minval', (minval(xd_cryoturb(ip,:,iv)))     

	ENDIF
      ENDDO
   ENDDO
 
!!  ENDIF  !!new diffusion scheme?
    
 
    
    ELSE
       !
       ! do not know this action
       !
       CALL ipslerr_p(3, 'cryoturbate', 'DO NOT KNOW WHAT TO DO:', TRIM(action), '')
       !
ENDIF !!action = diffuse??
    ! keep last action in mind
    !    
    last_action = action
END  SUBROUTINE cryoturbate_doc_POC
!
!ENDIF
!
!
!  
!
 END MODULE stomate_soilcarbon
